#
# Copyright (C) 2017 Cumulus Networks, Inc.
#

#----------------------------------------------------------------------------
#
#   Chassis base class
#
#   Do not instantiate this class directly.  Use the parent
#   module's probe() function.  See __init__.py.
#
#----------------------------------------------------------------------------

class chassisBase:
    """
    This is the base class which specific chassis implementations will use. Most
    of the routines here will need to be overridden by classes which use this
    as the base class.
    """
    def __init__(self):
        pass

    def HasSlotNum(self):
	"""
	Return true if slot number is available.
	For example, slot number may be unavailable at product installation
	time when the platform driver is not loaded.
	"""
        raise NotImplementedError

    def GetSlotNum(self):
        """
        Return the slot number of the module on which this code is running.
        The lowest number has the highest priority.
        """
        raise NotImplementedError

    def GetMgmtAddr(self):
        """
        Get the IPv6 link local address of the management network for the card
        on which this code is running.
        """
        raise NotImplementedError

    def GetV4MgmtAddr(self):
        """
        Get the IPv4 link local address of the management network for the card
        on which this code is running.
        """
        raise NotImplementedError

    def GetMgmtIf(self):
        """
        Return the interface name of the management interface used for intra-
        chassis communication.
        """
        return "eth0"

    def GetNumFabIntfs(self):
        """
        Return the number of fabric interfaces on this card.
        """
        raise NotImplementedError

    def isFabricCard(self):
        """
        Return true if this is a fabric card. Line cards return False.
        """
        raise NotImplementedError

    def GetModuleName(self, slot = None):
        """
        Return the short, printable name of a slot.
        Default is the module on which this code is running.
        """
        raise NotImplementedError

    def GetMasterChassisBmcNum(self):
        """
        Retrieve the current master chassis management BMC number.
        """
        raise NotImplementedError

    def GetChassisBmc(self, number):
	"""
	Return a platform BMC object (an instanace of a subclass
	of cumulus.platform.BMC) for the indicated chassis
	management BMC.
	"""
        raise NotImplementedError

    def GetChassisName(self):
        """
        Return the name of the chassis.
        """
        raise NotImplementedError

    def GetV4Hosts(self):
        """
        Return the IPv4 host entries for all slots of the chassis.
        """
        raise NotImplementedError

    def GetV6Hosts(self):
        """
        Return the IPv6 host entries for all slots of the chassis.
        """
        raise NotImplementedError

    #
    # Private utility functions
    #

    def _readint(self, file):
        """
        Read a number from a file (probably a sysfs node).
        """
        return int(self._readstring(file), 0)

    def _readstring(self, file):
        """
        Read a string from a file (probably a sysfs node).
        Returns one line with leading and trailing whitespace
        (including new line) stripped.
        """
        with open(file) as f:
            return f.readline().strip()
