#
# Copyright (C) 2017 Cumulus Networks, Inc.
#

#----------------------------------------------------------------------------
#
#   Celestica BigStone-G chassis
#
#   Do not instantiate this class or import this module directly.
#   Use the parent module's probe() function.  See __init__.py.
#
#----------------------------------------------------------------------------

import os.path
from cumulus.chassis.base import chassisBase
import cumulus.platform

class chassisImpl(chassisBase):
    """
    Functions specific to the Celestica BigStone-G Chassis
    """

    # Constants
    _SYSFS_DEVICE       = "/sys/devices/platform/syscpld.0"
    _SYSFS_SLOT         = _SYSFS_DEVICE + "/slot_id"
    _SYSFS_SWE          = _SYSFS_DEVICE + "/swe_id"
    _SYSFS_CMM1_STATUS  = _SYSFS_DEVICE + "/cmm1_status"
    _SYSFS_CMM2_STATUS  = _SYSFS_DEVICE + "/cmm2_status"

    def __init__(self):
        chassisBase.__init__(self)
        self._slot = None
        if not os.path.isdir(self._SYSFS_DEVICE):
            return
        try:
            hw_slot = self._readint(self._SYSFS_SLOT)
            if hw_slot in (8, 9, 10, 11):
                sw_slot = hw_slot - 7
            elif hw_slot in (0, 1, 2, 3):
                hw_swe = self._readint(self._SYSFS_SWE)
                if hw_swe not in (0, 1):
                    raise RuntimeError("Invalid sysfs SWE ID (%d)" % hw_swe)
                sw_slot = hw_slot * 2 + 5 + hw_swe
            else:
                raise RuntimeError("Invalid sysfs slot ID (%d)" % hw_slot)
        except (IOError, ValueError, TypeError) as e:
            raise RuntimeError("Could not determine chassis slot number: %s" %
                               str(e))
        self._slot = sw_slot

    def HasSlotNum(self):
        return self._slot is not None

    def GetSlotNum(self):
        """
        Return the slot number of the module on which this code is running.
        Fabric cards are 1 to 4.  Line cards are 5 to 12.
        Line cards are in this order: 1-left, 1-right, 2-left, 2-right, ...
        The slot number is computed once at instance creation time,
        and not expected to change (duh).
        """
        if self._slot is None:
            raise RuntimeError("Platform driver not loaded")
        return self._slot

    def GetMgmtAddr(self):
        """
        Get the IPv6 link local management address of this chassis card
        """
        slot = self.GetSlotNum()
        if slot < 5:
            # fabric card
            return "fe80::%x:2" % (slot,)
        elif slot % 2 == 1:
            # line card left
            return "fe80::%x01:3" % ((slot - 3)/2,)
        else:
            # line card right
            return "fe80::%x02:3" % ((slot - 4)/2,)

    def GetV4MgmtAddr(self):
        """
        Get the IPv4 link local management address of this chassis card
        """
        return "169.254.251.%d" % (self.GetSlotNum(),)

    def GetMgmtIf(self):
        """
        Return the name of the management interface
        """
        return "eth0.4088"

    def GetNumFabIntfs(self):
        """
        Return the number of fabric interfaces on this card.
        """
        if self.GetSlotNum() < 5:
            return 32
        else:
            return 16

        return 0

    def isFabricCard(self):
        """
        Return true if this is a fabric card. Line cards return False.
        """
        return self.GetSlotNum() in (1, 2, 3, 4)

    def GetModuleName(self, slot = None):
        """
        Return the name of a slot (fc[1-4] or lc[1-4]0[12]).
        Default slot is this module.
        """
        if slot is None:
            slot = self.GetSlotNum()
        if slot < 5:
            return "fc%d" % slot
        else:
            return "lc%d0%d" % ((slot - 3) / 2, (slot - 5) % 2 + 1)

    def GetMasterChassisBmcNum(self):
        """
        Retrieve the current master CMM number (1 or 2).
        """
        try:
            s1 = self._readstring(self._SYSFS_CMM1_STATUS)
            if s1 == "master":
                return 1
            s2 = self._readstring(self._SYSFS_CMM2_STATUS)
            if s2 == "master":
                return 2
        except (IOError, ValueError, TypeError) as e:
            raise RuntimeError("Could not determine master CMM: %s" % str(e))
        raise RuntimeError("Neither CMM is master (status \"%s\" \"%s\")" %
                           (s1, s2))

    def GetChassisBmc(self, number):
	"""
	Return a platform BMC object for the indicated CMM.
	"""
        if number < 1 or number > 2:
            raise RuntimeError("Invalid CMM number (%d)" % number)
	bmc = cumulus.platform.BMC()
	bmc.name = "cmm%d" % number
	bmc.type = "openbmc"
	bmc.ipv6_addr = "fe80::c%d:1%%%s" % (number, self.GetMgmtIf())
	return bmc

    def GetChassisName(self):
        """
        Return the name of the chassis.
        """
        return "cel_bigstone_g"

    def GetV4Hosts(self):
        """
        Return the IPv4 host entries for all slots of the chassis.
        """

        base_ip = "169.254.251."
        hosts = ""
        index = 1
        for fc in range(1, 5):
            ip = "%s%d" % (base_ip, index)
            name = "fc%d" % (fc)
            hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)
            index += 1

        for lc in range(1, 5):
            for swe in range(1,3):
                ip = "%s%d" % (base_ip, index)
                name = "lc%d0%d" % (lc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)
                index += 1

        return hosts

    def GetV6Hosts(self):
        """
        Return the IPv6 host entries for all slots of the chassis.
        """

        base_ip = "fe80::"
        hosts = ""
        for fc in range(1, 5):
            ip = "%s%d:2" % (base_ip, fc)
            name = "fc%d-v6" % (fc)
            hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        for lc in range(1, 5):
            for swe in range(1,3):
                ip = "%s%d0%d:3" % (base_ip, lc, swe)
                name = "lc%d0%d-v6" % (lc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        # Next the BMC addresses
        for fc in range(1, 5):
            ip = "%s%d:1" % (base_ip, fc)
            name = "fc%d-bmc-v6" % (fc)
            hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        for lc in range(1, 5):
            for swe in range(1,3):
                ip = "%s%d0%d:1" % (base_ip, lc, swe)
                name = "lc%d0%d-bmc-v6" % (lc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        for cmm in range(1, 3):
            ip = "%sc%d:1" % (base_ip, cmm)
            name = "cmm%d-bmc-v6" % (cmm)
            hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        return hosts
