#
# Copyright (C) 2016, 2017 Cumulus Networks, Inc. All rights reserved
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# https://www.debian.org/legal/licenses/mit
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS5812_54T10GT(cumulus.platform.TridentTwoPlus10GPort):
    _set_led = '''
;;; AcctonAS5812_54T10GT
;;; Two bits per port: (1 0) orange, (0 1) green
set_led:
	call set_green_led
        pushst PORTSTATUS_ONE
        pack
        ret

set_green_led:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack
	ret
'''
    def __init__(self, label, serdes, phy_addrs, leds):
        cumulus.platform.TridentTwoPlusPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=("84858",),
                                                 phy_addrs=phy_addrs,
                                                 phy_layer='10GBaseT',
                                                 prim_offset=None,
                                                 leds=leds,
                                                 unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class AcctonAS5812_54TQSFPp(cumulus.platform.TridentTwoPlus40GPort):
    '''
    ;;;
    ;;; There are 5 leds per qsfp port on this platform
    ;;;   1 led to use when the port is in 40G mode
    ;;;   The other 4 are used when the port is in 4x10G mode
    ;;;   all are 1 bit per led, green - active low
    ;;;
    '''
    _set_led_40G = '''
;;; Act5812_QSFP_40G
set_led:
	call set_green_led
	ret
'''
    _set_led_4x10 = '''
;;; Act5812_QSFP_4x10
set_led:
  	call set_green_led
	inc a
  	call set_green_led
	inc a
  	call set_green_led
	inc a
  	call set_green_led

        ret
'''
    _set_led_4x10_off = '''
;;; Act5812_QSFP_4x10_off
set_led:
        pushst PORTSTATUS_ONE
        pack
        pushst PORTSTATUS_ONE
        pack
        pushst PORTSTATUS_ONE
        pack
        pushst PORTSTATUS_ONE
        pack
        ret
'''

    def __init__(self, label, serdes, leds):
        cumulus.platform.TridentTwoPlus40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_addrs=None,
                                                 phy_types=None,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode in ('4x1G', '4x10G')) and (sub != 0):
            return set_calls

        if mode in ('4x1G', '4x10G'):
            call_40G = 1
	    call_4x10G = ("Act5812_QSFP_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call_40G = ("Act5812_QSFP_40G", self._set_led_40G, led_port)
	    call_4x10G = ("Act5812_QSFP_4x10_off", self._set_led_4x10_off, led_port)

        set_calls[self.leds[1]] = call_40G
        set_calls[self.leds[2]] = call_4x10G
        set_calls[self.leds[3]] = None
        set_calls[self.leds[4]] = None
        set_calls[self.leds[5]] = None

        return set_calls


class AcctonAS5812_54TLED_1_36(cumulus.ledup.TridentTwoPlusLEDController):
    num_leds = 72

class AcctonAS5812_54TLED_37_54(cumulus.ledup.TridentTwoPlusLEDController):
    num_leds = 54
        
class AcctonAS5812_54TSwitch(cumulus.platform.BCMSwitch):
    ledup = (AcctonAS5812_54TLED_1_36, AcctonAS5812_54TLED_37_54)
    _10GT = AcctonAS5812_54T10GT
    _qsfp = AcctonAS5812_54TQSFPp

    has_phy_ucode_mdio = True
    has_phy_ucode_spi_rom = False

    ports = (
        _10GT(label="1",  serdes=(13,), phy_addrs=(0x1,), leds=(0, 68)),
        _10GT(label="2",  serdes=(12,), phy_addrs=(0x0,), leds=(0, 70)),
        _10GT(label="3",  serdes=(15,), phy_addrs=(0x3,), leds=(0, 64)),
        _10GT(label="4",  serdes=(14,), phy_addrs=(0x2,), leds=(0, 66)),
        _10GT(label="5",  serdes=(21,), phy_addrs=(0x5,), leds=(0, 60)),
        _10GT(label="6",  serdes=(20,), phy_addrs=(0x4,), leds=(0, 62)),
        _10GT(label="7",  serdes=(23,), phy_addrs=(0x7,), leds=(0, 56)),
        _10GT(label="8",  serdes=(22,), phy_addrs=(0x6,), leds=(0, 58)),
        _10GT(label="9",  serdes=(25,), phy_addrs=(0x9,), leds=(0, 52)),
        _10GT(label="10", serdes=(24,), phy_addrs=(0x8,), leds=(0, 54)),
        _10GT(label="11", serdes=(27,), phy_addrs=(0xb,), leds=(0, 48)),
        _10GT(label="12", serdes=(26,), phy_addrs=(0xa,), leds=(0, 50)),
        _10GT(label="13", serdes=(29,), phy_addrs=(0x21,), leds=(0, 44)),
        _10GT(label="14", serdes=(28,), phy_addrs=(0x20,), leds=(0, 46)),
        _10GT(label="15", serdes=(31,), phy_addrs=(0x23,), leds=(0, 40)),
        _10GT(label="16", serdes=(30,), phy_addrs=(0x22,), leds=(0, 42)),
        _10GT(label="17", serdes=(45,), phy_addrs=(0x25,), leds=(0, 36)),
        _10GT(label="18", serdes=(44,), phy_addrs=(0x24,), leds=(0, 38)),
        _10GT(label="19", serdes=(47,), phy_addrs=(0x27,), leds=(0, 32)),
        _10GT(label="20", serdes=(46,), phy_addrs=(0x26,), leds=(0, 34)),
        _10GT(label="21", serdes=(49,), phy_addrs=(0x29,), leds=(0, 28)),
        _10GT(label="22", serdes=(48,), phy_addrs=(0x28,), leds=(0, 30)),
        _10GT(label="23", serdes=(51,), phy_addrs=(0x2b,), leds=(0, 24)),
        _10GT(label="24", serdes=(50,), phy_addrs=(0x2a,), leds=(0, 26)),
        _10GT(label="25", serdes=(53,), phy_addrs=(0x41,), leds=(0, 20)),
        _10GT(label="26", serdes=(52,), phy_addrs=(0x40,), leds=(0, 22)),
        _10GT(label="27", serdes=(55,), phy_addrs=(0x43,), leds=(0, 16)),
        _10GT(label="28", serdes=(54,), phy_addrs=(0x42,), leds=(0, 18)),
        _10GT(label="29", serdes=(57,), phy_addrs=(0x45,), leds=(0, 12)),
        _10GT(label="30", serdes=(56,), phy_addrs=(0x44,), leds=(0, 14)),
        _10GT(label="31", serdes=(59,), phy_addrs=(0x47,), leds=(0, 8)),
        _10GT(label="32", serdes=(58,), phy_addrs=(0x46,), leds=(0, 10)),
        _10GT(label="33", serdes=(61,), phy_addrs=(0x49,), leds=(0, 4)),
        _10GT(label="34", serdes=(60,), phy_addrs=(0x48,), leds=(0, 6)),
        _10GT(label="35", serdes=(63,), phy_addrs=(0x4b,), leds=(0, 0)),
        _10GT(label="36", serdes=(62,), phy_addrs=(0x4a,), leds=(0, 2)),
        _10GT(label="37", serdes=(65,), phy_addrs=(0x61,), leds=(1, 50)),
        _10GT(label="38", serdes=(64,), phy_addrs=(0x60,), leds=(1, 52)),
        _10GT(label="39", serdes=(67,), phy_addrs=(0x63,), leds=(1, 46)),
        _10GT(label="40", serdes=(66,), phy_addrs=(0x62,), leds=(1, 48)),
        _10GT(label="41", serdes=(69,), phy_addrs=(0x65,), leds=(1, 42)),
        _10GT(label="42", serdes=(68,), phy_addrs=(0x64,), leds=(1, 44)),
        _10GT(label="43", serdes=(71,), phy_addrs=(0x67,), leds=(1, 38)),
        _10GT(label="44", serdes=(70,), phy_addrs=(0x66,), leds=(1, 40)),
        _10GT(label="45", serdes=(73,), phy_addrs=(0x69,), leds=(1, 34)),
        _10GT(label="46", serdes=(72,), phy_addrs=(0x68,), leds=(1, 36)),
        _10GT(label="47", serdes=(75,), phy_addrs=(0x6b,), leds=(1, 30)),
        _10GT(label="48", serdes=(74,), phy_addrs=(0x6a,), leds=(1, 32)),

        _qsfp(label="49", serdes=(96, 97, 98, 99),     leds=(1, 27, 0, 1, 2, 3)),
        _qsfp(label="50", serdes=(100, 101, 102, 103), leds=(1, 26, 4, 5, 6, 7)),
        _qsfp(label="51", serdes=(76, 77, 78, 79),     leds=(1, 28, 8, 9, 10, 11)),
        _qsfp(label="52", serdes=(104, 105, 106, 107), leds=(1, 25, 12, 13, 14, 15)),
        _qsfp(label="53", serdes=(108, 109, 110, 111), leds=(1, 24, 16, 17, 18, 19)),
        _qsfp(label="54", serdes=(80, 81, 82, 83),     leds=(1, 29, 20, 21, 22, 23)),
    )

class AcctonAS5812_54TCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/accton_as5812_54t_cpld.0'

'''
   Accton AS5812_54T LEDs
       Front Panel PSU1:
         leave under hardware control, not managed by software.
       Front Panel PSU2:
         leave under hardware control, not managed by software.
       Front Panel Diag:
         GREEN, RED, YELLOW, OFF
       Front Panel FAN Status:
         leave under hardware control, not managed by software.
       Front Panel Locator:
         YELLOW, Blinking YELLOW, OFF
       Back Panel - 5 individual FAN tray LEDs:
         leave under hardware control, not managed by software.

'''

class AcctonAS5812_54TSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_diag'

class AcctonAS5812_54TPSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'

class AcctonAS5812_54TPSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'

class AcctonAS5812_54TFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    # Depends on system fans 01 through 10
    sources = [ "Fan%02d" % (x) for x in range(1,11) ]
    led_location = 'led_fan'

class AcctonAS5812_54TLEDs():
    units = (AcctonAS5812_54TSystemLED(),
             AcctonAS5812_54TPSU1LED(),
             AcctonAS5812_54TPSU2LED(),
             AcctonAS5812_54TFanLED(),
             )

# lm-75 sensors
class AcctonAS5812_54TTemp(cumulus.sensors.Temp_Unit):
    temp_crit    = 85
    temp_max     = 72
    temp_fan_max = 59
    temp_fan_min = 49

# U30, rear middle sensor, 0x48
class AcctonAS5812_54TTemp1(AcctonAS5812_54TTemp):
    name = 'Temp1'
    description = 'Rear middle sensor'
    driver_path = '/sys/bus/i2c/devices/15-0048'

# U23, front left sensor, 0x49
class AcctonAS5812_54TTemp2(AcctonAS5812_54TTemp):
    name = 'Temp2'
    description = 'Front left sensor'
    driver_path = '/sys/bus/i2c/devices/16-0049'

# U30, front right sensor, 0x4A
class AcctonAS5812_54TTemp3(AcctonAS5812_54TTemp):
    name = 'Temp3'
    description = 'Front right sensor'
    driver_path = '/sys/bus/i2c/devices/17-004a'

# Intel CPU die sensors
AcctonAS5812_54T_core_path = '/sys/devices/platform/coretemp.0'
class AcctonAS5812_54TTempCPU(cumulus.sensors.Temp_Unit):
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS5812_54T_core_path
    temp_crit = 98
    temp_max  = 80
    temp_fan_max = 80
    temp_fan_min = 54

class AcctonAS5812_54TTemp4(AcctonAS5812_54TTempCPU):
    name = 'Temp4'
    driver_hwmon = ['temp2']

class AcctonAS5812_54TTemp5(AcctonAS5812_54TTempCPU):
    name = 'Temp5'
    driver_hwmon = ['temp3']

class AcctonAS5812_54TTemp6(AcctonAS5812_54TTempCPU):
    name = 'Temp6'
    driver_hwmon = ['temp4']

class AcctonAS5812_54TTemp7(AcctonAS5812_54TTempCPU):
    name = 'Temp7'
    driver_hwmon = ['temp5']

# DIMM temp sensors
class AcctonAS5812_54TDIMMTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_crit = 90
    temp_fan_max = 80
    temp_fan_min = 54
    temp_max = 80

    def generate_driver_path(self, i2c_addr):
        pci_0_1f_3_path = '/sys/devices/pci0000:00/0000:00:1f.3'

        if os.path.exists(pci_0_1f_3_path):
            for x in os.listdir(pci_0_1f_3_path):
                if x.startswith('i2c-'):
                    a,b = x.split('-')
                    return os.path.join(pci_0_1f_3_path, x) + '/' + b + '-00' + i2c_addr
        return None

class AcctonAS5812_54TTemp8(AcctonAS5812_54TDIMMTemp):
    name = 'Temp8'

    def __init__(self):
        self.driver_path = self.generate_driver_path('1a')
        AcctonAS5812_54TDIMMTemp.__init__(self)

class AcctonAS5812_54TTemp9(AcctonAS5812_54TDIMMTemp):
    name = 'Temp9'

    def __init__(self):
        self.driver_path = self.generate_driver_path('1b')
        AcctonAS5812_54TDIMMTemp.__init__(self)

# Trident 2 plus on chip temp sensors
class AcctonAS5812_54T_T2PlusTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 80
    temp_crit    = 95
    temp_fan_max = 70
    temp_fan_min = 55

class AcctonAS5812_54T_T2PlusTemp1(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp10'
    driver_hwmon = ['temp1']

class AcctonAS5812_54T_T2PlusTemp2(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp11'
    driver_hwmon = ['temp2']

class AcctonAS5812_54T_T2PlusTemp3(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp12'
    driver_hwmon = ['temp3']

class AcctonAS5812_54T_T2PlusTemp4(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp13'
    driver_hwmon = ['temp4']

class AcctonAS5812_54T_T2PlusTemp5(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp14'
    driver_hwmon = ['temp5']

class AcctonAS5812_54T_T2PlusTemp6(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp15'
    driver_hwmon = ['temp6']

class AcctonAS5812_54T_T2PlusTemp7(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp16'
    driver_hwmon = ['temp7']

class AcctonAS5812_54T_T2PlusTemp8(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp17'
    driver_hwmon = ['temp8']

class AcctonAS5812_54T_T2PlusTemp9(AcctonAS5812_54T_T2PlusTemp):
    name = 'Temp18'
    driver_hwmon = ['temp9']

class AcctonAS5812_54T_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    supported_psu_vendors = {'3Y POWER': '3Y',
                             'COMPUWARE': 'CPR',
                             '': 'CPR'}

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'cpr4011']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'cpr4011']
                subprocess.call(cmd)
            self.init_done = True

class AcctonAS5812_54TPSU1(AcctonAS5812_54T_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS5812_54TPSU2(AcctonAS5812_54T_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# The fan modules on the AS5812_54T contain two fans each, one called
# "FAN" (21500 RPM) and one called "FANR" (18000 RPM).  All the odd
# fans are of type FAN and the evens are of type FANR.  Create a base
# class for each type.
class AcctonAS5812_54TFan(cumulus.sensors.Fan_Unit):
    driver_path = AcctonAS5812_54TCPLD.path
    fan_min = 9600
    fan_max = 21000
    hwmon_search = False
    minpwm = 102

class AcctonAS5812_54TFanR(cumulus.sensors.Fan_Unit):
    driver_path = AcctonAS5812_54TCPLD.path
    fan_min = 8100
    fan_max = 18000
    hwmon_search = False
    minpwm = 102

class AcctonAS5812_54TFan1(AcctonAS5812_54TFan):
    name = 'Fan01'
    description = 'Fan Tray 1, Fan A'
    # driver is fan speed ...
    driver_hwmon = ['fan01']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan01']

    # The CPLD driver only exports "pwm1", which controls all the
    # fans.
    pwm_hwmon = ['pwm1']
    # Accton has specified that the default fan speed must be 40% of max.

class AcctonAS5812_54TFan2(AcctonAS5812_54TFanR):
    name = 'Fan02'
    description = 'Fan Tray 1, Fan B'
    driver_hwmon = ['fan02']
    cpld_hwmon = ['fan02']

class AcctonAS5812_54TFan3(AcctonAS5812_54TFan):
    name = 'Fan03'
    description = 'Fan Tray 2, Fan A'
    driver_hwmon = ['fan03']
    cpld_hwmon = ['fan03']

class AcctonAS5812_54TFan4(AcctonAS5812_54TFanR):
    name = 'Fan04'
    description = 'Fan Tray 2, Fan B'
    driver_hwmon = ['fan04']
    cpld_hwmon = ['fan04']

class AcctonAS5812_54TFan5(AcctonAS5812_54TFan):
    name = 'Fan05'
    description = 'Fan Tray 3, Fan A'
    driver_hwmon = ['fan05']
    cpld_hwmon = ['fan05']

class AcctonAS5812_54TFan6(AcctonAS5812_54TFanR):
    name = 'Fan06'
    description = 'Fan Tray 3, Fan B'
    driver_hwmon = ['fan06']
    cpld_hwmon = ['fan06']

class AcctonAS5812_54TFan7(AcctonAS5812_54TFan):
    name = 'Fan07'
    description = 'Fan Tray 4, Fan A'
    driver_hwmon = ['fan07']
    cpld_hwmon = ['fan07']

class AcctonAS5812_54TFan8(AcctonAS5812_54TFanR):
    name = 'Fan08'
    description = 'Fan Tray 4, Fan B'
    driver_hwmon = ['fan08']
    cpld_hwmon = ['fan08']

class AcctonAS5812_54TFan9(AcctonAS5812_54TFan):
    name = 'Fan09'
    description = 'Fan Tray 5, Fan A'
    driver_hwmon = ['fan09']
    cpld_hwmon = ['fan09']

class AcctonAS5812_54TFan10(AcctonAS5812_54TFanR):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan B'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan10']

# Compuware PSU I2C addresses
AcctonAS5812_54T_CPR_PSU1_i2c_path = '/sys/bus/i2c/devices/11-003c'
AcctonAS5812_54T_CPR_PSU2_i2c_path = '/sys/bus/i2c/devices/12-003f'

# 3Y Power PSUs I2C addresses
AcctonAS5812_54T_3Y_PSU1_i2c_path = '/sys/bus/i2c/devices/11-0058'
AcctonAS5812_54T_3Y_PSU2_i2c_path = '/sys/bus/i2c/devices/12-005b'

# PSU Temp Sensors
class AcctonAS5812_54TPSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 25
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class AcctonAS5812_54T_CPR_PSUTemp(AcctonAS5812_54TPSUTemp):
    temp_crit = 100
    temp_max = 60

class AcctonAS5812_54T_3Y_PSUTemp(AcctonAS5812_54TPSUTemp):
    temp_crit = 66
    temp_max = 55

# Compuware's PSU temp sensors
class AcctonAS5812_54T_CPR_Temp10(AcctonAS5812_54T_CPR_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS5812_54T_CPR_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS5812_54T_CPR_Temp11(AcctonAS5812_54T_CPR_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS5812_54T_CPR_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS5812_54T_CPR_Temp12(AcctonAS5812_54T_CPR_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS5812_54T_CPR_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS5812_54T_CPR_Temp13(AcctonAS5812_54T_CPR_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS5812_54T_CPR_PSU2_i2c_path
    driver_hwmon = ['temp2']

# 3Y Power's PSU temp sensors
class AcctonAS5812_54T_3Y_Temp10(AcctonAS5812_54T_3Y_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS5812_54T_3Y_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS5812_54T_3Y_Temp11(AcctonAS5812_54T_3Y_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS5812_54T_3Y_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS5812_54T_3Y_Temp12(AcctonAS5812_54T_3Y_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS5812_54T_3Y_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS5812_54T_3Y_Temp13(AcctonAS5812_54T_3Y_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS5812_54T_3Y_PSU2_i2c_path
    driver_hwmon = ['temp2']

# Compuware PSU Fans
class AcctonAS5812_54T_CPR_PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_max = 22000
    fan_min = 7800
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

# 3Y PSU Fans
class AcctonAS5812_54T_3Y_PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_max = 23800
    fan_min = 12500
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class AcctonAS5812_54T_CPR_Fan11(AcctonAS5812_54T_CPR_PSUFan):
    psu = 'PSU1'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS5812_54T_CPR_PSU1_i2c_path

class AcctonAS5812_54T_CPR_Fan12(AcctonAS5812_54T_CPR_PSUFan):
    psu = 'PSU2'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS5812_54T_CPR_PSU2_i2c_path

class AcctonAS5812_54T_3Y_Fan11(AcctonAS5812_54T_3Y_PSUFan):
    psu = 'PSU1'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS5812_54T_3Y_PSU1_i2c_path

class AcctonAS5812_54T_3Y_Fan12(AcctonAS5812_54T_3Y_PSUFan):
    psu = 'PSU2'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS5812_54T_3Y_PSU2_i2c_path

class PSUObj_Factory():
    psu_fans = ()
    psu_temp_sensors = ()

    def get_psu_objects(self, sensor_str):
        return (globals()['AcctonAS5812_54T_' + self.psu_class_obj.psu_type(self.psu_class_obj.name) + sensor_str](),)

    def get_psu_fans(self):
        if self.name == 'PSU1':
            return self.get_psu_objects('_Fan11')
        elif self.name == 'PSU2':
            return self.get_psu_objects('_Fan12')

    def get_psu_temp_sensors(self):
        if self.name == 'PSU1':
            return self.get_psu_objects('_Temp10') + \
                   self.get_psu_objects('_Temp11')
        elif self.name == 'PSU2':
            return self.get_psu_objects('_Temp12') + \
                   self.get_psu_objects('_Temp13')

    def PSUObject(self):
        return (self.psu_class_obj,)

    def __init__(self, psu_name):
        self.name = psu_name
        self.psu_class_obj = globals()['AcctonAS5812_54T' + psu_name]()
        self.psu_fans = self.get_psu_fans()
        self.psu_temp_sensors = self.get_psu_temp_sensors()

class AcctonAS5812_54TSensors():
    @property
    def units(self):
        return (AcctonAS5812_54TFan1(),
                AcctonAS5812_54TFan2(),
                AcctonAS5812_54TFan3(),
                AcctonAS5812_54TFan4(),
                AcctonAS5812_54TFan5(),
                AcctonAS5812_54TFan6(),
                AcctonAS5812_54TFan7(),
                AcctonAS5812_54TFan8(),
                AcctonAS5812_54TFan9(),
                AcctonAS5812_54TFan10(),
                AcctonAS5812_54TTemp1(),
                AcctonAS5812_54TTemp2(),
                AcctonAS5812_54TTemp3(),
                AcctonAS5812_54TTemp4(),
                AcctonAS5812_54TTemp5(),
                AcctonAS5812_54TTemp6(),
                AcctonAS5812_54TTemp7(),
                AcctonAS5812_54TTemp8(),
                AcctonAS5812_54TTemp9(),
                AcctonAS5812_54T_T2PlusTemp1(),
                AcctonAS5812_54T_T2PlusTemp2(),
                AcctonAS5812_54T_T2PlusTemp3(),
                AcctonAS5812_54T_T2PlusTemp4(),
                AcctonAS5812_54T_T2PlusTemp5(),
                AcctonAS5812_54T_T2PlusTemp6(),
                AcctonAS5812_54T_T2PlusTemp7(),
                AcctonAS5812_54T_T2PlusTemp8(),
                AcctonAS5812_54T_T2PlusTemp9(),
                ) + \
               PSUObj_Factory('PSU1').psu_fans + \
               PSUObj_Factory('PSU1').psu_temp_sensors + \
               PSUObj_Factory('PSU2').psu_fans + \
               PSUObj_Factory('PSU2').psu_temp_sensors + \
               PSUObj_Factory('PSU1').PSUObject() + \
               PSUObj_Factory('PSU2').PSUObject()

class AcctonAS5812_54TPWM1(cumulus.sensors.PWM_Unit):
    # This is only one PWM control knob for all 10 system fans, so
    # only use Fan1 here.
    name = 'System PWM'
    fans = (AcctonAS5812_54TFan1(),
            AcctonAS5812_54TFan2(),
            AcctonAS5812_54TFan3(),
            AcctonAS5812_54TFan4(),
            AcctonAS5812_54TFan5(),
            AcctonAS5812_54TFan6(),
            AcctonAS5812_54TFan7(),
            AcctonAS5812_54TFan8(),
            AcctonAS5812_54TFan9(),
            AcctonAS5812_54TFan10(),
            )
    temps = ( AcctonAS5812_54TTemp1(),
              AcctonAS5812_54TTemp2(),
              AcctonAS5812_54TTemp3(),
              AcctonAS5812_54TTemp4(),
              AcctonAS5812_54TTemp5(),
              AcctonAS5812_54TTemp6(),
              AcctonAS5812_54TTemp7(),
              AcctonAS5812_54TTemp8(),
              AcctonAS5812_54TTemp9(),
              AcctonAS5812_54T_T2PlusTemp1(),
              AcctonAS5812_54T_T2PlusTemp2(),
              AcctonAS5812_54T_T2PlusTemp3(),
              AcctonAS5812_54T_T2PlusTemp4(),
              AcctonAS5812_54T_T2PlusTemp5(),
              AcctonAS5812_54T_T2PlusTemp6(),
              AcctonAS5812_54T_T2PlusTemp7(),
              AcctonAS5812_54T_T2PlusTemp8(),
              AcctonAS5812_54T_T2PlusTemp9(),
              )

# PSU1 Fan Controller
class AcctonAS5812_54TPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = PSUObj_Factory('PSU1').psu_fans
    temps = PSUObj_Factory('PSU1').psu_temp_sensors

# PSU2 Fan Controller
class AcctonAS5812_54TPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = PSUObj_Factory('PSU2').psu_fans
    temps = PSUObj_Factory('PSU2').psu_temp_sensors

class AcctonAS5812_54TPWMs():
    units = (AcctonAS5812_54TPWM1(),
             AcctonAS5812_54TPWM2(),
             AcctonAS5812_54TPWM3(),
             )

class AcctonAS5812_54T(cumulus.platform.Platform):
    name = 'accton,as5812_54t'
    chip = cumulus.platform.TridentTwoPlus_56864_Chip()
    switch = AcctonAS5812_54TSwitch(chip)
    cpld = AcctonAS5812_54TCPLD()
    statusleds = AcctonAS5812_54TLEDs()
    pwms = AcctonAS5812_54TPWMs()
    sensors = AcctonAS5812_54TSensors()

Platform = AcctonAS5812_54T
