#
# Copyright (C) 2016,2017 Cumulus Networks, Inc. All rights reserved
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# https://www.debian.org/legal/licenses/mit
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors


class AcctonAS6712_32XQSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; AcctonAS6712_32XQSFPp
;;; This box uses 9 bits per port to set 40G and 4x10G modes.
;;; Bit0 sets 40G mode/non 40G mode.
;;; If bit0=1, we are in 40G mode. Except bit2, we ignore the other bits.
;;; Setting bit2 to 0/1 toggles the led from green to off.
;;; If bit0=0, we are in 4x10G mode. Each port is then
;;; represented by 2 bits.
;;; bit_1, bit_3, bit_5, bit_7, when 1, indicate 10G mode
;;; bit_2, bit_4, bit_6, bit_8, when 1=orange, 0= off

set_off_led:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret

set_led:
        ; bit0=1 for 40G mode
        port a
        pushst  PORTSTATUS_ONE
        pack

        pushst  PORTSTATUS_ONE
        pack
        ; set color
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ; setting other bits to zero
        call set_off_led
        call set_off_led
        call set_off_led
        ret
'''
    _set_led_4x10 = '''
;;; AcctonAS6712_32XQSFPp_4x10
;;; This box uses 9 bits per port to set 40G and 4x10G modes.
;;; Bit0 sets 40G mode/non 40G mode.
;;; If bit0=1, we are in 40G mode. Except bit2, we ignore the other bits.
;;; Setting bit2 to 0/1 toggles the led from green to off.
;;; If bit0=0, we are in 4x10G mode. Each port is then
;;; represented by 2 bits.
;;; bit_1, bit_3, bit_5, bit_7, when 1, indicate 10G mode
;;; bit_2, bit_4, bit_6, bit_8, when 1=orange, 0= off

set_color_led:
        port a

        ; LED 10G Mode
        pushst  PORTSTATUS_ONE
        pack

        ; set color
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret

set_led:
        port a

        ; Set to Non-40G Mode
        pushst  PORTSTATUS_ZERO
        pack

        call set_color_led
        inc a
        call set_color_led
        inc a
        call set_color_led
        inc a
        call set_color_led
        inc a
        ret
'''

    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None
        set_calls[led_index_list[0] + 2] = None
        set_calls[led_index_list[0] + 3] = None
        set_calls[led_index_list[0] + 4] = None
        set_calls[led_index_list[0] + 5] = None
        set_calls[led_index_list[0] + 6] = None
        set_calls[led_index_list[0] + 7] = None
        set_calls[led_index_list[0] + 8] = None

        return set_calls

class AcctonAS6712_32XLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 255
    default_pad = 0

class AcctonAS6712_32XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS6712_32XLED
    _qsfp = AcctonAS6712_32XQSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp(label="1", serdes=(  0,   1,   2,   3), leds=(0, 111)),
        _qsfp(label="2", serdes=(  4,   5,   6,   7), leds=(0, 120)),
        _qsfp(label="3", serdes=(  8,   9,  10,  11), leds=(0, 129)),
        _qsfp(label="4", serdes=( 12,  13,  14,  15), leds=(0, 138)),
        _qsfp(label="5", serdes=( 16,  17,  18,  19), leds=(0, 147)),
        _qsfp(label="6", serdes=( 20,  21,  22,  23), leds=(0, 156)),
        _qsfp(label="7", serdes=( 24,  25,  26,  27), leds=(0, 165)),
        _qsfp(label="8", serdes=( 28,  29,  30,  31), leds=(0, 174)),
        _qsfp(label="9", serdes=( 32,  33,  34,  35), leds=(0, 183)),
        _qsfp(label="10", serdes=( 36,  37,  38,  39), leds=(0, 192)),
        _qsfp(label="11", serdes=( 40,  41,  42,  43), leds=(0, 201)),
        _qsfp(label="12", serdes=( 44,  45,  46,  47), leds=(0, 210)),
        _qsfp(label="13", serdes=( 48,  49,  50,  51), leds=(0, 219)),
        _qsfp(label="14", serdes=( 52,  53,  54,  55), leds=(0, 228)),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 237)),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 246)),
        _qsfp(label="17", serdes=( 64,  65,  66,  67), leds=(1, 111)),
        _qsfp(label="18", serdes=( 68,  69,  70,  71), leds=(1, 120)),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 129)),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 138)),
        _qsfp(label="21", serdes=( 80,  81,  82,  83), leds=(1, 147)),
        _qsfp(label="22", serdes=( 84,  85,  86,  87), leds=(1, 156)),
        _qsfp(label="23", serdes=( 88,  89,  90,  91), leds=(1, 165)),
        _qsfp(label="24", serdes=( 92,  93,  94,  95), leds=(1, 174)),
        _qsfp(label="25", serdes=( 96,  97,  98,  99), leds=(1, 183)),
        _qsfp(label="26", serdes=(100, 101, 102, 103), leds=(1, 192)),
        _qsfp(label="27", serdes=(104, 105, 106, 107), leds=(1, 201)),
        _qsfp(label="28", serdes=(108, 109, 110, 111), leds=(1, 210)),
        _qsfp(label="29", serdes=(112, 113, 114, 115), leds=(1, 219)),
        _qsfp(label="30", serdes=(116, 117, 118, 119), leds=(1, 228)),
        _qsfp(label="31", serdes=(120, 121, 122, 123), leds=(1, 237)),
        _qsfp(label="32", serdes=(124, 125, 126, 127), leds=(1, 246)),
    )

class AcctonAS6712_32XCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/accton_as6712_32x_cpld.0'

'''
   Accton AS6712_32X LEDs
       Front Panel PSU1:
         leave under hardware control, not managed by software.
       Front Panel PSU2:
         leave under hardware control, not managed by software.
       Front Panel Diag:
         GREEN, RED, YELLOW, OFF
       Front Panel FAN Status:
         leave under hardware control, not managed by software.
       Front Panel Locator:
         YELLOW, Blinking YELLOW, OFF
       Back Panel - 5 individual FAN tray LEDs:
         leave under hardware control, not managed by software.

'''

class AcctonAS6712_32XSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'Diag'
    led_location = 'led_diag'

class AcctonAS6712_32XPSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'

class AcctonAS6712_32XPSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'

class AcctonAS6712_32XFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    # Depends on system fans 01 through 10
    sources = [ "Fan%02d" % (x) for x in range(1,11) ]
    led_location = 'led_fan'

class AcctonAS6712_32XLEDs():
    units = (AcctonAS6712_32XSystemLED(),
             AcctonAS6712_32XPSU1LED(),
             AcctonAS6712_32XPSU2LED(),
             AcctonAS6712_32XFanLED(),
             )

# Compuware PSU I2C addresses
AcctonAS6712_32X_CPR_PSU1_i2c_path = '/sys/bus/i2c/devices/11-003c'
AcctonAS6712_32X_CPR_PSU2_i2c_path = '/sys/bus/i2c/devices/12-003f'

# 3Y Power PSUs I2C addresses
AcctonAS6712_32X_3Y_PSU1_i2c_path = '/sys/bus/i2c/devices/11-0058'
AcctonAS6712_32X_3Y_PSU2_i2c_path = '/sys/bus/i2c/devices/12-005b'

# Compuware PSU Fan Sensors
class AcctonAS6712_32X_CPR_PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_max = 22000
    fan_min = 8000
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

# 3Y PSU Fan Sensors
class AcctonAS6712_32X_3Y_PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_max = 23800
    fan_min = 12500
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

# PSU Temp Sensors
class AcctonAS6712_32XPSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 25
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class AcctonAS6712_32XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Front Middle Outlet sensor near MAC'
    driver_path = '/sys/bus/i2c/devices/14-0048'
    # Accton has specified that fan speed should be at 40% till 46 C
    # and then increase to 100% at 53 C
    temp_crit = 85
    temp_max = 53
    temp_fan_min = 46

class AcctonAS6712_32XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Front Right Outlet sensor'
    driver_path = '/sys/bus/i2c/devices/15-0049'
    # Accton has specified that fan speed should be at 40% till 49.5 C
    # and then increase to 100% at 57.7
    temp_crit = 85
    temp_max = 53
    temp_fan_min = 46

class AcctonAS6712_32XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Front Left Outlet Sensor'
    driver_path = '/sys/bus/i2c/devices/16-004a'
    temp_crit = 85
    temp_max = 53
    temp_fan_min = 46

class AcctonAS6712_32XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Rear Middle Outlet Sensor'
    driver_path = '/sys/bus/i2c/devices/17-004b'
    temp_crit = 85
    temp_max = 53
    temp_fan_min = 46

AcctonAS6712_32X_core_path = '/sys/devices/platform/coretemp.0'
class AcctonAS6712_32XTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS6712_32X_core_path
    driver_hwmon = ['temp2']
    temp_crit = 98

class AcctonAS6712_32XTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS6712_32X_core_path
    driver_hwmon = ['temp3']
    temp_fan_min = 50
    temp_crit = 98

class AcctonAS6712_32XTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS6712_32X_core_path
    driver_hwmon = ['temp4']
    temp_fan_min = 50
    temp_crit = 98

class AcctonAS6712_32XTemp8(cumulus.sensors.Temp_Unit):
    name = 'Temp8'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS6712_32X_core_path
    driver_hwmon = ['temp5']
    temp_fan_min = 50
    temp_crit = 98

class AcctonAS6712_32XDIMMTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_crit = 95
    temp_fan_min = 50
    temp_max = 90

    def generate_driver_path(self, i2c_addr):
        pci_0_1f_3_path = '/sys/devices/pci0000:00/0000:00:1f.3'

        if os.path.exists(pci_0_1f_3_path):
            for x in os.listdir(pci_0_1f_3_path):
                if x.startswith('i2c-'):
                    a,b = x.split('-')
                    return os.path.join(pci_0_1f_3_path, x) + '/' + b + '-00' + i2c_addr
        return None

class AcctonAS6712_32XTemp9(AcctonAS6712_32XDIMMTemp):
    name = 'Temp9'

    def __init__(self):
        self.driver_path = self.generate_driver_path('1a')
        AcctonAS6712_32XDIMMTemp.__init__(self)

class AcctonAS6712_32XTemp10(AcctonAS6712_32XDIMMTemp):
    name = 'Temp10'

    def __init__(self):
        self.driver_path = self.generate_driver_path('1b')
        AcctonAS6712_32XDIMMTemp.__init__(self)

class AcctonAS6712T2Temp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_fan_min = 65
    temp_fan_max = 70
    temp_max     = 85
    temp_crit    = 95

class AcctonAS6712T2Temp1(AcctonAS6712T2Temp):
    name = 'Temp11'
    driver_hwmon = ['temp1']

class AcctonAS6712T2Temp2(AcctonAS6712T2Temp):
    name = 'Temp12'
    driver_hwmon = ['temp2']

class AcctonAS6712T2Temp3(AcctonAS6712T2Temp):
    name = 'Temp13'
    driver_hwmon = ['temp3']

class AcctonAS6712T2Temp4(AcctonAS6712T2Temp):
    name = 'Temp14'
    driver_hwmon = ['temp4']

class AcctonAS6712T2Temp5(AcctonAS6712T2Temp):
    name = 'Temp15'
    driver_hwmon = ['temp5']

class AcctonAS6712T2Temp6(AcctonAS6712T2Temp):
    name = 'Temp16'
    driver_hwmon = ['temp6']

class AcctonAS6712T2Temp7(AcctonAS6712T2Temp):
    name = 'Temp17'
    driver_hwmon = ['temp7']

class AcctonAS6712T2Temp8(AcctonAS6712T2Temp):
    name = 'Temp18'
    driver_hwmon = ['temp8']

class AcctonAS6712T2Temp9(AcctonAS6712T2Temp):
    name = 'Temp19'
    driver_hwmon = ['temp9']

class AcctonAS6712_32X_CPR_PSUTemp(AcctonAS6712_32XPSUTemp):
    temp_crit = 60
    temp_max = 50

class AcctonAS6712_32X_3Y_PSUTemp(AcctonAS6712_32XPSUTemp):
    temp_crit = 66
    temp_max = 55

class AcctonAS6712_32X_CPR_Temp11(AcctonAS6712_32X_CPR_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS6712_32X_CPR_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS6712_32X_CPR_Temp12(AcctonAS6712_32X_CPR_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS6712_32X_CPR_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS6712_32X_CPR_Temp13(AcctonAS6712_32X_CPR_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS6712_32X_CPR_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS6712_32X_CPR_Temp14(AcctonAS6712_32X_CPR_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS6712_32X_CPR_PSU2_i2c_path
    driver_hwmon = ['temp2']

# 3Y Power's PSU temp sensors
class AcctonAS6712_32X_3Y_Temp11(AcctonAS6712_32X_3Y_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS6712_32X_3Y_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS6712_32X_3Y_Temp12(AcctonAS6712_32X_3Y_PSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS6712_32X_3Y_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS6712_32X_3Y_Temp13(AcctonAS6712_32X_3Y_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS6712_32X_3Y_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS6712_32X_3Y_Temp14(AcctonAS6712_32X_3Y_PSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS6712_32X_3Y_PSU2_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS6712_32X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    supported_psu_vendors = {'3Y POWER': '3Y',
                             'COMPUWARE': 'CPR',
                             '': 'CPR'}

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'cpr4011']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'cpr4011']
                subprocess.call(cmd)
            self.init_done = True

class AcctonAS6712_32XPSU1(AcctonAS6712_32X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS6712_32XPSU2(AcctonAS6712_32X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# The fan modules on the AS6712_32X contain two fans each, one called
# "FAN" (21500 RPM) and one called "FANR" (18000 RPM).  All the odd
# fans are of type FAN and the evens are of type FANR.  Create a base
# class for each type.
class AcctonAS6712_32XFan(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    driver_path = AcctonAS6712_32XCPLD.path
    fan_min = 9000
    fan_max = 20000
    # Accton has specified that the default fan speed must be 40% of max.
    minpwm = 102

class AcctonAS6712_32XFanR(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    driver_path = AcctonAS6712_32XCPLD.path
    fan_min = 8000
    fan_max = 18000
    # Accton has specified that the default fan speed must be 40% of max.
    minpwm = 102

class AcctonAS6712_32XFan1(AcctonAS6712_32XFan):
    name = 'Fan01'
    description = 'Fan Tray 1, Fan A'
    # driver is fan speed ...
    driver_hwmon = ['fan09']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan09']

    # The CPLD driver only exports "pwm1", which controls all the
    # fans.
    pwm_hwmon = ['pwm1']

class AcctonAS6712_32XFan2(AcctonAS6712_32XFanR):
    name = 'Fan02'
    description = 'Fan Tray 1, Fan B'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan10']

class AcctonAS6712_32XFan3(AcctonAS6712_32XFan):
    name = 'Fan03'
    description = 'Fan Tray 2, Fan A'
    driver_hwmon = ['fan07']
    cpld_hwmon = ['fan07']

class AcctonAS6712_32XFan4(AcctonAS6712_32XFanR):
    name = 'Fan04'
    description = 'Fan Tray 2, Fan B'
    driver_hwmon = ['fan08']
    cpld_hwmon = ['fan08']

class AcctonAS6712_32XFan5(AcctonAS6712_32XFan):
    name = 'Fan05'
    description = 'Fan Tray 3, Fan A'
    driver_hwmon = ['fan05']
    cpld_hwmon = ['fan05']

class AcctonAS6712_32XFan6(AcctonAS6712_32XFanR):
    name = 'Fan06'
    description = 'Fan Tray 3, Fan B'
    driver_hwmon = ['fan06']
    cpld_hwmon = ['fan06']

class AcctonAS6712_32XFan7(AcctonAS6712_32XFan):
    name = 'Fan07'
    description = 'Fan Tray 4, Fan A'
    driver_hwmon = ['fan03']
    cpld_hwmon = ['fan03']

class AcctonAS6712_32XFan8(AcctonAS6712_32XFanR):
    name = 'Fan08'
    description = 'Fan Tray 4, Fan B'
    driver_hwmon = ['fan04']
    cpld_hwmon = ['fan04']

class AcctonAS6712_32XFan9(AcctonAS6712_32XFan):
    name = 'Fan09'
    description = 'Fan Tray 5, Fan A'
    driver_hwmon = ['fan01']
    cpld_hwmon = ['fan01']

class AcctonAS6712_32XFan10(AcctonAS6712_32XFanR):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan B'
    driver_hwmon = ['fan02']
    cpld_hwmon = ['fan02']

# Compuware PSU fans
class AcctonAS6712_32X_CPR_Fan11(AcctonAS6712_32X_CPR_PSUFan):
    psu = 'PSU1'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS6712_32X_CPR_PSU1_i2c_path

class AcctonAS6712_32X_CPR_Fan12(AcctonAS6712_32X_CPR_PSUFan):
    psu = 'PSU2'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS6712_32X_CPR_PSU2_i2c_path

# 3Y PSU fans
class AcctonAS6712_32X_3Y_Fan11(AcctonAS6712_32X_3Y_PSUFan):
    psu = 'PSU1'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS6712_32X_3Y_PSU1_i2c_path

class AcctonAS6712_32X_3Y_Fan12(AcctonAS6712_32X_3Y_PSUFan):
    psu = 'PSU2'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS6712_32X_3Y_PSU2_i2c_path

class PSUObj_Factory():
    psu_fans = ()
    psu_temp_sensors = ()

    def get_psu_objects(self, sensor_str):
        return (globals()['AcctonAS6712_32X_' + self.psu_class_obj.psu_type(self.psu_class_obj.name) + sensor_str](),)

    def get_psu_fans(self):
        if self.name == 'PSU1':
            return self.get_psu_objects('_Fan11')
        elif self.name == 'PSU2':
            return self.get_psu_objects('_Fan12')

    def get_psu_temp_sensors(self):
        if self.name == 'PSU1':
            return self.get_psu_objects('_Temp11') + \
                   self.get_psu_objects('_Temp12')
        elif self.name == 'PSU2':
            return self.get_psu_objects('_Temp13') + \
                   self.get_psu_objects('_Temp14')

    def PSUObject(self):
        return (self.psu_class_obj,)

    def __init__(self, psu_name):
        self.name = psu_name
        self.psu_class_obj = globals()['AcctonAS6712_32X' + psu_name]()
        self.psu_fans = self.get_psu_fans()
        self.psu_temp_sensors = self.get_psu_temp_sensors()

class AcctonAS6712_32XSensors():
    @property
    def units(self):
        return (AcctonAS6712_32XFan1(),
                AcctonAS6712_32XFan2(),
                AcctonAS6712_32XFan3(),
                AcctonAS6712_32XFan4(),
                AcctonAS6712_32XFan5(),
                AcctonAS6712_32XFan6(),
                AcctonAS6712_32XFan7(),
                AcctonAS6712_32XFan8(),
                AcctonAS6712_32XFan9(),
                AcctonAS6712_32XFan10(),
                AcctonAS6712_32XTemp1(),
                AcctonAS6712_32XTemp2(),
                AcctonAS6712_32XTemp3(),
                AcctonAS6712_32XTemp4(),
                AcctonAS6712_32XTemp5(),
                AcctonAS6712_32XTemp6(),
                AcctonAS6712_32XTemp7(),
                AcctonAS6712_32XTemp8(),
                AcctonAS6712_32XTemp9(),
                AcctonAS6712_32XTemp10(),
                AcctonAS6712T2Temp1(),
                AcctonAS6712T2Temp2(),
                AcctonAS6712T2Temp3(),
                AcctonAS6712T2Temp4(),
                AcctonAS6712T2Temp5(),
                AcctonAS6712T2Temp6(),
                AcctonAS6712T2Temp7(),
                AcctonAS6712T2Temp8(),
                AcctonAS6712T2Temp9(),
                ) + \
                PSUObj_Factory('PSU1').psu_fans + \
                PSUObj_Factory('PSU1').psu_temp_sensors + \
                PSUObj_Factory('PSU2').psu_fans + \
                PSUObj_Factory('PSU2').psu_temp_sensors + \
                PSUObj_Factory('PSU1').PSUObject() + \
                PSUObj_Factory('PSU2').PSUObject()

class AcctonAS6712_32XPWM1(cumulus.sensors.PWM_Unit):
    # This is only one PWM control knob for all 10 system fans, so
    # only use Fan1 here.
    name = 'System PWM'
    fans = (AcctonAS6712_32XFan1(),
            AcctonAS6712_32XFan2(),
            AcctonAS6712_32XFan3(),
            AcctonAS6712_32XFan4(),
            AcctonAS6712_32XFan5(),
            AcctonAS6712_32XFan6(),
            AcctonAS6712_32XFan7(),
            AcctonAS6712_32XFan8(),
            AcctonAS6712_32XFan9(),
            AcctonAS6712_32XFan10(),
            )
    temps = (AcctonAS6712_32XTemp1(),
             AcctonAS6712_32XTemp2(),
             AcctonAS6712_32XTemp3(),
             AcctonAS6712_32XTemp4(),
             AcctonAS6712_32XTemp5(),
             AcctonAS6712_32XTemp6(),
             AcctonAS6712_32XTemp7(),
             AcctonAS6712_32XTemp8(),
             AcctonAS6712_32XTemp9(),
             AcctonAS6712_32XTemp10(),
             AcctonAS6712T2Temp1(),
             AcctonAS6712T2Temp2(),
             AcctonAS6712T2Temp3(),
             AcctonAS6712T2Temp4(),
             AcctonAS6712T2Temp5(),
             AcctonAS6712T2Temp6(),
             AcctonAS6712T2Temp7(),
             AcctonAS6712T2Temp8(),
             AcctonAS6712T2Temp9(),
             )

# PSU1 Fan Controller
class AcctonAS6712_32XPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = PSUObj_Factory('PSU1').psu_fans
    temps = PSUObj_Factory('PSU1').psu_temp_sensors

# PSU2 Fan Controller
class AcctonAS6712_32XPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = PSUObj_Factory('PSU2').psu_fans
    temps = PSUObj_Factory('PSU2').psu_temp_sensors

class AcctonAS6712_32XPWMs():
    units = (AcctonAS6712_32XPWM1(),
             AcctonAS6712_32XPWM2(),
             AcctonAS6712_32XPWM3(),
             )

class AcctonAS6712_32X(cumulus.platform.Platform):
    name = 'accton,as6712_32x'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = AcctonAS6712_32XSwitch(chip)
    cpld = AcctonAS6712_32XCPLD()
    statusleds = AcctonAS6712_32XLEDs()
    pwms = AcctonAS6712_32XPWMs()
    sensors = AcctonAS6712_32XSensors()

Platform = AcctonAS6712_32X
