#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.chassis

class CelBigStoneG100QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
    QSFP28 ports have 12 LED bits per port.
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_port_led:
        port    a                   ; The port upon which we are operating

        push    1                   ; First bit (always on)
        pack

        pushst  PORTSTATUS_RX       ; Any RX or TX activity recently?
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)     ; If we are blinking, turn it off
        tand
        tinv
        pushst  PORTSTATUS_LINKEN   ; Is there any link on this port?
        tand
        tinv                        ; Invert for correct color.
        pack                        ; Green = 101, Off = 111

        push    1                   ; Last bit, (always on)
        pack

        ret

set_led_off:
        push    1
        pack
        push    1
        pack
        push    1
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the first 3 bits of the LED chain set the LED color.
;;;
set_led:
        call    set_port_led        ; Set the first LED in the chain
        call    set_led_off         ; The rest are ignored
        call    set_led_off
        call    set_led_off
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, the first and third 3 bits of the LED chain set the LED color.
;;;
set_led:
        call    set_port_led        ; Set the first LED in the chain
        call    set_led_off         ; These three bits are ignored
        add     a, 2
        call    set_port_led        ; Set the third LED in the chain
        call    set_led_off         ; These three bits are ignored
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the all 3-bit LED values in the LED chain must be set.
;;; indicate link/activity.
;;;
set_led:
        ld      b, 4
set_next_led:
        call    set_port_led        ; Set the 3-bit port LED
        inc     a
        dec     b
        jnz     set_next_led
        ret
'''

    def __init__(self, label, serdes, leds, tx_polarity_flip, 
                 rx_polarity_flip, tx_lane_map, rx_lane_map):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=leds,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        for led in led_index_list:
            set_calls[led] = call
            for x in range(1,12):
                set_calls[led + x] = None

        return set_calls

class CelBigStoneG100LED0(cumulus.ledup.TomahawkLEDController):
    led_functions = (CelBigStoneG100QSFP28._set_led_common)
    num_leds = 144

class CelBigStoneG100LED1(cumulus.ledup.TomahawkLEDController):
    led_functions = (CelBigStoneG100QSFP28._set_led_common)
    num_leds = 48

# sensors: there are lots of them

class CelBigStoneG100Temp(cumulus.sensors.Temp_Unit):
    # these numbers are arbitrary
    temp_max = 80
    temp_crit = 85
    hwmon_search = False
    no_shutdown = True

class CelBigStoneG100CoreTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 90
    temp_crit = 95
    no_shutdown = True

vhwmon_path = '/sys/class/misc/vhwmon/'

# on-board temps

class CelBigStoneG100Temp1(CelBigStoneG100CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class CelBigStoneG100Temp2(CelBigStoneG100CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class CelBigStoneG100Temp3(CelBigStoneG100CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class CelBigStoneG100Temp4(CelBigStoneG100CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# fc1 temps

class CelBigStoneG100TempF11(CelBigStoneG100CoreTemp):
    name = 'TempF11'
    description = 'fc1: Core 0'
    driver_path = vhwmon_path + 'fc1:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempF12(CelBigStoneG100CoreTemp):
    name = 'TempF12'
    description = 'fc1: Core 1'
    driver_path = vhwmon_path + 'fc1:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempF13(CelBigStoneG100CoreTemp):
    name = 'TempF13'
    description = 'fc1: Core 2'
    driver_path = vhwmon_path + 'fc1:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempF14(CelBigStoneG100CoreTemp):
    name = 'TempF14'
    description = 'fc1: Core 3'
    driver_path = vhwmon_path + 'fc1:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempF1B1(CelBigStoneG100Temp):
    name = 'TempF1B1'
    description = 'fc1-bmc: SCM-FC1 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc1_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempF1B2(CelBigStoneG100Temp):
    name = 'TempF1B2'
    description = 'fc1-bmc: FC1 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc1_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# fc2 temps

class CelBigStoneG100TempF21(CelBigStoneG100CoreTemp):
    name = 'TempF21'
    description = 'fc2: Core 0'
    driver_path = vhwmon_path + 'fc2:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempF22(CelBigStoneG100CoreTemp):
    name = 'TempF22'
    description = 'fc2: Core 1'
    driver_path = vhwmon_path + 'fc2:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempF23(CelBigStoneG100CoreTemp):
    name = 'TempF23'
    description = 'fc2: Core 2'
    driver_path = vhwmon_path + 'fc2:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempF24(CelBigStoneG100CoreTemp):
    name = 'TempF24'
    description = 'fc2: Core 3'
    driver_path = vhwmon_path + 'fc2:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempF2B1(CelBigStoneG100Temp):
    name = 'TempF2B1'
    description = 'fc2-bmc: SCM-FC2 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc2_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempF2B2(CelBigStoneG100Temp):
    name = 'TempF2B2'
    description = 'fc2-bmc: FC2 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc2_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# fc3 temps

class CelBigStoneG100TempF31(CelBigStoneG100CoreTemp):
    name = 'TempF31'
    description = 'fc3: Core 0'
    driver_path = vhwmon_path + 'fc3:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempF32(CelBigStoneG100CoreTemp):
    name = 'TempF32'
    description = 'fc3: Core 1'
    driver_path = vhwmon_path + 'fc3:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempF33(CelBigStoneG100CoreTemp):
    name = 'TempF33'
    description = 'fc3: Core 2'
    driver_path = vhwmon_path + 'fc3:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempF34(CelBigStoneG100CoreTemp):
    name = 'TempF34'
    description = 'fc3: Core 3'
    driver_path = vhwmon_path + 'fc3:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempF3B1(CelBigStoneG100Temp):
    name = 'TempF3B1'
    description = 'fc3-bmc: SCM-FC3 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc3_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempF3B2(CelBigStoneG100Temp):
    name = 'TempF3B2'
    description = 'fc3-bmc: FC3 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc3_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# fc4 temps

class CelBigStoneG100TempF41(CelBigStoneG100CoreTemp):
    name = 'TempF41'
    description = 'fc4: Core 0'
    driver_path = vhwmon_path + 'fc4:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempF42(CelBigStoneG100CoreTemp):
    name = 'TempF42'
    description = 'fc4: Core 1'
    driver_path = vhwmon_path + 'fc4:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempF43(CelBigStoneG100CoreTemp):
    name = 'TempF43'
    description = 'fc4: Core 2'
    driver_path = vhwmon_path + 'fc4:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempF44(CelBigStoneG100CoreTemp):
    name = 'TempF44'
    description = 'fc4: Core 3'
    driver_path = vhwmon_path + 'fc4:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempF4B1(CelBigStoneG100Temp):
    name = 'TempF4B1'
    description = 'fc4-bmc: SCM-FC4 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc4_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempF4B2(CelBigStoneG100Temp):
    name = 'TempF4B2'
    description = 'fc4-bmc: FC4 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'fc4_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# lc101 temps

class CelBigStoneG100TempL111(CelBigStoneG100CoreTemp):
    name = 'TempL111'
    description = 'lc101: Core 0'
    driver_path = vhwmon_path + 'lc101:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL112(CelBigStoneG100CoreTemp):
    name = 'TempL112'
    description = 'lc101: Core 1'
    driver_path = vhwmon_path + 'lc101:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL113(CelBigStoneG100CoreTemp):
    name = 'TempL113'
    description = 'lc101: Core 2'
    driver_path = vhwmon_path + 'lc101:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL114(CelBigStoneG100CoreTemp):
    name = 'TempL114'
    description = 'lc101: Core 3'
    driver_path = vhwmon_path + 'lc101:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempL11B1(CelBigStoneG100Temp):
    name = 'TempL11B1'
    description = 'lc101-bmc: SCM-LC1 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc101_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempL11B2(CelBigStoneG100Temp):
    name = 'TempL11B2'
    description = 'lc101-bmc: LC1 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc101_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# lc102 temps

class CelBigStoneG100TempL121(CelBigStoneG100CoreTemp):
    name = 'TempL121'
    description = 'lc102: Core 0'
    driver_path = vhwmon_path + 'lc102:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL122(CelBigStoneG100CoreTemp):
    name = 'TempL122'
    description = 'lc102: Core 1'
    driver_path = vhwmon_path + 'lc102:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL123(CelBigStoneG100CoreTemp):
    name = 'TempL123'
    description = 'lc102: Core 2'
    driver_path = vhwmon_path + 'lc102:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL124(CelBigStoneG100CoreTemp):
    name = 'TempL124'
    description = 'lc102: Core 3'
    driver_path = vhwmon_path + 'lc102:coretemp_isa_0000'
    driver_hwmon = ['temp5']

# lc201 temps

class CelBigStoneG100TempL211(CelBigStoneG100CoreTemp):
    name = 'TempL211'
    description = 'lc201: Core 0'
    driver_path = vhwmon_path + 'lc201:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL212(CelBigStoneG100CoreTemp):
    name = 'TempL212'
    description = 'lc201: Core 1'
    driver_path = vhwmon_path + 'lc201:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL213(CelBigStoneG100CoreTemp):
    name = 'TempL213'
    description = 'lc201: Core 2'
    driver_path = vhwmon_path + 'lc201:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL214(CelBigStoneG100CoreTemp):
    name = 'TempL214'
    description = 'lc201: Core 3'
    driver_path = vhwmon_path + 'lc201:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempL21B1(CelBigStoneG100Temp):
    name = 'TempL21B1'
    description = 'lc201-bmc: SCM-LC2 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc201_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempL21B2(CelBigStoneG100Temp):
    name = 'TempL21B2'
    description = 'lc201-bmc: LC2 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc201_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# lc202 temps

class CelBigStoneG100TempL221(CelBigStoneG100CoreTemp):
    name = 'TempL221'
    description = 'lc202: Core 0'
    driver_path = vhwmon_path + 'lc202:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL222(CelBigStoneG100CoreTemp):
    name = 'TempL222'
    description = 'lc202: Core 1'
    driver_path = vhwmon_path + 'lc202:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL223(CelBigStoneG100CoreTemp):
    name = 'TempL223'
    description = 'lc202: Core 2'
    driver_path = vhwmon_path + 'lc202:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL224(CelBigStoneG100CoreTemp):
    name = 'TempL224'
    description = 'lc202: Core 3'
    driver_path = vhwmon_path + 'lc202:coretemp_isa_0000'
    driver_hwmon = ['temp5']

# lc301 temps

class CelBigStoneG100TempL311(CelBigStoneG100CoreTemp):
    name = 'TempL311'
    description = 'lc301: Core 0'
    driver_path = vhwmon_path + 'lc301:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL312(CelBigStoneG100CoreTemp):
    name = 'TempL312'
    description = 'lc301: Core 1'
    driver_path = vhwmon_path + 'lc301:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL313(CelBigStoneG100CoreTemp):
    name = 'TempL313'
    description = 'lc301: Core 2'
    driver_path = vhwmon_path + 'lc301:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL314(CelBigStoneG100CoreTemp):
    name = 'TempL314'
    description = 'lc301: Core 3'
    driver_path = vhwmon_path + 'lc301:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempL31B1(CelBigStoneG100Temp):
    name = 'TempL31B1'
    description = 'lc301-bmc: SCM-LC3 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc301_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempL31B2(CelBigStoneG100Temp):
    name = 'TempL31B2'
    description = 'lc301-bmc: LC3 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc301_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# lc302 temps

class CelBigStoneG100TempL321(CelBigStoneG100CoreTemp):
    name = 'TempL321'
    description = 'lc302: Core 0'
    driver_path = vhwmon_path + 'lc302:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL322(CelBigStoneG100CoreTemp):
    name = 'TempL322'
    description = 'lc302: Core 1'
    driver_path = vhwmon_path + 'lc302:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL323(CelBigStoneG100CoreTemp):
    name = 'TempL323'
    description = 'lc302: Core 2'
    driver_path = vhwmon_path + 'lc302:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL324(CelBigStoneG100CoreTemp):
    name = 'TempL324'
    description = 'lc302: Core 3'
    driver_path = vhwmon_path + 'lc302:coretemp_isa_0000'
    driver_hwmon = ['temp5']

# lc401 temps

class CelBigStoneG100TempL411(CelBigStoneG100CoreTemp):
    name = 'TempL411'
    description = 'lc401: Core 0'
    driver_path = vhwmon_path + 'lc401:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL412(CelBigStoneG100CoreTemp):
    name = 'TempL412'
    description = 'lc401: Core 1'
    driver_path = vhwmon_path + 'lc401:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL413(CelBigStoneG100CoreTemp):
    name = 'TempL413'
    description = 'lc401: Core 2'
    driver_path = vhwmon_path + 'lc401:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL414(CelBigStoneG100CoreTemp):
    name = 'TempL414'
    description = 'lc401: Core 3'
    driver_path = vhwmon_path + 'lc401:coretemp_isa_0000'
    driver_hwmon = ['temp5']

class CelBigStoneG100TempL41B1(CelBigStoneG100Temp):
    name = 'TempL41B1'
    description = 'lc401-bmc: SCM-LC4 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc401_bmc:adm1278_i2c_0_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempL41B2(CelBigStoneG100Temp):
    name = 'TempL41B2'
    description = 'lc401-bmc: LC4 Hot-Swap NFET Temp'
    driver_path = vhwmon_path + 'lc401_bmc:adm1278_i2c_11_10'
    driver_hwmon = ['temp1']

# lc402 temps

class CelBigStoneG100TempL421(CelBigStoneG100CoreTemp):
    name = 'TempL421'
    description = 'lc402: Core 0'
    driver_path = vhwmon_path + 'lc402:coretemp_isa_0000'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempL422(CelBigStoneG100CoreTemp):
    name = 'TempL422'
    description = 'lc402: Core 1'
    driver_path = vhwmon_path + 'lc402:coretemp_isa_0000'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempL423(CelBigStoneG100CoreTemp):
    name = 'TempL423'
    description = 'lc402: Core 2'
    driver_path = vhwmon_path + 'lc402:coretemp_isa_0000'
    driver_hwmon = ['temp4']

class CelBigStoneG100TempL424(CelBigStoneG100CoreTemp):
    name = 'TempL424'
    description = 'lc402: Core 3'
    driver_path = vhwmon_path + 'lc402:coretemp_isa_0000'
    driver_hwmon = ['temp5']

# chassis temps

class CelBigStoneG100TempC1(CelBigStoneG100Temp):
    name = 'TempC1'
    description = 'chassis: PSU 1 temp1'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_24_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC2(CelBigStoneG100Temp):
    name = 'TempC2'
    description = 'chassis: PSU 1 temp2'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_24_10'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempC3(CelBigStoneG100Temp):
    name = 'TempC3'
    description = 'chassis: PSU 1 temp3'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_24_10'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempC4(CelBigStoneG100Temp):
    name = 'TempC4'
    description = 'chassis: PSU 2 temp1'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_25_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC5(CelBigStoneG100Temp):
    name = 'TempC5'
    description = 'chassis: PSU 2 temp2'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_25_10'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempC6(CelBigStoneG100Temp):
    name = 'TempC6'
    description = 'chassis: PSU 2 temp3'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_25_10'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempC7(CelBigStoneG100Temp):
    name = 'TempC7'
    description = 'chassis: PSU 3 temp1'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_26_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC8(CelBigStoneG100Temp):
    name = 'TempC8'
    description = 'chassis: PSU 3 temp2'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_26_10'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempC9(CelBigStoneG100Temp):
    name = 'TempC9'
    description = 'chassis: PSU 3 temp3'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_26_10'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempC10(CelBigStoneG100Temp):
    name = 'TempC10'
    description = 'chassis: PSU 4 temp1'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_27_10'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC11(CelBigStoneG100Temp):
    name = 'TempC11'
    description = 'chassis: PSU 4 temp2'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_27_10'
    driver_hwmon = ['temp2']

class CelBigStoneG100TempC12(CelBigStoneG100Temp):
    name = 'TempC12'
    description = 'chassis: PSU 4 temp3'
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_27_10'
    driver_hwmon = ['temp3']

class CelBigStoneG100TempC13(CelBigStoneG100Temp):
    name = 'TempC13'
    description = 'chassis: FC1 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_106_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC14(CelBigStoneG100Temp):
    name = 'TempC14'
    description = 'chassis: FC1 Outlet Bottom Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_106_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC15(CelBigStoneG100Temp):
    name = 'TempC15'
    description = 'chassis: FC1 Outlet Top Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_106_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC16(CelBigStoneG100Temp):
    name = 'TempC16'
    description = 'chassis: FC1 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_106_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC17(CelBigStoneG100Temp):
    name = 'TempC17'
    description = 'chassis: SCM-FC1 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_113_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC18(CelBigStoneG100Temp):
    name = 'TempC18'
    description = 'chassis: SCM-FC1 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_114_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC19(CelBigStoneG100Temp):
    name = 'TempC19'
    description = 'chassis: FC2 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_122_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC20(CelBigStoneG100Temp):
    name = 'TempC20'
    description = 'chassis: FC2 Outlet Bottom Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_122_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC21(CelBigStoneG100Temp):
    name = 'TempC21'
    description = 'chassis: FC2 Outlet Top Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_122_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC22(CelBigStoneG100Temp):
    name = 'TempC22'
    description = 'chassis: FC2 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_122_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC23(CelBigStoneG100Temp):
    name = 'TempC23'
    description = 'chassis: SCM-FC2 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_129_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC24(CelBigStoneG100Temp):
    name = 'TempC24'
    description = 'chassis: SCM-FC2 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_130_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC25(CelBigStoneG100Temp):
    name = 'TempC25'
    description = 'chassis: FC3 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_138_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC26(CelBigStoneG100Temp):
    name = 'TempC26'
    description = 'chassis: FC3 Outlet Bottom Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_138_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC27(CelBigStoneG100Temp):
    name = 'TempC27'
    description = 'chassis: FC3 Outlet Top Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_138_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC28(CelBigStoneG100Temp):
    name = 'TempC28'
    description = 'chassis: FC3 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_138_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC29(CelBigStoneG100Temp):
    name = 'TempC29'
    description = 'chassis: SCM-FC3 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_145_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC30(CelBigStoneG100Temp):
    name = 'TempC30'
    description = 'chassis: SCM-FC3 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_146_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC31(CelBigStoneG100Temp):
    name = 'TempC31'
    description = 'chassis: FC4 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_154_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC32(CelBigStoneG100Temp):
    name = 'TempC32'
    description = 'chassis: FC4 Outlet Bottom Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_154_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC33(CelBigStoneG100Temp):
    name = 'TempC33'
    description = 'chassis: FC4 Outlet Top Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_154_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC34(CelBigStoneG100Temp):
    name = 'TempC34'
    description = 'chassis: FC4 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_154_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC35(CelBigStoneG100Temp):
    name = 'TempC35'
    description = 'chassis: SCM-FC4 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_161_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC36(CelBigStoneG100Temp):
    name = 'TempC36'
    description = 'chassis: SCM-FC4 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_162_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC37(CelBigStoneG100Temp):
    name = 'TempC37'
    description = 'chassis: CMM Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_3_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC38(CelBigStoneG100Temp):
    name = 'TempC38'
    description = 'chassis: CMM Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_3_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC39(CelBigStoneG100Temp):
    name = 'TempC39'
    description = 'chassis: LC101 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_42_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC40(CelBigStoneG100Temp):
    name = 'TempC40'
    description = 'chassis: LC101 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_42_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC41(CelBigStoneG100Temp):
    name = 'TempC41'
    description = 'chassis: LC101 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_42_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC42(CelBigStoneG100Temp):
    name = 'TempC42'
    description = 'chassis: LC101 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_42_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC43(CelBigStoneG100Temp):
    name = 'TempC43'
    description = 'chassis: LC102 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_45_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC44(CelBigStoneG100Temp):
    name = 'TempC44'
    description = 'chassis: LC102 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_45_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC45(CelBigStoneG100Temp):
    name = 'TempC45'
    description = 'chassis: LC102 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_45_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC46(CelBigStoneG100Temp):
    name = 'TempC46'
    description = 'chassis: LC102 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_45_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC47(CelBigStoneG100Temp):
    name = 'TempC47'
    description = 'chassis: SCM-LC1 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_49_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC48(CelBigStoneG100Temp):
    name = 'TempC48'
    description = 'chassis: SCM-LC1 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_50_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC49(CelBigStoneG100Temp):
    name = 'TempC49'
    description = 'chassis: LC201 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_58_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC50(CelBigStoneG100Temp):
    name = 'TempC50'
    description = 'chassis: LC201 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_58_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC51(CelBigStoneG100Temp):
    name = 'TempC51'
    description = 'chassis: LC201 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_58_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC52(CelBigStoneG100Temp):
    name = 'TempC52'
    description = 'chassis: LC201 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_58_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC53(CelBigStoneG100Temp):
    name = 'TempC53'
    description = 'chassis: LC202 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_61_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC54(CelBigStoneG100Temp):
    name = 'TempC54'
    description = 'chassis: LC202 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_61_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC55(CelBigStoneG100Temp):
    name = 'TempC55'
    description = 'chassis: LC202 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_61_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC56(CelBigStoneG100Temp):
    name = 'TempC56'
    description = 'chassis: LC202 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_61_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC57(CelBigStoneG100Temp):
    name = 'TempC57'
    description = 'chassis: SCM-LC2 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_65_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC58(CelBigStoneG100Temp):
    name = 'TempC58'
    description = 'chassis: SCM-LC2 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_66_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC59(CelBigStoneG100Temp):
    name = 'TempC59'
    description = 'chassis: LC301 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_74_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC60(CelBigStoneG100Temp):
    name = 'TempC60'
    description = 'chassis: LC301 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_74_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC61(CelBigStoneG100Temp):
    name = 'TempC61'
    description = 'chassis: LC301 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_74_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC62(CelBigStoneG100Temp):
    name = 'TempC62'
    description = 'chassis: LC301 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_74_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC63(CelBigStoneG100Temp):
    name = 'TempC63'
    description = 'chassis: LC302 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_77_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC64(CelBigStoneG100Temp):
    name = 'TempC64'
    description = 'chassis: LC302 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_77_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC65(CelBigStoneG100Temp):
    name = 'TempC65'
    description = 'chassis: LC302 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_77_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC66(CelBigStoneG100Temp):
    name = 'TempC66'
    description = 'chassis: LC302 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_77_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC67(CelBigStoneG100Temp):
    name = 'TempC67'
    description = 'chassis: SCM-LC3 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_81_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC68(CelBigStoneG100Temp):
    name = 'TempC68'
    description = 'chassis: SCM-LC3 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_82_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC69(CelBigStoneG100Temp):
    name = 'TempC69'
    description = 'chassis: LC401 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_90_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC70(CelBigStoneG100Temp):
    name = 'TempC70'
    description = 'chassis: LC401 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_90_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC71(CelBigStoneG100Temp):
    name = 'TempC71'
    description = 'chassis: LC401 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_90_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC72(CelBigStoneG100Temp):
    name = 'TempC72'
    description = 'chassis: LC401 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_90_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC73(CelBigStoneG100Temp):
    name = 'TempC73'
    description = 'chassis: LC402 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_93_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC74(CelBigStoneG100Temp):
    name = 'TempC74'
    description = 'chassis: LC402 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_93_49'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC75(CelBigStoneG100Temp):
    name = 'TempC75'
    description = 'chassis: LC402 Switch Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_93_4a'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC76(CelBigStoneG100Temp):
    name = 'TempC76'
    description = 'chassis: LC402 QSFP Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_93_4b'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC77(CelBigStoneG100Temp):
    name = 'TempC77'
    description = 'chassis: SCM-LC4 Inlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_97_48'
    driver_hwmon = ['temp1']

class CelBigStoneG100TempC78(CelBigStoneG100Temp):
    name = 'TempC78'
    description = 'chassis: SCM-LC4 Outlet Temp'
    driver_path = vhwmon_path + 'chassis:tmp75_i2c_98_49'
    driver_hwmon = ['temp1']

# chassis fans

class CelBigStoneG100Fan(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    no_pwm = True
    # using defaults for min, max, and var

class CelBigStoneG100Fan111(CelBigStoneG100Fan):
    name = "Fan111"
    description = "chassis: Fan 1-1 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_171_33'
    driver_hwmon = ['fan1']

class CelBigStoneG100Fan112(CelBigStoneG100Fan):
    name = "Fan112"
    description = "chassis: Fan 1-1 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_171_33'
    driver_hwmon = ['fan2']

class CelBigStoneG100Fan121(CelBigStoneG100Fan):
    name = "Fan121"
    description = "chassis: Fan 1-2 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_171_33'
    driver_hwmon = ['fan3']

class CelBigStoneG100Fan122(CelBigStoneG100Fan):
    name = "Fan122"
    description = "chassis: Fan 1-2 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_171_33'
    driver_hwmon = ['fan4']

class CelBigStoneG100Fan131(CelBigStoneG100Fan):
    name = "Fan131"
    description = "chassis: Fan 1-3 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_171_33'
    driver_hwmon = ['fan5']

class CelBigStoneG100Fan132(CelBigStoneG100Fan):
    name = "Fan132"
    description = "chassis: Fan 1-3 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_171_33'
    driver_hwmon = ['fan6']

class CelBigStoneG100Fan211(CelBigStoneG100Fan):
    name = "Fan211"
    description = "chassis: Fan 2-1 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_179_33'
    driver_hwmon = ['fan1']

class CelBigStoneG100Fan212(CelBigStoneG100Fan):
    name = "Fan212"
    description = "chassis: Fan 2-1 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_179_33'
    driver_hwmon = ['fan2']

class CelBigStoneG100Fan221(CelBigStoneG100Fan):
    name = "Fan221"
    description = "chassis: Fan 2-2 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_179_33'
    driver_hwmon = ['fan3']

class CelBigStoneG100Fan222(CelBigStoneG100Fan):
    name = "Fan222"
    description = "chassis: Fan 2-2 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_179_33'
    driver_hwmon = ['fan4']

class CelBigStoneG100Fan231(CelBigStoneG100Fan):
    name = "Fan231"
    description = "chassis: Fan 2-3 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_179_33'
    driver_hwmon = ['fan5']

class CelBigStoneG100Fan232(CelBigStoneG100Fan):
    name = "Fan232"
    description = "chassis: Fan 2-3 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_179_33'
    driver_hwmon = ['fan6']

class CelBigStoneG100Fan311(CelBigStoneG100Fan):
    name = "Fan311"
    description = "chassis: Fan 3-1 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_187_33'
    driver_hwmon = ['fan1']

class CelBigStoneG100Fan312(CelBigStoneG100Fan):
    name = "Fan312"
    description = "chassis: Fan 3-1 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_187_33'
    driver_hwmon = ['fan2']

class CelBigStoneG100Fan321(CelBigStoneG100Fan):
    name = "Fan321"
    description = "chassis: Fan 3-2 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_187_33'
    driver_hwmon = ['fan3']

class CelBigStoneG100Fan322(CelBigStoneG100Fan):
    name = "Fan322"
    description = "chassis: Fan 3-2 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_187_33'
    driver_hwmon = ['fan4']

class CelBigStoneG100Fan331(CelBigStoneG100Fan):
    name = "Fan331"
    description = "chassis: Fan 3-3 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_187_33'
    driver_hwmon = ['fan5']

class CelBigStoneG100Fan332(CelBigStoneG100Fan):
    name = "Fan332"
    description = "chassis: Fan 3-3 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_187_33'
    driver_hwmon = ['fan6']

class CelBigStoneG100Fan411(CelBigStoneG100Fan):
    name = "Fan411"
    description = "chassis: Fan 4-1 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_195_33'
    driver_hwmon = ['fan1']

class CelBigStoneG100Fan412(CelBigStoneG100Fan):
    name = "Fan412"
    description = "chassis: Fan 4-1 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_195_33'
    driver_hwmon = ['fan2']

class CelBigStoneG100Fan421(CelBigStoneG100Fan):
    name = "Fan421"
    description = "chassis: Fan 4-2 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_195_33'
    driver_hwmon = ['fan3']

class CelBigStoneG100Fan422(CelBigStoneG100Fan):
    name = "Fan422"
    description = "chassis: Fan 4-2 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_195_33'
    driver_hwmon = ['fan4']

class CelBigStoneG100Fan431(CelBigStoneG100Fan):
    name = "Fan431"
    description = "chassis: Fan 4-3 Front"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_195_33'
    driver_hwmon = ['fan5']

class CelBigStoneG100Fan432(CelBigStoneG100Fan):
    name = "Fan432"
    description = "chassis: Fan 4-3 Rear"
    driver_path = vhwmon_path + 'chassis:fancpld_i2c_195_33'
    driver_hwmon = ['fan6']

class CelBigStoneG100FanPSU1(CelBigStoneG100Fan):
    name = "FanPSU1"
    description = "chassis: PSU 1 Fan"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_24_10'
    driver_hwmon = ['fan1']

class CelBigStoneG100FanPSU2(CelBigStoneG100Fan):
    name = "FanPSU2"
    description = "chassis: PSU 2 Fan"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_25_10'
    driver_hwmon = ['fan1']

class CelBigStoneG100FanPSU3(CelBigStoneG100Fan):
    name = "FanPSU3"
    description = "chassis: PSU 3 Fan"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_26_10'
    driver_hwmon = ['fan1']

class CelBigStoneG100FanPSU4(CelBigStoneG100Fan):
    name = "FanPSU4"
    description = "chassis: PSU 4 Fan"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_27_10'
    driver_hwmon = ['fan1']

# chass psus

class CelBigStoneG100PSU(cumulus.sensors.PSU_Unit):
    hwmon_search = False
    # power1 is input, power2 and power3 are output
    driver_hwmon = ['power1', 'power2', 'power3']
    def get_status(self):
        val = 1
        val &= self.attrs.get('power1_input', 0) != 0
        val &= self.attrs.get('power2_input', 0) != 0
        val &= self.attrs.get('power3_input', 0) != 0
        self.attrs['psu_pwr_all_ok'] = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class CelBigStoneG100PSU1(CelBigStoneG100PSU):
    name = "PSU1"
    description = "chassis: PSU 1"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_24_10'

class CelBigStoneG100PSU2(CelBigStoneG100PSU):
    name = "PSU2"
    description = "chassis: PSU 2"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_25_10'

class CelBigStoneG100PSU3(CelBigStoneG100PSU):
    name = "PSU3"
    description = "chassis: PSU 3"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_26_10'

class CelBigStoneG100PSU4(CelBigStoneG100PSU):
    name = "PSU4"
    description = "chassis: PSU 4"
    driver_path = vhwmon_path + 'chassis:pfe3000_i2c_27_10'

class CelBigStoneG100Sensors():
    units = (CelBigStoneG100Temp1(),
             CelBigStoneG100Temp2(),
             CelBigStoneG100Temp3(),
             CelBigStoneG100Temp4(),

             CelBigStoneG100TempF11(),
             CelBigStoneG100TempF12(),
             CelBigStoneG100TempF13(),
             CelBigStoneG100TempF14(),
             CelBigStoneG100TempF1B1(),
             CelBigStoneG100TempF1B2(),

             CelBigStoneG100TempF21(),
             CelBigStoneG100TempF22(),
             CelBigStoneG100TempF23(),
             CelBigStoneG100TempF24(),
             CelBigStoneG100TempF2B1(),
             CelBigStoneG100TempF2B2(),

             CelBigStoneG100TempF31(),
             CelBigStoneG100TempF32(),
             CelBigStoneG100TempF33(),
             CelBigStoneG100TempF34(),
             CelBigStoneG100TempF3B1(),
             CelBigStoneG100TempF3B2(),

             CelBigStoneG100TempF41(),
             CelBigStoneG100TempF42(),
             CelBigStoneG100TempF43(),
             CelBigStoneG100TempF44(),
             CelBigStoneG100TempF4B1(),
             CelBigStoneG100TempF4B2(),

             CelBigStoneG100TempL111(),
             CelBigStoneG100TempL112(),
             CelBigStoneG100TempL113(),
             CelBigStoneG100TempL114(),
             CelBigStoneG100TempL11B1(),
             CelBigStoneG100TempL11B2(),

             CelBigStoneG100TempL121(),
             CelBigStoneG100TempL122(),
             CelBigStoneG100TempL123(),
             CelBigStoneG100TempL124(),

             CelBigStoneG100TempL211(),
             CelBigStoneG100TempL212(),
             CelBigStoneG100TempL213(),
             CelBigStoneG100TempL214(),
             CelBigStoneG100TempL21B1(),
             CelBigStoneG100TempL21B2(),

             CelBigStoneG100TempL221(),
             CelBigStoneG100TempL222(),
             CelBigStoneG100TempL223(),
             CelBigStoneG100TempL224(),

             CelBigStoneG100TempL311(),
             CelBigStoneG100TempL312(),
             CelBigStoneG100TempL313(),
             CelBigStoneG100TempL314(),
             CelBigStoneG100TempL31B1(),
             CelBigStoneG100TempL31B2(),

             CelBigStoneG100TempL321(),
             CelBigStoneG100TempL322(),
             CelBigStoneG100TempL323(),
             CelBigStoneG100TempL324(),

             CelBigStoneG100TempL411(),
             CelBigStoneG100TempL412(),
             CelBigStoneG100TempL413(),
             CelBigStoneG100TempL414(),
             CelBigStoneG100TempL41B1(),
             CelBigStoneG100TempL41B2(),

             CelBigStoneG100TempL421(),
             CelBigStoneG100TempL422(),
             CelBigStoneG100TempL423(),
             CelBigStoneG100TempL424(),

             CelBigStoneG100TempC1(),
             CelBigStoneG100TempC2(),
             CelBigStoneG100TempC3(),
             CelBigStoneG100TempC4(),
             CelBigStoneG100TempC5(),
             CelBigStoneG100TempC6(),
             CelBigStoneG100TempC7(),
             CelBigStoneG100TempC8(),
             CelBigStoneG100TempC9(),
             CelBigStoneG100TempC10(),
             CelBigStoneG100TempC11(),
             CelBigStoneG100TempC12(),
             CelBigStoneG100TempC13(),
             CelBigStoneG100TempC14(),
             CelBigStoneG100TempC15(),
             CelBigStoneG100TempC16(),
             CelBigStoneG100TempC17(),
             CelBigStoneG100TempC18(),
             CelBigStoneG100TempC19(),
             CelBigStoneG100TempC20(),
             CelBigStoneG100TempC21(),
             CelBigStoneG100TempC22(),
             CelBigStoneG100TempC23(),
             CelBigStoneG100TempC24(),
             CelBigStoneG100TempC25(),
             CelBigStoneG100TempC26(),
             CelBigStoneG100TempC27(),
             CelBigStoneG100TempC28(),
             CelBigStoneG100TempC29(),
             CelBigStoneG100TempC30(),
             CelBigStoneG100TempC31(),
             CelBigStoneG100TempC32(),
             CelBigStoneG100TempC33(),
             CelBigStoneG100TempC34(),
             CelBigStoneG100TempC35(),
             CelBigStoneG100TempC36(),
             CelBigStoneG100TempC37(),
             CelBigStoneG100TempC38(),
             CelBigStoneG100TempC39(),
             CelBigStoneG100TempC40(),
             CelBigStoneG100TempC41(),
             CelBigStoneG100TempC42(),
             CelBigStoneG100TempC43(),
             CelBigStoneG100TempC44(),
             CelBigStoneG100TempC45(),
             CelBigStoneG100TempC46(),
             CelBigStoneG100TempC47(),
             CelBigStoneG100TempC48(),
             CelBigStoneG100TempC49(),
             CelBigStoneG100TempC50(),
             CelBigStoneG100TempC51(),
             CelBigStoneG100TempC52(),
             CelBigStoneG100TempC53(),
             CelBigStoneG100TempC54(),
             CelBigStoneG100TempC55(),
             CelBigStoneG100TempC56(),
             CelBigStoneG100TempC57(),
             CelBigStoneG100TempC58(),
             CelBigStoneG100TempC59(),
             CelBigStoneG100TempC60(),
             CelBigStoneG100TempC61(),
             CelBigStoneG100TempC62(),
             CelBigStoneG100TempC63(),
             CelBigStoneG100TempC64(),
             CelBigStoneG100TempC65(),
             CelBigStoneG100TempC66(),
             CelBigStoneG100TempC67(),
             CelBigStoneG100TempC68(),
             CelBigStoneG100TempC69(),
             CelBigStoneG100TempC70(),
             CelBigStoneG100TempC71(),
             CelBigStoneG100TempC72(),
             CelBigStoneG100TempC73(),
             CelBigStoneG100TempC74(),
             CelBigStoneG100TempC75(),
             CelBigStoneG100TempC76(),
             CelBigStoneG100TempC77(),
             CelBigStoneG100TempC78(),

             CelBigStoneG100Fan111(),
             CelBigStoneG100Fan112(),
             CelBigStoneG100Fan121(),
             CelBigStoneG100Fan122(),
             CelBigStoneG100Fan131(),
             CelBigStoneG100Fan132(),
             CelBigStoneG100Fan211(),
             CelBigStoneG100Fan212(),
             CelBigStoneG100Fan221(),
             CelBigStoneG100Fan222(),
             CelBigStoneG100Fan231(),
             CelBigStoneG100Fan232(),
             CelBigStoneG100Fan311(),
             CelBigStoneG100Fan312(),
             CelBigStoneG100Fan321(),
             CelBigStoneG100Fan322(),
             CelBigStoneG100Fan331(),
             CelBigStoneG100Fan332(),
             CelBigStoneG100Fan411(),
             CelBigStoneG100Fan412(),
             CelBigStoneG100Fan421(),
             CelBigStoneG100Fan422(),
             CelBigStoneG100Fan431(),
             CelBigStoneG100Fan432(),

             CelBigStoneG100FanPSU1(),
             CelBigStoneG100FanPSU2(),
             CelBigStoneG100FanPSU3(),
             CelBigStoneG100FanPSU4(),

             CelBigStoneG100PSU1(),
             CelBigStoneG100PSU2(),
             CelBigStoneG100PSU3(),
             CelBigStoneG100PSU4(),
            )

class CelBigStoneG100BMC(cumulus.platform.BMC):
    type = "openbmc"
    def __init__(self, chassis):
        if "__init__" in cumulus.platform.BMC.__dict__:
            cumulus.platform.BMC.__init__(self)
        if not chassis.HasSlotNum():
            return
        slot = chassis.GetSlotNum()
        self.name = "%s-bmc" % chassis.GetModuleName(slot)
        if slot < 5:
            # fabric card
            addr = "fe80::%x:1" % (slot,)
        elif slot % 2 == 1:
            # line card left
            addr = "fe80::%x01:1" % ((slot - 3)/2,)
        else:
            # line card right
            addr = "fe80::%x02:1" % ((slot - 4)/2,)
        self.ipv6_addr = addr + "%" + chassis.GetMgmtIf()

class CelBigStoneG100(cumulus.platform.Platform):
    cpld = None
    statusleds = None
    pwms = None
    sensors = CelBigStoneG100Sensors()
    chassis = cumulus.chassis.probe()
    bmc = CelBigStoneG100BMC(chassis)
