#
# Copyright 2016, 2017.  Cumulus Networks, Inc.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelSeastoneMgmtSFP(cumulus.platform.TomahawkMgmt10GPort):
    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
        cumulus.platform.TomahawkMgmt10GPort.__init__(self,
                                                      label=label,
                                                      serdes=serdes,
                                                      phy_types=None,
                                                      phy_addrs=None,
                                                      phy_layer=sfp.type.upper(),
                                                      leds=leds,
                                                      rx_lane_map=None,
                                                      rx_polarity_flip=None,
                                                      tx_lane_map=None,
                                                      tx_polarity_flip=None,
                                                      pre_emphasis=None,
                                                      oversub=True,
                                                      unit=0)
        self.sfp = sfp

    _set_led_mgmt = '''
;;;
;;; A single bi-color LED is used to indicate link/activity on the
;;; management ports
;;;
set_led:
    call    set_green_led
    ret
'''

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        set_calls[led_index_list[0]] = (fn_name, self._set_led_mgmt, led_port)
        set_calls[led_index_list[0]+1] = None

        return set_calls

class CelSeastoneQSFP28(cumulus.platform.Tomahawk100GPort):
    '''
;;; Celestica Seastone
;;;
;;; Each port has eight bits of LED data controlling four bi-color green/red
;;; LEDs.  Each LED behaves as follows:
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 | green | amber |
;;;  2nd bit   +-------+-------+
;;;          1 |  off  |  off  |
;;;            +-------+-------+
;;;'''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_green_led:
; pack two bits
;  00 => green
;  11 => off
        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        jmp     pack_it


set_led_off:
; pack two '1' bits
;  11 => off
        push   1
        jmp    pack_it
        ret

id_mode:
        push    1
        cmp     a, (ID_PORT_P)
        jnz     pack_it
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
        tinv
pack_it:
; pack two copies of T
        pop
        push    cy
        pack
        push    cy
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity.
;;;
set_led:
        call    set_green_led  ; set the first LED to green/off
        call    set_led_off
        call    set_led_off
        call    set_led_off

        ret
'''
    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and second bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_green_led
        inc     a
        inc     a
        call    set_green_led
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class CelSeastoneLED(cumulus.ledup.TomahawkLEDController):
    led_functions = (CelSeastoneQSFP28._set_led_common)
    num_leds = 128
    has_phys_id = True


# Celestica Seastone Switch Ports

class CelSeastoneSwitch(cumulus.platform.BCMSwitch):
    ledup = CelSeastoneLED
    _mgmtSfp = CelSeastoneMgmtSFP
    _qsfp28 = CelSeastoneQSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1", serdes=( 64,  65,  66,  67), leds=(1, 0),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x2310),
        _qsfp28(label="2", serdes=( 68,  69,  70,  71), leds=(1, 8),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x0002,
              tx_lane_map=0x3201, rx_lane_map=0x2301),
        _qsfp28(label="3", serdes=( 72,  73,  74,  75), leds=(1, 16),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0132, rx_lane_map=0x0123),
        _qsfp28(label="4", serdes=( 76,  77,  78, 79), leds=(1,  24),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x1320),
        _qsfp28(label="5", serdes=( 32,  33,  34,  35), leds=(1, 32),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0000,
              tx_lane_map=0x3021, rx_lane_map=0x0213),
        _qsfp28(label="6", serdes=( 36,  37,  38,  39), leds=(1, 40),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x1023),
        _qsfp28(label="7", serdes=( 40,  41,  42,  43), leds=(1, 48),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x0000,
              tx_lane_map=0x2310, rx_lane_map=0x3210),
        _qsfp28(label="8", serdes=( 44,  45,  46,  47), leds=(1, 56),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x1302),
        _qsfp28(label="9", serdes=( 48,  49,  50,  51), leds=(1, 64),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x0213),
        _qsfp28(label="10", serdes=( 52,  53,  54,  55), leds=(1, 72),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x2301, rx_lane_map=0x2310),
        _qsfp28(label="11", serdes=( 56,  57,  58,  59), leds=(1, 80),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x0213),
        _qsfp28(label="12", serdes=( 60,  61,  62,  63), leds=(1, 88),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x2310),
        _qsfp28(label="13", serdes=( 80,  81,  82,  83), leds=(1, 96),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000e,
              tx_lane_map=0x0213, rx_lane_map=0x2301),
        _qsfp28(label="14", serdes=( 84,  85,  86,  87), leds=(1, 104),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000d,
              tx_lane_map=0x0132, rx_lane_map=0x3210),
        _qsfp28(label="15", serdes=( 88,  89,  90,  91), leds=(1, 112),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0008,
              tx_lane_map=0x0132, rx_lane_map=0x2031),
        _qsfp28(label="16", serdes=( 92,  93,  94,  95), leds=(1, 120),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x3201),
        _qsfp28(label="17", serdes=( 96,  97,  98,  99), leds=(0, 0),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x2301),
        _qsfp28(label="18", serdes=(100, 101, 102, 103), leds=(0, 8),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0132, rx_lane_map=0x3201),
        _qsfp28(label="19", serdes=(104, 105, 106, 107), leds=(0, 16),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x2031),
        _qsfp28(label="20", serdes=(108, 109, 110, 111), leds=(0, 24),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,
              tx_lane_map=0x2031, rx_lane_map=0x2301),
        _qsfp28(label="21", serdes=(  0,   1,   2,   3), leds=(0, 32),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3120),
        _qsfp28(label="22", serdes=(  4,   5,   6,   7), leds=(0, 40),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x0000,
              tx_lane_map=0x0132, rx_lane_map=0x1023),
        _qsfp28(label="23", serdes=(  8,   9,  10,  11), leds=(0, 48),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0008,
              tx_lane_map=0x3201, rx_lane_map=0x3120),
        _qsfp28(label="24", serdes=( 12,  13,  14,  15), leds=(0, 56),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x1032),
        _qsfp28(label="25", serdes=( 16,  17,  18,  19), leds=(0, 64),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x0000,
              tx_lane_map=0x2310, rx_lane_map=0x3210),
        _qsfp28(label="26", serdes=( 20,  21,  22,  23), leds=(0, 72),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x2301, rx_lane_map=0x3120),
        _qsfp28(label="27", serdes=( 24,  25,  26,  27), leds=(0, 80),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0005,
              tx_lane_map=0x3201, rx_lane_map=0x0213),
        _qsfp28(label="28", serdes=( 28,  29,  30,  31), leds=(0, 88),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1023),
        _qsfp28(label="29", serdes=(112, 113, 114, 115), leds=(0, 96),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x1320, rx_lane_map=0x1302),
        _qsfp28(label="30", serdes=(116, 117, 118, 119), leds=(0, 104),
              tx_polarity_flip=0x0004, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x1023),
        _qsfp28(label="31", serdes=(120, 121, 122, 123), leds=(0, 112),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3120, rx_lane_map=0x3120),
        _qsfp28(label="32", serdes=(124, 125, 126, 127), leds=(0, 120),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x2310),
        _mgmtSfp(label="33", serdes=(130,), leds=(2,126)),
    )
    ''' rev A0 systems have some different port mappings and polarity
        settings. The rev A0 boards are detected by getting a value
        of '4.0' in the cpld4_version register. Anything else and
        we assume a newer unit and don't do the fixup.
    '''
    try:
        if open('/sys/bus/platform/devices/cel_seastone_cpld.0/cpld4_version').read().strip() == '4.0':
            # port 24
            ports[23].rx_lane_map = 0x1023
            ports[23].rx_polarity_flip = 0x00c
            # port 26
            ports[25].tx_lane_map = 0x2013
            ports[25].rx_lane_map = 0x2130
    except:
        pass

# ***** Celestica Seastone Sensors *****

# Celestica Seastone Power Supplies

class CelSeastonePSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod',  'dps460']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'dps460']
                subprocess.call(cmd)
            self.init_done = True

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '1':
            self.attrs[self.psu_present] = 1
        else:
            self.attrs[self.psu_present] = 0
        if self.get_gpio_value(self.psu_all_ok) == '1':
            self.attrs[self.psu_all_ok] = 1
        else:
            self.attrs[self.psu_all_ok] = 0
        return 0

class CelSeastonePSU1(CelSeastonePSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_all_ok = 'psu_pwr1_all_ok'

class CelSeastonePSU2(CelSeastonePSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_all_ok = 'psu_pwr2_all_ok'


# Celestica Seastone Board Temperature Sensors
### Need to check descriptions of all the sensors

# Intel Rangeley on-die temperature sensors
CelSeastone_core_path = '/sys/devices/platform/coretemp.0'

class CelSeastoneTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Intel Core 0 Temp Sensor'
    driver_path = CelSeastone_core_path
    driver_hwmon = ['temp2']
    temp_crit = 98
    temp_max = 90
    temp_max_hyst = 65

class CelSeastoneTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Intel Core 1 Temp Sensor'
    driver_path = CelSeastone_core_path
    driver_hwmon = ['temp3']
    temp_crit = 98
    temp_max = 90
    temp_max_hyst = 65

class CelSeastoneTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Intel Core 2 Temp Sensor'
    driver_path = CelSeastone_core_path
    driver_hwmon = ['temp4']
    temp_crit = 98
    temp_max = 90
    temp_max_hyst = 65

class CelSeastoneTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Intel Core 3 Temp Sensor'
    driver_path = CelSeastone_core_path
    driver_hwmon = ['temp5']
    temp_crit = 98
    temp_max = 90
    temp_max_hyst = 65

# NOTE from Smallstone XP (saving for reference if needed...)
# This sensor is giving low temperature (0-05C) readings at a customer site.
# Chances are this sensor is broken. So commenting it out of smond and pwmd.

class CelSeastoneTemp5(cumulus.sensors.Temp_Unit):
     name = 'Temp5'
     description = 'Intel CPU external sensor 1'
     driver_path = '/sys/bus/i2c/devices/14-0048'
     temp_crit = 70
     temp_max = 65

class CelSeastoneTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Intel CPU external sensor 2'
    driver_path = '/sys/bus/i2c/devices/15-004e'
    temp_crit = 70
    temp_max = 65

class CelSeastoneTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Rear Outlet Air sensor'
    driver_path = '/sys/bus/i2c/devices/24-0049'
    temp_crit = 60
    temp_max = 48
    temp_min = -10
    temp_lcrit = -15

class CelSeastoneTemp8(cumulus.sensors.Temp_Unit):
    name = 'Temp8'
    description = 'Front Outlet Air sensor'
    driver_path = '/sys/bus/i2c/devices/23-0048'
    temp_crit = 60
    temp_max = 48
    temp_min = -10
    temp_lcrit = -15

class CelSeastoneTemp9(cumulus.sensors.Temp_Unit):
    name = 'Temp9'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/bus/i2c/devices/25-004a'
    temp_crit = 80
    temp_max = 75

class CelSeastoneTemp10(cumulus.sensors.Temp_Unit):
    name = 'Temp10'
    description = "DIMM 1 Temp Sensor"
    driver_path = '/sys/bus/i2c/devices/0-0018'
    temp_crit = 95
    temp_max = 90
    temp_hyst = 65
    temp_min = 5

class CelSeastoneTHTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class CelSeastoneTHTemp1(CelSeastoneTHTemp):
    name = 'Temp11'
    driver_hwmon = ['temp1']

class CelSeastoneTHTemp2(CelSeastoneTHTemp):
    name = 'Temp12'
    driver_hwmon = ['temp2']

class CelSeastoneTHTemp3(CelSeastoneTHTemp):
    name = 'Temp13'
    driver_hwmon = ['temp3']

class CelSeastoneTHTemp4(CelSeastoneTHTemp):
    name = 'Temp14'
    driver_hwmon = ['temp4']

class CelSeastoneTHTemp5(CelSeastoneTHTemp):
    name = 'Temp15'
    driver_hwmon = ['temp5']

class CelSeastoneTHTemp6(CelSeastoneTHTemp):
    name = 'Temp16'
    driver_hwmon = ['temp6']

class CelSeastoneTHTemp7(CelSeastoneTHTemp):
    name = 'Temp17'
    driver_hwmon = ['temp7']

class CelSeastoneTHTemp8(CelSeastoneTHTemp):
    name = 'Temp18'
    driver_hwmon = ['temp8']

# The second DIMM is not installed at this time.
#
#class CelSeastoneTemp11(cumulus.sensors.Temp_Unit):
#    name = 'Temp11'
#    description = "DIMM 2 Temp Sensor"
#    driver_path = '/sys/bus/i2c/devices/0-0019'
#    temp_crit = 95
#    temp_max = 90
#    temp_hyst = 65
#    temp_min = 5


# Celestica Seastone PSU Temp Sensors

class CelSeastonePSUTemp(cumulus.sensors.Temp_Unit):
    def find_and_add_attrs(self):
        if ((self.get_gpio_value(self.temp_present) == '1') and
            (self.get_gpio_value(self.temp_all_ok) == '1')):
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelSeastonePSU1Temp1(CelSeastonePSUTemp):
    psu = 'PSU1'
    name = 'PSU1Temp1'
    description = 'PSU1 Inlet Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/10-005a'
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr1_present'
    temp_all_ok = 'psu_pwr1_all_ok'
    temp_crit = 100
    temp_max = 60

class CelSeastonePSU1Temp2(CelSeastonePSUTemp):
    psu = 'PSU1'
    name = 'PSU1Temp2'
    description = 'PSU1 Heatsink Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/10-005a'
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr1_present'
    temp_all_ok = 'psu_pwr1_all_ok'
    temp_crit = 100
    temp_max = 60

class CelSeastonePSU2Temp1(CelSeastonePSUTemp):
    psu = 'PSU2'
    name = 'PSU2Temp1'
    description = 'PSU2 Inlet Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/11-005b'
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr2_present'
    temp_all_ok = 'psu_pwr2_all_ok'
    temp_crit = 100
    temp_max = 60

class CelSeastonePSU2Temp2(CelSeastonePSUTemp):
    psu = 'PSU2'
    name = 'PSU2Temp2'
    description = 'PSU2 Heatsink Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/11-005b'
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr2_present'
    temp_all_ok = 'psu_pwr2_all_ok'
    temp_crit = 100
    temp_max = 60


# Celestica Seastone Fans

class CelSeastoneFan(cumulus.sensors.Fan_Unit):
    fan_min = 5300
    fan_max = 23000
    hwmon_search = False

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

# Note the inlet and outlet fans have their Tachs/PWMs swizzled differently.
# Refer to the hardware spec for more details.

CelSeastone_emc2305_1 = '/sys/bus/i2c/devices/13-002e'
CelSeastone_emc2305_2 = '/sys/bus/i2c/devices/13-004d'

class CelSeastoneFan1(CelSeastoneFan):
    name = 'Fan1Out'
    description = 'Fan1 Outlet'
    driver_path = CelSeastone_emc2305_2
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan1_present'

class CelSeastoneFan2(CelSeastoneFan):
    name = 'Fan1In'
    description = 'Fan1 Inlet'
    driver_path = CelSeastone_emc2305_1
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan1_present'

class CelSeastoneFan3(CelSeastoneFan):
    name = 'Fan2Out'
    description = 'Fan2 Outlet'
    driver_path = CelSeastone_emc2305_2
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan2_present'

class CelSeastoneFan4(CelSeastoneFan):
    name = 'Fan2In'
    description = 'Fan2 Inlet'
    driver_path = CelSeastone_emc2305_1
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan2_present'

class CelSeastoneFan5(CelSeastoneFan):
    name = 'Fan3Out'
    description = 'Fan3 Outlet'
    driver_path = CelSeastone_emc2305_2
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan3_present'

class CelSeastoneFan6(CelSeastoneFan):
    name = 'Fan3In'
    description = 'Fan3 Inlet'
    driver_path = CelSeastone_emc2305_1
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_present = 'fan3_present'

class CelSeastoneFan7(CelSeastoneFan):
    name = 'Fan4Out'
    description = 'Fan4 Outlet'
    driver_path = CelSeastone_emc2305_2
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_present = 'fan4_present'

class CelSeastoneFan8(CelSeastoneFan):
    name = 'Fan4In'
    description = 'Fan4 Inlet'
    driver_path = CelSeastone_emc2305_1
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan4_present'

class CelSeastoneFan9(CelSeastoneFan):
    name = 'Fan5Out'
    description = 'Fan5 Outlet'
    driver_path = CelSeastone_emc2305_2
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan5_present'

class CelSeastoneFan10(CelSeastoneFan):
    name = 'Fan5In'
    description = 'Fan5 Inlet'
    driver_path = CelSeastone_emc2305_1
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan5_present'


# Celestica Seastone PSU Fans

class CelSeastonePSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    minpwm = 64
    fan_max = 18000
    fan_min = 4700

    def find_and_add_attrs(self):
        if ((self.get_gpio_value(self.fan_present) == '1') and
            (self.get_gpio_value(self.fan_all_ok) == '1')):
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelSeastonePSU1Fan(CelSeastonePSUFan):
    name = 'PSU1Fan'
    description = 'PSU1 Fan'
    driver_path = '/sys/bus/i2c/devices/10-005a'
    fan_present = 'psu_pwr1_present'
    fan_all_ok = 'psu_pwr1_all_ok'

class CelSeastonePSU2Fan(CelSeastonePSUFan):
    name = 'PSU2Fan'
    description = 'PSU2 Fan'
    driver_path = '/sys/bus/i2c/devices/11-005b'
    fan_present = 'psu_pwr2_present'
    fan_all_ok = 'psu_pwr2_all_ok'

class CelSeastoneSensors():
    units = (CelSeastonePSU1(),
             CelSeastonePSU2(),
             CelSeastoneTemp1(),
             CelSeastoneTemp2(),
             CelSeastoneTemp3(),
             CelSeastoneTemp4(),
             CelSeastoneTemp5(),
             CelSeastoneTemp6(),
             CelSeastoneTemp7(),
             CelSeastoneTemp8(),
             CelSeastoneTemp9(),
             CelSeastoneTemp10(),
#             CelSeastoneTemp11(),
             CelSeastoneTHTemp1(),
             CelSeastoneTHTemp2(),
             CelSeastoneTHTemp3(),
             CelSeastoneTHTemp4(),
             CelSeastoneTHTemp5(),
             CelSeastoneTHTemp6(),
             CelSeastoneTHTemp7(),
             CelSeastoneTHTemp8(),
             CelSeastonePSU1Temp1(),
             CelSeastonePSU1Temp2(),
             CelSeastonePSU2Temp1(),
             CelSeastonePSU2Temp2(),
             CelSeastoneFan1(),
             CelSeastoneFan2(),
             CelSeastoneFan3(),
             CelSeastoneFan4(),
             CelSeastoneFan5(),
             CelSeastoneFan6(),
             CelSeastoneFan7(),
             CelSeastoneFan8(),
             CelSeastoneFan9(),
             CelSeastoneFan10(),
             CelSeastonePSU1Fan(),
             CelSeastonePSU2Fan(),
            )


# Celestica Seastone PWM Controllers

class CelSeastonePWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (CelSeastoneFan1(),
            CelSeastoneFan2(),
            CelSeastoneFan3(),
            CelSeastoneFan4(),
            CelSeastoneFan5(),
            CelSeastoneFan6(),
            CelSeastoneFan7(),
            CelSeastoneFan8(),
            CelSeastoneFan9(),
            CelSeastoneFan10(),
    )
    temps = (CelSeastoneTemp1(),
             CelSeastoneTemp2(),
             CelSeastoneTemp3(),
             CelSeastoneTemp4(),
             CelSeastoneTemp5(),
             CelSeastoneTemp6(),
             CelSeastoneTemp7(),
             CelSeastoneTemp8(),
             CelSeastoneTemp9(),
             CelSeastoneTemp10(),
             CelSeastoneTHTemp1(),
             CelSeastoneTHTemp2(),
             CelSeastoneTHTemp3(),
             CelSeastoneTHTemp4(),
             CelSeastoneTHTemp5(),
             CelSeastoneTHTemp6(),
             CelSeastoneTHTemp7(),
             CelSeastoneTHTemp8(),
            )

class CelSeastonePSU1PWM(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (CelSeastonePSU1Fan(),
            )
    temps = (CelSeastonePSU1Temp1(),
             CelSeastonePSU1Temp2(),
             )

class CelSeastonePSU2PWM(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (CelSeastonePSU2Fan(),
            )
    temps = (CelSeastonePSU2Temp1(),
             CelSeastonePSU2Temp2(),
             )

class CelSeastonePWMs():
    units = (CelSeastonePWM(),
             CelSeastonePSU1PWM(),
             CelSeastonePSU2PWM(),
             )


# ***** Celestica Seastone CPLD *****

class CelSeastoneCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/cel_seastone_cpld.0'

'''
    Celestica Seastone LEDs

    3x1 Front Panel LEDs
        System:
          green - all systems ok
          green slow blinking - fan missing or broken, psu missing or broken
        PSU1:
          green - psu1 installed and working
          off - psu1 missing or broken
        PSU2:
          green - psu2 installed and working
          off - psu2 missing or broken

    Fan Tray LEDs
        Fan1 - Fan5:
          green - fan installed and working
          off - fan missing or broken
'''

# ***** Celestica Seastone Status LEDs *****

class CelSeastoneSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System LED'
    led_location = 'led_status'
    bad_led_color = 'green_blinking'

class CelSeastonePSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1 LED'
    sources = ['PSU1']
    led_location = 'led_psu1'
    bad_led_color = 'off'

class CelSeastonePSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2 LED'
    sources = ['PSU2']
    led_location = 'led_psu2'
    bad_led_color = 'off'

class CelSeastoneFanLED(cumulus.sensors.StatusLED_Unit):
    good_led_color = 'green'
    bad_led_color  = 'amber'
    green_colormap = {'green':'1',  'amber':'1',  'red':'0',  'off':'0'}
    red_colormap   = {'green':'0',  'amber':'1',  'red':'1',  'off':'0'}
    colormap       = {'10':'green', '11':'amber', '01':'red', '00':'off'}

    def led_update(self, color):
        self.set_gpio_value(self.led_location[0], self.green_colormap[color])
        self.set_gpio_value(self.led_location[1], self.red_colormap[color])

    def led_read(self):
        green_color = self.get_gpio_value(self.led_location[0])
        red_color = self.get_gpio_value(self.led_location[1])
        return self.name, self.colormap[green_color + red_color]

class CelSeastoneFan1LED(CelSeastoneFanLED):
    name = 'Fan1 LED'
    sources = ['Fan1Out', 'Fan1In']
    led_location = ['led_fan1_green', 'led_fan1_red']

class CelSeastoneFan2LED(CelSeastoneFanLED):
    name = 'Fan2 LED'
    sources = ['Fan2Out', 'Fan2In']
    led_location = ['led_fan2_green', 'led_fan2_red']

class CelSeastoneFan3LED(CelSeastoneFanLED):
    name = 'Fan3 LED'
    sources = ['Fan3Out', 'Fan3In']
    led_location = ['led_fan3_green', 'led_fan3_red']

class CelSeastoneFan4LED(CelSeastoneFanLED):
    name = 'Fan4 LED'
    sources = ['Fan4Out', 'Fan4In']
    led_location = ['led_fan4_green', 'led_fan4_red']

class CelSeastoneFan5LED(CelSeastoneFanLED):
    name = 'Fan5 LED'
    sources = ['Fan5Out', 'Fan5In']
    led_location = ['led_fan5_green', 'led_fan5_red']

class CelSeastoneLEDs():
    units = (CelSeastoneSystemLED(),
             CelSeastonePSU1LED(),
             CelSeastonePSU2LED(),
             CelSeastoneFan1LED(),
             CelSeastoneFan2LED(),
             CelSeastoneFan3LED(),
             CelSeastoneFan4LED(),
             CelSeastoneFan5LED(),
             )

# ***** Celestica Seastone Class *****

class CelSeastone(cumulus.platform.Platform):
    name = 'cel,seastone'
    chip = cumulus.platform.TomahawkChip()
    switch = CelSeastoneSwitch(chip)
    cpld = CelSeastoneCPLD()
    statusleds = CelSeastoneLEDs()
    pwms = CelSeastonePWMs()
    sensors = CelSeastoneSensors()

Platform = CelSeastone
