#
# Copyright 2016.  Cumulus Networks, Inc.
#
# dni_3048up.py --
#     DNI-3048UP switch platforms
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

''' DNI 3048UP
'''
class DNI3048UPSFPp(cumulus.platform.Helix410GPort):
    _set_led = '''
;;; DNI3048UPSFPp
;;;
;;; 3 bits  per port, 1: off, 0: on
;;;
;;; 1st 2-bits: LINK/SPD LED
;;; Bit-0  Bit-1  Color
;;;    0    0     Amber
;;;    0    1     Green
;;;
;;; 3rd bit: Activity LED
;;;
set_led:
        port a
        pushst  PORTSTATUS_ONE
        pack

        port a
        pushst  PORTSTATUS_LINKEN
        tinv
        pack

        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pack

        ret

'''
    def __init__(self, label, hw_port, leds, phy_addr, rx_polarity_flip=None,
                        tx_polarity_flip=None):
        cumulus.platform.Helix410GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type='84754',
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addr, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None

        return set_calls

class DNI3048UPGigE(cumulus.platform.Helix41GPort):
    _set_led = '''
;;; DNI3048UP1GigE
;;;
;;; 4 bits  per port, 1: off, 0: on
;;;
;;; 1st bit: activity
;;; 2nd bit: POE
;;; 3nd bit: green
;;; 4rd bit: yellow
;;;
set_led:
	port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

	port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

        ret

'''
    def __init__(self, label, hw_port, phy_addr, prim_offset, leds):
        cumulus.platform.Helix41GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=leds,
                                                unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None
        set_calls[led_index + 3] = None

        return set_calls

class DNI3048UPLED(cumulus.ledup.Helix4LEDController):
    num_leds = 255

class DNI3048UPSwitch(cumulus.platform.BCMSwitch):
    ledup = DNI3048UPLED
    _gige = DNI3048UPGigE
    _sfp = DNI3048UPSFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True

    ports = (
        _gige(label="1",   hw_port='ge0',   phy_addr=0x1, prim_offset=0x0100, leds=None),
        _gige(label="2",   hw_port='ge1',   phy_addr=0x2, prim_offset=0x0101, leds=None),
        _gige(label="3",   hw_port='ge2',   phy_addr=0x3, prim_offset=0x0102, leds=None),
        _gige(label="4",   hw_port='ge3',   phy_addr=0x4, prim_offset=0x0103, leds=None),
        _gige(label="5",   hw_port='ge4',   phy_addr=0x5, prim_offset=0x0104, leds=None),
        _gige(label="6",   hw_port='ge5',   phy_addr=0x6, prim_offset=0x0105, leds=None),
        _gige(label="7",   hw_port='ge6',   phy_addr=0x7, prim_offset=0x0106, leds=None),
        _gige(label="8",   hw_port='ge7',   phy_addr=0x8, prim_offset=0x0107, leds=None),

        _gige(label="9",   hw_port='ge8',   phy_addr=0xa, prim_offset=0x0900, leds=None),
        _gige(label="10",  hw_port='ge9',   phy_addr=0xb, prim_offset=0x0901, leds=None),
        _gige(label="11",  hw_port='ge10',  phy_addr=0xc, prim_offset=0x0902, leds=None),
        _gige(label="12",  hw_port='ge11',  phy_addr=0xd, prim_offset=0x0903, leds=None),
        _gige(label="13",  hw_port='ge12',  phy_addr=0xe, prim_offset=0x0904, leds=None),
        _gige(label="14",  hw_port='ge13',  phy_addr=0xf, prim_offset=0x0905, leds=None),
        _gige(label="15",  hw_port='ge14',  phy_addr=0x10, prim_offset=0x0906, leds=None),
        _gige(label="16",  hw_port='ge15',  phy_addr=0x11, prim_offset=0x0907, leds=None),

        _gige(label="17",  hw_port='ge16',  phy_addr=0x13, prim_offset=0x1100, leds=None),
        _gige(label="18",  hw_port='ge17',  phy_addr=0x14, prim_offset=0x1101, leds=None),
        _gige(label="19",  hw_port='ge18',  phy_addr=0x15, prim_offset=0x1102, leds=None),
        _gige(label="20",  hw_port='ge19',  phy_addr=0x16, prim_offset=0x1103, leds=None),
        _gige(label="21",  hw_port='ge20',  phy_addr=0x17, prim_offset=0x1104, leds=None),
        _gige(label="22",  hw_port='ge21',  phy_addr=0x18, prim_offset=0x1105, leds=None),
        _gige(label="23",  hw_port='ge22',  phy_addr=0x19, prim_offset=0x1106, leds=None),
        _gige(label="24",  hw_port='ge23',  phy_addr=0x1a, prim_offset=0x1107, leds=None),

        _gige(label="25",  hw_port='ge24',  phy_addr=0x21, prim_offset=0x1900, leds=None),
        _gige(label="26",  hw_port='ge25',  phy_addr=0x22, prim_offset=0x1901, leds=None),
        _gige(label="27",  hw_port='ge26',  phy_addr=0x23, prim_offset=0x1902, leds=None),
        _gige(label="28",  hw_port='ge27',  phy_addr=0x24, prim_offset=0x1903, leds=None),
        _gige(label="29",  hw_port='ge28',  phy_addr=0x25, prim_offset=0x1904, leds=None),
        _gige(label="30",  hw_port='ge29',  phy_addr=0x26, prim_offset=0x1905, leds=None),
        _gige(label="31",  hw_port='ge30',  phy_addr=0x27, prim_offset=0x1906, leds=None),
        _gige(label="32",  hw_port='ge31',  phy_addr=0x28, prim_offset=0x1907, leds=None),

        _gige(label="33",  hw_port='ge32',  phy_addr=0x2a, prim_offset=0x2100, leds=None),
        _gige(label="34",  hw_port='ge33',  phy_addr=0x2b, prim_offset=0x2101, leds=None),
        _gige(label="35",  hw_port='ge34',  phy_addr=0x2c, prim_offset=0x2102, leds=None),
        _gige(label="36",  hw_port='ge35',  phy_addr=0x2d, prim_offset=0x2103, leds=None),
        _gige(label="37",  hw_port='ge36',  phy_addr=0x2e, prim_offset=0x2104, leds=None),
        _gige(label="38",  hw_port='ge37',  phy_addr=0x2f, prim_offset=0x2105, leds=None),
        _gige(label="39",  hw_port='ge38',  phy_addr=0x30, prim_offset=0x2106, leds=None),
        _gige(label="40",  hw_port='ge39',  phy_addr=0x31, prim_offset=0x2107, leds=None),

        _gige(label="41",  hw_port='ge40',  phy_addr=0x33, prim_offset=0x2900, leds=None),
        _gige(label="42",  hw_port='ge41',  phy_addr=0x34, prim_offset=0x2901, leds=None),
        _gige(label="43",  hw_port='ge42',  phy_addr=0x35, prim_offset=0x2902, leds=None),
        _gige(label="44",  hw_port='ge43',  phy_addr=0x36, prim_offset=0x2903, leds=None),
        _gige(label="45",  hw_port='ge44',  phy_addr=0x37, prim_offset=0x2904, leds=None),
        _gige(label="46",  hw_port='ge45',  phy_addr=0x38, prim_offset=0x2905, leds=None),
        _gige(label="47",  hw_port='ge46',  phy_addr=0x39, prim_offset=0x2906, leds=None),
        _gige(label="48",  hw_port='ge47',  phy_addr=0x3a, prim_offset=0x2907, leds=None),

        _sfp(label="49",   hw_port="hg2",   phy_addr=0x44, leds=(0,235)),
        _sfp(label="50",   hw_port="hg3",   phy_addr=0x45, leds=(0,232)),
        _sfp(label="51",   hw_port="hg0",   phy_addr=0x46, leds=(0,241)),
        _sfp(label="52",   hw_port="hg1",   phy_addr=0x47, leds=(0,238)),
    )


class DNI3048UPCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/0-0028'
    types = { 'port_raw_ctrl' : 'L' }

class DNI3048UPPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class DNI3048UPPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class DNI3048UPTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = "Close to MAC sensor"
    driver_path = '/sys/bus/i2c/devices/0-0049'
    driver_hwmon = ['temp1']
    temp_max_hyst = 50
    temp_max = 75
    temp_crit = 105

class DNI3048UPTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = "Close to PHY sensor"
    driver_path = '/sys/bus/i2c/devices/0-004a'
    driver_hwmon = ['temp1']
    temp_max_hyst = 50
    temp_max = 75
    temp_crit = 100

class DNI3048UP_H4Temp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_fan_min = 50
    temp_fan_max = 75
    temp_max     = 85
    temp_crit    = 105

class DNI3048UP_H4Temp1(DNI3048UP_H4Temp):
    name = 'Temp3'
    driver_hwmon = ['temp1']
    
class DNI3048UP_H4Temp2(DNI3048UP_H4Temp):
    name = 'Temp4'
    driver_hwmon = ['temp2']

class DNI3048UP_H4Temp3(DNI3048UP_H4Temp):
    name = 'Temp5'
    driver_hwmon = ['temp3']

class DNI3048UP_POE(cumulus.sensors.Temp_Unit):
    driver_path = '/var/run/cumulus/poe/sensors'
    hwmon_search = False
    temp_max = 120
    temp_crit = 150

class DNI3048UP_POETemp1(DNI3048UP_POE):
    name = 'Temp6'
    description = 'POE device 1'
    driver_hwmon = ['temp1']

class DNI3048UP_POETemp2(DNI3048UP_POE):
    name = 'Temp7'
    description = 'POE device 2'
    driver_hwmon = ['temp2']

class DNI3048UP_POETemp3(DNI3048UP_POE):
    name = 'Temp8'
    description = 'POE device 3'
    driver_hwmon = ['temp3']

class DNI3048UP_POETemp4(DNI3048UP_POE):
    name = 'Temp9'
    description = 'POE device 4'
    driver_hwmon = ['temp4']

class DNI3048UP_POETemp5(DNI3048UP_POE):
    name = 'Temp10'
    description = 'POE device 5'
    driver_hwmon = ['temp5']

class DNI3048UP_POETemp6(DNI3048UP_POE):
    name = 'Temp11'
    description = 'POE device 6'
    driver_hwmon = ['temp6']

class DNI3048UP_POETemp7(DNI3048UP_POE):
    name = 'Temp12'
    description = 'POE device 7'
    driver_hwmon = ['temp7']

class DNI3048UP_POETemp8(DNI3048UP_POE):
    name = 'Temp13'
    description = 'POE device 8'
    driver_hwmon = ['temp8']

class DNI3048UPFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    driver_path = '/sys/bus/i2c/devices/1-002c'
    cpld_hwmon = ['fan_tray']
    driver_hwmon = ['fan1']
    rpmmode = 1
    fan_min = 7000
    fan_max = 12000

class DNI3048UPFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    driver_path = '/sys/bus/i2c/devices/1-002c'
    cpld_hwmon = ['fan_tray']
    driver_hwmon = ['fan2']
    rpmmode = 1
    fan_min = 7000
    fan_max = 12000

class DNI3048UPSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    bad_led_color = 'red_blinking'

class DNI3048UPFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan'
    bad_led_color = 'red'

class DNI3048UPPSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'
    bad_led_color = 'green_blinking'

class DNI3048UPPSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'
    bad_led_color = 'green_blinking'

class DNI3048UPTempLED(cumulus.sensors.StatusLED_Unit):
    name = 'Temp'
    sources = ['Temp1', 'Temp2']
    led_location = 'led_temp'
    bad_led_color = 'red'

class DNI3048UPPWM1(cumulus.sensors.PWM_Unit):
    fans = (DNI3048UPFan1(),
            DNI3048UPFan2(),
            )
    temps = (DNI3048UPTemp1(),
             DNI3048UPTemp2(),
             DNI3048UP_H4Temp1(),
             DNI3048UP_H4Temp2(),
             DNI3048UP_H4Temp3(),
             DNI3048UP_POETemp1(),
             DNI3048UP_POETemp2(),
             DNI3048UP_POETemp3(),
             DNI3048UP_POETemp4(),
             DNI3048UP_POETemp5(),
             DNI3048UP_POETemp6(),
             DNI3048UP_POETemp7(),
             DNI3048UP_POETemp8(),
             )

class DNI3048UPSensors():
    units = ( DNI3048UPPSU1(),
              DNI3048UPPSU2(),
              DNI3048UPFan1(),
              DNI3048UPFan2(),
              DNI3048UPTemp1(),
              DNI3048UPTemp2(),
              DNI3048UP_H4Temp1(),
              DNI3048UP_H4Temp2(),
              DNI3048UP_H4Temp3(),
              DNI3048UP_POETemp1(),
              DNI3048UP_POETemp2(),
              DNI3048UP_POETemp3(),
              DNI3048UP_POETemp4(),
              DNI3048UP_POETemp5(),
              DNI3048UP_POETemp6(),
              DNI3048UP_POETemp7(),
              DNI3048UP_POETemp8(),
              )

class DNI3048UPPWMs():
    units = (DNI3048UPPWM1(),
             )

class DNI3048UPLEDs():
    units = (DNI3048UPSystemLED(),
             DNI3048UPFanLED(),
             DNI3048UPPSU1LED(),
             DNI3048UPPSU2LED(),
             DNI3048UPTempLED(),
             )

class DNI3048UP(cumulus.platform.Platform):
    name = 'dni,3048up'
    cpld = DNI3048UPCPLD()
    chip = cumulus.platform.Helix4Chip()
    switch = DNI3048UPSwitch(chip)
    pwms = DNI3048UPPWMs()
    sensors = DNI3048UPSensors()
    statusleds = DNI3048UPLEDs()

Platform = DNI3048UP
