#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DNI7448QSFPp(cumulus.platform.Trident40GPort):
    _set_led = '''
; DNI7448QSFPp - single led for activity and link
set_led:
        port	a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv                    ; LEDs are active low
        pack

        ret
'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - activity (TX _OR_ RX)
;;;   register B[1] - link enabled

SUB_INTF_ACTIVITY	equ	0x0  ; activity
SUB_INTF_LINKEN		equ	0x1  ; link enabled

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; DNI7448QSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Two LED bits per port: yellow(active low), green(active low)
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:
        ;;; initialize B
        ld	b,0

        ;;; load sub-0 info
        call	sub_intf_info

        ;;; load sub-1 info
        inc	a
        call	sub_intf_info

        ;;; load sub-2 info
        inc	a
        call	sub_intf_info

        ;;; load sub-3 info
        inc	a
        call	sub_intf_info

        ;;; push accumlated link activity onto stack
        tst	B, SUB_INTF_ACTIVITY
        push	cy

        ;;; combine with BLINK state
        push	(BLINK_STATE_P)
        tand
        tinv	; flash off with activity

        ;;; push accumlated link enable onto stack
        tst	B, SUB_INTF_LINKEN
        push	cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand

        ;;; LEDs are active low
        tinv
        pack

        ret
'''
    def __init__(self, label, serdes, phy_addrs, leds):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=("84740",) * 4,
                                                 phy_addrs=phy_addrs,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addrs[0], 'qsfp+')
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}


        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.hw_port_num(mode, 0)
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call

        return set_calls

class DNI7448SFPp(cumulus.platform.Trident10GPort):
    _set_led = '''
; DNI7448SFPp - single led for activity and link
set_led:
        port    a

        ld      b,LINKSCAN_P
        add     b,a
        ld      b,(b)
        tst     b,LINKSCAN_TX
        push    cy
        tst     b,LINKSCAN_RX
        push    cy
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv                    ; LEDs are active low
        pack

        ret

'''

    def __init__(self, label, serdes, phy_addr, leds):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type="84754",
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addr, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call

        return set_calls

class DNI7448CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ffe05000.localbus/ffdf0000.cpld'

class DNI7448LED(cumulus.ledup.TridentLEDController):
    num_leds = 36

class DNI7448Switch(cumulus.platform.BCMSwitch):
    ledup = DNI7448LED
    _sfp = DNI7448SFPp
    _qsfp = DNI7448QSFPp
    has_phy_ucode_spi_rom = True
    has_phy_ucode_mdio = True
    ports = (
        _sfp(label="1",  serdes=0,  phy_addr=0x4,  leds=(0, 0)),
        _sfp(label="2",  serdes=1,  phy_addr=0x5,  leds=(0, 1)),
        _sfp(label="3",  serdes=2,  phy_addr=0x6,  leds=(0, 2)),
        _sfp(label="4",  serdes=3,  phy_addr=0x7,  leds=(0, 3)),
        _sfp(label="5",  serdes=4,  phy_addr=0x8,  leds=(0, 4)),
        _sfp(label="6",  serdes=5,  phy_addr=0x9,  leds=(0, 5)),
        _sfp(label="7",  serdes=6,  phy_addr=0xa,  leds=(0, 6)),
        _sfp(label="8",  serdes=7,  phy_addr=0xb,  leds=(0, 7)),
        _sfp(label="9",  serdes=8,  phy_addr=0xc,  leds=(0, 8)),
        _sfp(label="10", serdes=9,  phy_addr=0xd,  leds=(0, 9)),
        _sfp(label="11", serdes=10, phy_addr=0xe,  leds=(0, 10)),
        _sfp(label="12", serdes=11, phy_addr=0xf,  leds=(0, 11)),
        _sfp(label="13", serdes=12, phy_addr=0x10, leds=(0, 12)),
        _sfp(label="14", serdes=13, phy_addr=0x11, leds=(0, 13)),
        _sfp(label="15", serdes=14, phy_addr=0x12, leds=(0, 14)),
        _sfp(label="16", serdes=15, phy_addr=0x13, leds=(0, 15)),
        _sfp(label="17", serdes=20, phy_addr=0x14, leds=(0, 20)),
        _sfp(label="18", serdes=21, phy_addr=0x15, leds=(0, 21)),
        _sfp(label="19", serdes=22, phy_addr=0x16, leds=(0, 22)),
        _sfp(label="20", serdes=23, phy_addr=0x17, leds=(0, 23)),
        _sfp(label="21", serdes=24, phy_addr=0x18, leds=(0, 24)),
        _sfp(label="22", serdes=25, phy_addr=0x19, leds=(0, 25)),
        _sfp(label="23", serdes=26, phy_addr=0x1a, leds=(0, 26)),
        _sfp(label="24", serdes=27, phy_addr=0x1b, leds=(0, 27)),
        _sfp(label="25", serdes=28, phy_addr=0x24, leds=(0, 28)),
        _sfp(label="26", serdes=29, phy_addr=0x25, leds=(0, 29)),
        _sfp(label="27", serdes=30, phy_addr=0x26, leds=(0, 30)),
        _sfp(label="28", serdes=31, phy_addr=0x27, leds=(0, 31)),
        _sfp(label="29", serdes=32, phy_addr=0x28, leds=(0, 32)),
        _sfp(label="30", serdes=33, phy_addr=0x29, leds=(0, 33)),
        _sfp(label="31", serdes=34, phy_addr=0x2a, leds=(0, 34)),
        _sfp(label="32", serdes=35, phy_addr=0x2b, leds=(0, 35)),
        _sfp(label="33", serdes=36, phy_addr=0x2c, leds=(1, 0)),
        _sfp(label="34", serdes=37, phy_addr=0x2d, leds=(1, 1)),
        _sfp(label="35", serdes=38, phy_addr=0x2e, leds=(1, 2)),
        _sfp(label="36", serdes=39, phy_addr=0x2f, leds=(1, 3)),
        _sfp(label="37", serdes=40, phy_addr=0x30, leds=(1, 4)),
        _sfp(label="38", serdes=41, phy_addr=0x31, leds=(1, 5)),
        _sfp(label="39", serdes=42, phy_addr=0x32, leds=(1, 6)),
        _sfp(label="40", serdes=43, phy_addr=0x33, leds=(1, 7)),
        _sfp(label="41", serdes=44, phy_addr=0x34, leds=(1, 8)),
        _sfp(label="42", serdes=45, phy_addr=0x35, leds=(1, 9)),
        _sfp(label="43", serdes=46, phy_addr=0x36, leds=(1, 10)),
        _sfp(label="44", serdes=47, phy_addr=0x37, leds=(1, 11)),
        _sfp(label="45", serdes=48, phy_addr=0x38, leds=(1, 12)),
        _sfp(label="46", serdes=49, phy_addr=0x39, leds=(1, 13)),
        _sfp(label="47", serdes=50, phy_addr=0x3a, leds=(1, 14)),
        _sfp(label="48", serdes=51, phy_addr=0x3b, leds=(1, 15)),

        _qsfp(label="49", serdes=(68,69,70,71), phy_addrs=(0x50,0x51,0x52,0x53), leds=(1,32)),
        _qsfp(label="50", serdes=(56,57,58,59), phy_addrs=(0x44,0x45,0x46,0x47), leds=(1,20)),
        _qsfp(label="51", serdes=(64,65,66,67), phy_addrs=(0x4c,0x4d,0x4e,0x4f), leds=(1,28)),
        _qsfp(label="52", serdes=(60,61,62,63), phy_addrs=(0x48,0x49,0x4a,0x4b), leds=(1,24)),
    )

class DNI7448PSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0042'
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']

class DNI7448PSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0040'
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']

class DNI7448Board(cumulus.sensors.Board_Unit):
    name = 'Board'
    description = 'S4810 CPLD Board'
    cpld_hwmon = ['board']

class DNI7448Fan(cumulus.sensors.Fan_Unit):
    minpwm = 136
    maxpwm = 221
    fan_max = 14500
    fan_min = 3000
    driver_hwmon = ['fan1']

class DNI7448Fan1(DNI7448Fan):
    name = 'Fan1'
    description = 'Tray2, Fan1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-001b'
    cpld_hwmon = ['fan_0']

class DNI7448Fan2(DNI7448Fan):
    name = 'Fan2'
    description = 'Tray2, Fan2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-001f'
    cpld_hwmon = ['fan_0']

class DNI7448Fan3(DNI7448Fan):
    name = 'Fan3'
    description = 'Tray1, Fan1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0048'
    cpld_hwmon = ['fan_1']

class DNI7448Fan4(DNI7448Fan):
    name = 'Fan4'
    description = 'Tray1, Fan2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004b'
    cpld_hwmon = ['fan_1']

class DNI7448Temp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'
    temp_max = 100
    temp_crit = 105

class DNI7448Temp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'CPU Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0049'

class DNI7448Temp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Left PHY Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004c'

class DNI7448Temp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Right PHY Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'

class DNI7448Temp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004a'

class DNI7448PSULED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'

class DNI7448FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    led_location = 'led_fan'

class DNI7448SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_status'
    bad_led_color = 'green_blinking'

class DNI7448FanTray1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan_tray_0'
    bad_led_color = 'red'

class DNI7448FanTray2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'red'

class DNI7448PWM1(cumulus.sensors.PWM_Unit):
    fans = (DNI7448Fan1(),
            DNI7448Fan2(),
            DNI7448Fan3(),
            DNI7448Fan4()
            )
    temps = (DNI7448Temp1(),
             DNI7448Temp2(),
             DNI7448Temp3(),
             DNI7448Temp4(),
             DNI7448Temp5()
             )

class DNI7448Sensors():
    units = ( DNI7448PSU1(),
              DNI7448PSU2(),
              DNI7448Board(),
              DNI7448Fan1(),
              DNI7448Fan2(),
              DNI7448Fan3(),
              DNI7448Fan4(),
              DNI7448Temp1(),
              DNI7448Temp2(),
              DNI7448Temp3(),
              DNI7448Temp4(),
              DNI7448Temp5(),
              )

class DNI7448PWMs():
    units = (DNI7448PWM1(),
             )

class DNI7448LEDs():
    units = (DNI7448PSULED(),
             DNI7448FanLED(),
             DNI7448FanTray1LED(),
             DNI7448FanTray2LED(),
             DNI7448SystemLED(),
             )

class DNI7448(cumulus.platform.Platform):
    name = 'dni,et-7448bf'
    cpld = DNI7448CPLD()
    chip = cumulus.platform.TridentChip()
    switch = DNI7448Switch(chip)
    statusleds = DNI7448LEDs()
    pwms = DNI7448PWMs()
    sensors = DNI7448Sensors()

Platform = DNI7448
