#
# Copyright 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.cpld
import cumulus.platform
import cumulus.sensors
import cumulus.sfp
import cumulus.platforms.mlnx_x86_MSN2700

LPCI2C_BUS_NAME = 'lpci2c'
def MlxSN2740_get_cpld_path():
    for i in range(3):
        try:
            data = open('/sys/bus/i2c/devices/i2c-%u/name' % i, 'r').read().strip()
            if data == LPCI2C_BUS_NAME:
                return '/sys/bus/i2c/devices/%u-0060' % i
        except:
            pass

asic_addr = 0x40
asic_bus = 2
MlxSN2740_cpld_path = MlxSN2740_get_cpld_path()
MlxSN2740_asic_path = '/sys/bus/i2c/devices/%u-00%x' % (asic_bus, asic_addr)
MlxSN2740_swb_temp_path = '/sys/bus/i2c/devices/15-0048'
MlxSN2740_mnb_temp_path = '/sys/bus/i2c/devices/14-0049'
MlxSN2740_core_temp_path = '/sys/devices/platform/coretemp.0'
MlxSN2740_psu1_path = '/sys/bus/i2c/devices/12-0059'
MlxSN2740_psu2_path = '/sys/bus/i2c/devices/12-0058'


class MlxSN2740CPLD(cumulus.cpld.SysFS):
    path = MlxSN2740_cpld_path


# System LEDs
#
class MlxSN2740StatusLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4',
               'PSU1', 'PSU1Fan1', 'PSU1Temp1',
               'PSU2', 'PSU2Fan1', 'PSU2Temp1',
               'Temp1', 'Temp2', 'Temp3',
               'Temp4', 'Temp5', 'Temp6', 'Temp7']
    led_location = 'led_status'
    bad_led_color = 'red'

class MlxSN2740PsuLED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu'
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    bad_led_color = 'red'

class MlxSN2740FanLED(cumulus.sensors.StatusLED_Unit):
    bad_led_color = 'red'

class MlxSN2740Fan1LED(MlxSN2740FanLED):
    name = 'Fan 1'
    sources = ['Fan1']
    led_location = 'led_fan1'

class MlxSN2740Fan2LED(MlxSN2740FanLED):
    name = 'Fan 2'
    sources = ['Fan2']
    led_location = 'led_fan2'

class MlxSN2740Fan3LED(MlxSN2740FanLED):
    name = 'Fan 3'
    sources = ['Fan3']
    led_location = 'led_fan3'

class MlxSN2740Fan4LED(MlxSN2740FanLED):
    name = 'Fan 4'
    sources = ['Fan4']
    led_location = 'led_fan4'

# Base class for power supply unit

# System Fans
#
# Base class for the system fans
#
# we access fans (fanX_input, fanX_enable, pwmX) through the
# mlx-asic-drv driver. This driver relies on the sx_core module
# which takes a while to come up. We keep things in an UNKNOWN
# state until we are able to successfully load the asic driver
# and read status. Once the driver is up, we write '2' to the
# fanX_enable nodes which allows us to control the fans via the
# pwms.
#
# An additional note:
#  - writing to any pwm (pwm1 - pwm4) affects all fans equally.
#
class MlxSN2740Fan(cumulus.sensors.Fan_Unit):
    fan_max = 23500
    fan_min = 12500
    minpwm = 51
    driver_path = MlxSN2740_asic_path
    pwm_path = driver_path

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(self.driver_path, ('%s_input' % self.driver_hwmon[0]))):
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        if not self.init_done:
            if self.asic_write('%s_enable' % self.driver_hwmon[0], '2'):
                self.init_done = True
        return 'OK'

    def asic_write(self, node, val):
        try:
            f = open(os.path.join(self.driver_path, node), 'w')
            f.write(val)
            f.close()
        except:
            return 0
        return 1

class MlxSN2740OddFan(MlxSN2740Fan):
    fan_max = 23000
    fan_min = 10500

#
# system fan #1
#
class MlxSN2740Fan_1(MlxSN2740OddFan):
    name = 'Fan1'
    description = 'Fan 1'
    cpld_hwmon = ['fan1_ok']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    init_done = False

    def __init__(self, asic_drv_name):
        self.asic_drv_name = asic_drv_name
        MlxSN2740Fan.__init__(self)

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(MlxSN2740_asic_path, 'hwmon')):
            if os.path.isdir(MlxSN2740_asic_path):
                f = open('/sys/bus/i2c/devices/i2c-%u/delete_device' % asic_bus, 'w')
                f.write('0x%x' % asic_addr)
                f.close()
            f = open('/sys/bus/i2c/devices/i2c-%u/new_device' % asic_bus, 'w')
            f.write('%s 0x%x' % (self.asic_drv_name, asic_addr))
            f.close()
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        try:
            open(os.path.join(self.driver_path, '%s_input' % self.name.lower()), 'r').read()
            if not self.init_done:
                if self.asic_write('%s_enable' % self.name.lower(), '2'):
                    self.init_done = True
        except:
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        return 'OK'

#
# system fan #2
#
class MlxSN2740Fan_2(MlxSN2740Fan):
    name = 'Fan2'
    description = 'Fan 2'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# system fan #3
#
class MlxSN2740Fan_3(MlxSN2740OddFan):
    name = 'Fan3'
    description = 'Fan 3'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# system fan #4
#
class MlxSN2740Fan_4(MlxSN2740Fan):
    name = 'Fan4'
    description = 'Fan 4'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm1']
    init_done = False
#
# Base class for the PSU Fans
#
class MlxSN2740PSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 20000
    fan_min = 1000

#
# PSU #1 Fan
#
class MlxSN2740PSUFan_1(MlxSN2740PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = MlxSN2740_psu1_path
    driver_hwmon = ['fan1']

#
# PSU #2 Fan
#
class MlxSN2740PSUFan_2(MlxSN2740PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = MlxSN2740_psu2_path
    driver_hwmon = ['fan1']

#
# Base class for the system Temp Sensors
class MlxSN2740Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 95
    temp_crit = 115

#
# system temp #1, CPU Package
#
class MlxSN2740Temp_1(MlxSN2740Temp):
    name = 'Temp1'
    driver_path = MlxSN2740_core_temp_path
    driver_hwmon = ['temp2']
    description = 'CPU Core Sensor 0'

#
# system temp #2, CPU Core 0
#
class MlxSN2740Temp_2(MlxSN2740Temp):
    name = 'Temp2'
    driver_path = MlxSN2740_core_temp_path
    driver_hwmon = ['temp3']
    description = 'CPU Core Sensor 1'

#
# system temp #3, CPU Core 2
#
class MlxSN2740Temp_3(MlxSN2740Temp):
    name = 'Temp3'
    driver_path = MlxSN2740_core_temp_path
    driver_hwmon = ['temp4']
    description = 'CPU Core Sensor 2'

#
# system temp #4, CPU Core 3
#
class MlxSN2740Temp_4(MlxSN2740Temp):
    name = 'Temp4'
    driver_path = MlxSN2740_core_temp_path
    driver_hwmon = ['temp5']
    description = 'CPU Core Sensor 3'

#
# system temp #5, Port Ambient Sensor
#
class MlxSN2740Temp_5(MlxSN2740Temp):
    name = 'Temp5'
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 110
    driver_path = MlxSN2740_swb_temp_path
    driver_hwmon = ['temp1']
    description = 'Port Ambient Sensor'

#
# system temp #6, Main Board Ambient
#
class MlxSN2740Temp_6(MlxSN2740Temp):
    name = 'Temp6'
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 110
    driver_path = MlxSN2740_mnb_temp_path
    driver_hwmon = ['temp1']
    description = 'Main Board Ambient Sensor'

#
# system temp #7, Asic Temp
#
class MlxSN2740Temp_7(MlxSN2740Temp):
    name = 'Temp7'
    temp_fan_min = 80
    temp_max = 100
    temp_crit = 120
    driver_path = MlxSN2740_asic_path
    driver_hwmon = ['temp1']
    description = 'Asic Temp Sensor'

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(MlxSN2740_asic_path, 'hwmon')):
            return 'UNKNOWN'
        return 'OK'

#
# Base class for the PSU temp sensors
#
class MlxSN2740PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 55
    temp_crit = 65
    pwm_hwmon = ['pwm1']
    driver_hwmon = ['temp1']
    cpld_path = MlxSN2740_cpld_path

#
# PSU1 temp sensor
#
class MlxSN2740PSUTemp_1(MlxSN2740PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = MlxSN2740_psu1_path

#
# PSU2 temp sensor
#
class MlxSN2740PSUTemp_2(MlxSN2740PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = MlxSN2740_psu2_path

#
# Base class for the PSUs
#
class MlxSN2740PSU(cumulus.sensors.PSU_Unit):
    cpld_path = MlxSN2740_cpld_path
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'pmbus']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'pmbus']
                subprocess.call(cmd)
            self.init_done = True

#
# PSU #1
#
class MlxSN2740PSU1(MlxSN2740PSU):
    name = 'PSU1'
    driver_path = MlxSN2740_psu1_path
    cpld_hwmon = ['psu_pwr1_present', 'psu_pwr1_all_ok']

#
# PSU #2
#
class MlxSN2740PSU2(MlxSN2740PSU):
    name = 'PSU2'
    driver_path = MlxSN2740_psu2_path
    cpld_hwmon = ['psu_pwr2_present', 'psu_pwr2_all_ok']

class MlxSN2740LEDs():
    units = (
        MlxSN2740StatusLED(),
        MlxSN2740PsuLED(),
        MlxSN2740Fan1LED(),
        MlxSN2740Fan2LED(),
        MlxSN2740Fan3LED(),
        MlxSN2740Fan4LED(),
    )

class MlxSN2740Sensors():

    def __init__(self, asic_drv_name='mlnx-asic-drv'):
        self.units = (MlxSN2740Fan_1(asic_drv_name),
                 MlxSN2740Fan_2(),
                 MlxSN2740Fan_3(),
                 MlxSN2740Fan_4(),
                 MlxSN2740PSUFan_1(),
                 MlxSN2740PSUFan_2(),
                 MlxSN2740Temp_1(),
                 MlxSN2740Temp_2(),
                 MlxSN2740Temp_3(),
                 MlxSN2740Temp_4(),
                 MlxSN2740Temp_5(),
                 MlxSN2740Temp_6(),
                 MlxSN2740Temp_7(),
                 MlxSN2740PSUTemp_1(),
                 MlxSN2740PSUTemp_2(),
                 MlxSN2740PSU1(),
                 MlxSN2740PSU2(),
        )

class MlxSN2740SystemPWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    def __init__(self, asic_drv_name):
        self.fans = (MlxSN2740Fan_1(asic_drv_name),
                MlxSN2740Fan_2(),
                MlxSN2740Fan_3(),
                MlxSN2740Fan_4(),
        )
        self.temps = (
                 MlxSN2740Temp_1(),
                 MlxSN2740Temp_2(),
                 MlxSN2740Temp_3(),
                 MlxSN2740Temp_4(),
                 MlxSN2740Temp_5(),
                 MlxSN2740Temp_6(),
                 MlxSN2740Temp_7(),
        )
        cumulus.sensors.PWM_Unit.__init__(self)

class MlxSN2740PWMs():

    def __init__(self, asic_drv_name='mlnx-asic-drv'):
        self.units = (
            MlxSN2740SystemPWM(asic_drv_name),
        )

class MlxSN2740(cumulus.platform.Platform):
    name = 'mlnx,msn2740'
    chip = cumulus.platform.SpectrumChip()
    switch = cumulus.platforms.mlnx_x86_MSN2700.MlxSN2700Switch(chip)
    cpld = MlxSN2740CPLD()
    sensors = MlxSN2740Sensors()
    pwms = MlxSN2740PWMs()
    statusleds = MlxSN2740LEDs()

Platform = MlxSN2740
