#
# Copyright 2016.  Cumulus Networks, Inc.
#

import os
import cumulus.platform
import cumulus.cpld
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx


def MlxSN2100_get_paths():
    i801_1f_3_path = '/sys/bus/pci/devices/0000:00:1f.3'
    if os.path.exists(i801_1f_3_path):
        for x in os.listdir(i801_1f_3_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                i801_1f_3_path = os.path.join(i801_1f_3_path, x, b)
                break

    ismt_0_13_0_path = '/sys/bus/pci/devices/0000:00:13.0'
    ismt_0_13_0_full_path = ''
    ismt_0_13_0_base_path = ''
    if os.path.exists(ismt_0_13_0_path):
        for x in os.listdir(ismt_0_13_0_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                ismt_0_13_0_base_path = os.path.join(ismt_0_13_0_path, x)
                ismt_0_13_0_full_path = os.path.join(ismt_0_13_0_base_path, b)
                break

    return i801_1f_3_path, ismt_0_13_0_base_path, ismt_0_13_0_full_path

MLX_ASIC_ADDR = '0x48'
MlxSN2100_i801_path, MlxSN2100_ismt_base_path, MlxSN2100_ismt_path = MlxSN2100_get_paths()
MlxSN2100_dimm1_path = MlxSN2100_i801_path + '-0018'
MlxSN2100_cpld_path = MlxSN2100_ismt_path + '-0060'
MlxSN2100_asic_path = MlxSN2100_ismt_path + '-0048'
MlxSN2100_port_temp_path = '/sys/bus/i2c/devices/6-004a'
MlxSN2100_board_temp_path = '/sys/bus/i2c/devices/6-004b'
MlxSN2100_core_temp_path = '/sys/devices/platform/coretemp.0'


# System LEDs
#
class MlxSN2100StatusLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4',
               'PSU1', 'PSU2',
               'Temp1', 'Temp2', 'Temp3', 'Temp4',
               'Temp5', 'Temp6', 'Temp7', 'Temp8']
    led_location = 'led_status'
    bad_led_color = 'red'

class MlxSN2100Psu1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu1'
    led_location = 'led_psu1'
    sources = ['PSU1']
    bad_led_color = 'red'

class MlxSN2100Psu2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu2'
    led_location = 'led_psu2'
    sources = ['PSU2']
    bad_led_color = 'red'

class MlxSN2100FanLED(cumulus.sensors.StatusLED_Unit):
    bad_led_color = 'red'
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    led_location = 'led_fan'


# System Fans
#
# Base class for the system fans
#
# we access fans (fanX_input, fanX_enable, pwmX) through the
# mlx-asic-drv driver. This driver relies on the sx_core module
# which takes a while to come up. We keep things in an UNKNOWN
# state until we are able to successfully load the asic driver
# and read status. Once the driver is up, we write '2' to the
# fanX_enable nodes which allows us to control the fans via the
# pwms.
#
# writing to any pwm (pwm1 - pwm4) affects all fans equally.
#
class MlxSN2100Fan(cumulus.sensors.Fan_Unit):
    fan_max = 23000
    fan_min = 15000
    minpwm = 77
    driver_path = MlxSN2100_asic_path
    pwm_path = driver_path

    # it takes some time for the asic driver to load and give us our fan paths,
    # we do this check so we prevent a transition to BAD before going to OK.
    # That is, we do: UNKNOWN => OK, rather than UNKNOWN => BAD => OK
    def detect_state_plat(self):
        if not os.path.exists(os.path.join(self.driver_path, ('%s_input' % self.driver_hwmon[0]))):
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        if not self.init_done:
            if self.asic_write('%s_enable' % self.driver_hwmon[0], '2'):
                self.init_done = True
        return 'OK'

    # asic driver might not be ready for our write yet so silently fail
    # if we get an error
    def asic_write(self, node, val):
        try:
            f = open(os.path.join(self.driver_path, node), 'w')
            f.write(val)
            f.close()
        except:
            return 0
        return 1

#
# system fan #1
#
class MlxSN2100Fan_1(MlxSN2100Fan):
    name = 'Fan1'
    description = 'Fan 1'
    cpld_hwmon = ['fan1_ok']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    init_done = False

    def __init__(self):
        self.asic_drv_name = 'mlnx-asic-drv-16'
        MlxSN2100Fan.__init__(self)

    def detect_state_plat(self):
        # we need to check to see if the asic driver loaded properly
        # (it is dependent on sx_core initializing)
        if not os.path.exists(os.path.join(MlxSN2100_asic_path, 'hwmon')):
            # asic driver did not create 'hwmon' path, remove and try again
            if os.path.isdir(MlxSN2100_asic_path):
                try:
                    f = open(os.path.join(MlxSN2100_ismt_base_path, 'delete_device'), 'w')
                    f.write(MLX_ASIC_ADDR)
                    f.close()
                except:
                    pass
            try:
                f = open(os.path.join(MlxSN2100_ismt_base_path, 'new_device'), 'w')
                f.write('%s %s' % (self.asic_drv_name, MLX_ASIC_ADDR))
                f.close()
            except:
                pass
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        try:
            open(os.path.join(self.driver_path, '%s_input' % self.name.lower()), 'r').read()
            if not self.init_done:
                if self.asic_write('%s_enable' % self.name.lower(), '2'):
                    self.init_done = True
        except:
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        return 'OK'

#
# system fan #2
#
class MlxSN2100Fan_2(MlxSN2100Fan):
    name = 'Fan2'
    description = 'Fan 2'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# system fan #3
#
class MlxSN2100Fan_3(MlxSN2100Fan):
    name = 'Fan3'
    description = 'Fan 3'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# system fan #4
#
class MlxSN2100Fan_4(MlxSN2100Fan):
    name = 'Fan4'
    description = 'Fan 4'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# Base class for the system Temp Sensors
class MlxSN2100Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 80
    temp_max = 100
    temp_crit = 120

class MlxSN2100AmbientTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 100

class MlxSN2100CoreTemp(MlxSN2100Temp):
    driver_path = MlxSN2100_core_temp_path

#
# system temp sensor #1, DIMM1
#
class MlxSN2100Temp_1(MlxSN2100Temp):
    name = 'Temp1'
    temp_fan_min = 65
    temp_max = 85
    temp_crit = 95
    driver_path = MlxSN2100_dimm1_path
    driver_hwmon = ['temp1']
    description = 'DIMM1 Temperature Sensor'

#
# system temp sensor #2, MLNX ASIC
#
class MlxSN2100Temp_2(MlxSN2100Temp):
    name = 'Temp2'
    driver_path = MlxSN2100_asic_path
    driver_hwmon = ['temp1']
    description = 'ASIC Temperature Sensor'

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(MlxSN2100_asic_path, 'hwmon')):
            return 'UNKNOWN'
        return 'OK'

#
# system temp #3, Port Ambient Sensor
#
class MlxSN2100Temp_3(MlxSN2100AmbientTemp):
    name = 'Temp3'
    driver_path = MlxSN2100_port_temp_path
    driver_hwmon = ['temp1']
    description = 'Port Ambient Sensor'

#
# system temp #4, Main Board Ambient Sensor
#
class MlxSN2100Temp_4(MlxSN2100AmbientTemp):
    name = 'Temp4'
    driver_path = MlxSN2100_board_temp_path
    driver_hwmon = ['temp1']
    description = 'Main Board Ambient Sensor'

#
# system temp #5, core0
#
class MlxSN2100Temp_5(MlxSN2100CoreTemp):
    name = 'Temp5'
    description = 'core0 Sensor'
    driver_hwmon = ['temp2']

#
# system temp #6, core1
#
class MlxSN2100Temp_6(MlxSN2100CoreTemp):
    name = 'Temp6'
    description = 'core1 Sensor'
    driver_hwmon = ['temp3']

#
# system temp #7, core2
#
class MlxSN2100Temp_7(MlxSN2100CoreTemp):
    name = 'Temp7'
    description = 'core2 Sensor'
    driver_hwmon = ['temp4']

#
# system temp #8, core3
#
class MlxSN2100Temp_8(MlxSN2100CoreTemp):
    name = 'Temp8'
    description = 'core3 Sensor'
    driver_hwmon = ['temp5']

#
# Base class for the PSUs
#
class MlxSN2100PSU(cumulus.sensors.PSU_Unit):
    cpld_path = MlxSN2100_cpld_path

#
# PSU #1
#
class MlxSN2100PSU1(MlxSN2100PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1', 'psu_pwr1']

#
# PSU #2
#
class MlxSN2100PSU2(MlxSN2100PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2', 'psu_pwr2']

class MlxSN2100LEDs():
    units = (
        MlxSN2100StatusLED(),
        MlxSN2100Psu1LED(),
        MlxSN2100Psu2LED(),
        MlxSN2100FanLED(),
    )

class MlxSN2100Sensors():

    def __init__(self):
        self.units = (
                 MlxSN2100Fan_1(),
                 MlxSN2100Fan_2(),
                 MlxSN2100Fan_3(),
                 MlxSN2100Fan_4(),
                 MlxSN2100Temp_1(),
                 MlxSN2100Temp_2(),
                 MlxSN2100Temp_3(),
                 MlxSN2100Temp_4(),
                 MlxSN2100Temp_5(),
                 MlxSN2100Temp_6(),
                 MlxSN2100Temp_7(),
                 MlxSN2100Temp_8(),
                 MlxSN2100PSU1(),
                 MlxSN2100PSU2(),
        )

class MlxSN2100SystemPWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    def __init__(self):
        self.fans = (
                MlxSN2100Fan_1(),
                MlxSN2100Fan_2(),
                MlxSN2100Fan_3(),
                MlxSN2100Fan_4(),
        )
        self.temps = (
                 MlxSN2100Temp_1(),
                 MlxSN2100Temp_2(),
                 MlxSN2100Temp_3(),
                 MlxSN2100Temp_4(),
                 MlxSN2100Temp_5(),
                 MlxSN2100Temp_6(),
                 MlxSN2100Temp_7(),
                 MlxSN2100Temp_8(),
        )
        cumulus.sensors.PWM_Unit.__init__(self)

class MlxSN2100PWMs():

    def __init__(self):
        self.units = (
            MlxSN2100SystemPWM(),
        )

class MlxSN2100QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSN2100QSFP28
;;;  port LEDs controlled by HW
'''
    def __init__(self, label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=None, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2100Switch(cumulus.platform.SpectrumSwitch):
    _sfp = None
    _qsfp = MlxSN2100QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="1",  phy_addrs=(0,0,0,0),     hw_ports=(25,26,27,28)),
        _qsfp(label="2",  phy_addrs=(1,1,1,1),     hw_ports=(29,30,31,32)),
        _qsfp(label="3",  phy_addrs=(2,2,2,2),     hw_ports=(17,18,19,20)),
        _qsfp(label="4",  phy_addrs=(3,3,3,3),     hw_ports=(21,22,23,24)),
        _qsfp(label="5",  phy_addrs=(4,4,4,4),     hw_ports=( 9,10,11,12)),
        _qsfp(label="6",  phy_addrs=(5,5,5,5),     hw_ports=(13,14,15,16)),
        _qsfp(label="7",  phy_addrs=(6,6,6,6),     hw_ports=(1,2,3,4)),
        _qsfp(label="8",  phy_addrs=(7,7,7,7),     hw_ports=(5,6,7,8)),
        _qsfp(label="9",  phy_addrs=(8,8,8,8),     hw_ports=(37,38,39,40)),
        _qsfp(label="10", phy_addrs=(9,9,9,9),     hw_ports=(33,34,35,36)),
        _qsfp(label="11", phy_addrs=(10,10,10,10), hw_ports=(45,46,47,48)),
        _qsfp(label="12", phy_addrs=(11,11,11,11), hw_ports=(41,42,43,44)),
        _qsfp(label="13", phy_addrs=(12,12,12,12), hw_ports=(53,54,55,56)),
        _qsfp(label="14", phy_addrs=(13,13,13,13), hw_ports=(49,50,51,52)),
        _qsfp(label="15", phy_addrs=(14,14,14,14), hw_ports=(61,62,63,64)),
        _qsfp(label="16", phy_addrs=(15,15,15,15), hw_ports=(57,58,59,60)),
    )

class MlxSN2100(cumulus.platform.Platform):
    name = 'mlnx,x86_MSN2100'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2100Switch(cumulus.platform.SpectrumChip())
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = MlxSN2100Sensors()
    pwms = MlxSN2100PWMs()
    statusleds = MlxSN2100LEDs()

Platform = MlxSN2100
