#
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018 Cumulus Networks, Inc. all rights reserved
#
# This software is subject to the Cumulus Networks End User License
# Agreement available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt
#
# portconfig.py --
#     Generate port configuration fragments.
#
import ConfigParser
import StringIO
import sys
import subprocess
import syslog
from itertools import groupby
import os
import shutil

import sfp
import json
import os

import cumulus.portvalidator

class DecodeSysEEPROM:
    '''
    Helper for decode-syseeprom.  Supports a dummy mode for testing.
    '''
    def __init__(self, dummy=False, sw_base=None):
        self.dummy = dummy
        self.sw_base = sw_base

    @property
    def macaddr(self):

        if self.dummy:
            return 'ff:ff:ff:ff:ff:00'
        else:
            try:
                return subprocess.check_output(('/usr/cumulus/bin/decode-syseeprom', '-a'))
            except:
                # no traceback (we are reporting). Also syslog because it's important
                sys.tracebacklimit = 0
                syslog.syslog(syslog.LOG_ERR, 'error: decode-syseeprom -a failed')
                raise RuntimeError('error: decode-syseeprom -a failed')

    @property
    def macrange(self):

        if self.dummy:
            mrange = 0xff
        else:
            try:
                mrange = subprocess.check_output(('/usr/cumulus/bin/decode-syseeprom', '-r'))
            except:
                # no traceback (we are reporting). Also syslog because it's important
                sys.tracebacklimit = 0
                syslog.syslog(syslog.LOG_ERR, 'error: decode-syseeprom -r failed')
                raise RuntimeError('error: decode-syseeprom -r failed')

        try:
            mrange = int(mrange)
        except ValueError:
            # no traceback (we are reporting). Also syslog because it's important
            sys.tracebacklimit = 0
            syslog.syslog(syslog.LOG_ERR, 'error: decode-syseeprom -r returned bogus value: %s' % mrange)
            raise RuntimeError('error: decode-syseeprom -r returned bogus value: %s' % mrange)

        return mrange

class IPRoute2:
    '''
    Helper for iproute2.  Supports a dummy mode for testing.
    '''
    def __init__(self, dummy=False):
        self.dummy = dummy

    def run(self, args):
        if self.dummy:
            return str()
        else:
            return subprocess.check_output([ 'ip' ] + args)

class SWPort:
    '''
    Helper for swport.  Supports a dummy mode for testing.
    '''
    def __init__(self, dummy=False):
        self.dummy = dummy

    def run(self, args):
        if self.dummy:
            return str()
        else:
            return subprocess.check_output([ '/usr/lib/cumulus/swport' ] + args)

class Config:
    '''
    Base class for port configuration.  Understands ports.conf format.
    Supports a dummy mode for testing.
    '''
    def __init__(self, platform, verbose=False, exploded=False, dummy=False):
        self.platform = platform
        self.verbose = verbose
        self.exploded = exploded
        self.dummy = dummy

        self.eeprom = DecodeSysEEPROM(dummy=dummy, sw_base=self.platform.switch.chip.sw_base)
        self.iproute2 = IPRoute2(dummy=dummy)
        self.swport = SWPort(dummy=dummy)

        self.ports = []
        self.macbase = []
        self.macrange = self.eeprom.macrange
        for byte in self.eeprom.macaddr.split(':'):
            self.macbase.append(int(byte, 16))
        self.macmode = 'standard'

    def config(self, config):
        '''
        Read a ports.conf config from a string

        configfile - config file string
        '''

        # add a default section so we can use ConfigParser
        configStr = '[ports]\n' + config
        configFP = StringIO.StringIO(configStr)

        parser = ConfigParser.RawConfigParser()
        parser.optionxform = str
        parser.readfp(configFP)

        self.config_map = dict(parser.items('ports'))
        self.config_map = self._complete_modes(self.config_map)

    def _complete_modes(self, config_map):
        '''
        We want to support setting the breakout configuration without an
        associated speed.  Therefore, complete the shorthand '1x', '/1' etc.
        into a fully qualified breakout + speed setting.
        '''
        for label in config_map:
            hwport = self.find_hw_port(label)
            if hwport is not None:
                config_map[label] = hwport.complete_mode(hwport, config_map[label])
        return config_map

    def find_hw_port(self, label):
        '''
        Find a platform port based upon the port's label.
        '''
        for hwport in self.platform.switch.ports:
            if hwport.label == label and not hwport.is_fabric:
                return hwport
        return None

    def read_config(self, configfile):
        '''
        Validate and read a ports.conf config file and store its key/value
        pairs in a dict.

        configfile - config file path
        '''

        if configfile == 'hw_desc' :
            self.hw_desc = self.platform.switch.chip.get_config(configfile)

        elif configfile == 'rc.forwarding' :
            self.rc_forwarding = self.platform.switch.chip.get_config(configfile)

        elif configfile == 'datapath.conf' :
            self.datapath_conf = self.platform.switch.chip.get_config(configfile)

        elif configfile == 'sdk.bcm' :
            self.sdk_bcm = self.platform.switch.chip.get_config(configfile)

        elif configfile == 'uft.json' :
            self.uft = self.platform.switch.chip.get_config(configfile)

        elif configfile == 'riot.json' :
            self.riot = self.platform.switch.chip.get_config(configfile)

        else :
            pv = cumulus.portvalidator.Portvalidator(configfile, debug=False)
            retlist = pv.portvalidator()
            if len(retlist) > 0 :
                pv.portwriteerrors(retlist)
                sys.exit(-1)
            self.config(open(configfile, 'r').read())

    def init_config(self):
        bcmd = '/etc/bcm.d'
        if not os.path.isdir(bcmd):
            os.mkdir(bcmd)

        shutil.copy(os.path.join(self.platform.switch.chip.bcm_config_path,
                                 'rc.soc'), bcmd)

        self.config_map = {}
        for port in self.platform.switch.ports:
            label = "Fabric-" + port.label if port.is_fabric else port.label
            self.config_map[label] = port.default_mode

    def write_config(self, configfile):
        cf = open(configfile, 'w')
        cf.write('# XXX - default config file (needs documentation comments)\n')
        output_keys = [k for k in self.config_map.keys() if not k.startswith("Fabric-")]
        for port in sorted(output_keys, key=lambda x: int(x)):
            cf.write('%s=%s\n' % (port, self.config_map[port]))

    def set_macmode(self, mode):
        '''
        Set the default mac mode for a configuration
        '''
        self.macmode = mode

    def makemacaddr(self, inmac, index):

        if inmac is None:
            return None

        curmac = [0] * 6
        # Increment the MAC (only the non-OUI part).
        ext = (inmac[5] | (inmac[4] << 8) | (inmac[3] << 16))
        ext += index
        if (ext & 0xff000000):
            sys.stdout.write('Tried to carry into OUI.\n')
            return 0
        curmac[5] = ext & 0xff
        curmac[4] = (ext >> 8) & 0xff
        curmac[3] = (ext >> 16) & 0xff
        curmac[2] = inmac[2]
        curmac[1] = inmac[1]
        curmac[0] = inmac[0]
        return curmac

    def destroyints(self, interface_set):
        for i in interface_set:
            if 'switch' not in i:
                cmd = 'link del dev %s' % i
                self.iproute2.run(cmd.split())
        interface_set.clear()

    def setmac(self, portname, index):
        if self.macmode == 'l3-only' :
            index = 0
        themac = self.makemacaddr(self.macbase, index)
        if themac is not None:
            cmd = 'link set %s down' % portname
            self.iproute2.run(cmd.split())
            cmd = 'link set dev %s address %x:%x:%x:%x:%x:%x' % \
                   (portname, themac[0], themac[1], themac[2], themac[3],
                    themac[4], themac[5])
            self.iproute2.run(cmd.split())

    def maketap(self, portname, index, interface_set):
        if portname in interface_set:
            interface_set.remove(portname)
        else:
            if self.macmode == 'l3-only' :
                index = 0
            themac = self.makemacaddr(self.macbase, index)
            if themac is not None:
                cmd = 'tuntap add dev %s mode tap one_queue' % portname
                self.iproute2.run(cmd.split())
                self.setmac(portname, index)

    def makekernnet(self, portname, index, unit, interface_set):
        if self.dummy:
            return

        if portname in interface_set:
            interface_set.remove(portname)
        else:
            if self.macmode == 'l3-only' :
                index = 0
            themac = self.makemacaddr(self.macbase, index)
            if themac is not None:
                platform = self.platform.switch.chip.sw_base
                if platform not in ["bcm", "mlx"]:
                    raise UnsupportedPlatformError("Kernel netdevs are not supported \
                          on %s, only on Broadcom and Mellanox" % platform)
                link_type = "bcm_knet" if platform == "bcm" else "mlx_sx_netdev"
                cmd = 'add name %s type %s unit %s' %(portname, link_type, unit)
                self.swport.run(cmd.split())
                self.setmac(portname, index)

        if self.platform.switch.chip.sw_base == 'bcm':
            with open('/proc/bcm/knet/link', 'w') as outfile:
                outfile.write(("%s=down" % (portname)))

class ConfiguredPort:
    portmap_capable = None

    '''
    Represents a port configured in a specific mode.
    '''
    def __init__(self, mode, mac_index, loopback):
        self.mode = mode
        self.mac_index = mac_index
        self.loopback = loopback

    @property
    def label(self):
        '''
        The text printed on the front of the switch.
        '''
        raise NotImplementedError

    @property
    def speed(self):
        '''
        The port speed in gigabits/sec.
        '''
        raise NotImplementedError

    @property
    def num_logical_ports(self):
        '''
        The number of logical ports (swp interfaces) for the port.
        '''
        raise NotImplementedError

    @property
    def max_logical_ports(self):
        '''
        The maximum number of logical ports (swp interfaces) for the port.
        '''
        raise NotImplementedError

    @property
    def leds(self):
        raise NotImplementedError

    @property
    def linux_prefix(self):
        if self.physical_port.is_fabric:
            return 'fp'
        else:
            return 'swp'

    @property
    def gang_label(self):
        '''
        Gang Identifier for Ganged ports
        Port Label for unganged port
        Assumption: Port numbering starts at swp1
        '''
        if self.mode == '40G/4':
            return str((((int(self.label)-1)/4) * 4) + 1)
        else:
            return self.label

    @property
    def serdes_if_type(self):
        raise NotImplementedError

    @property
    def an_c37(self):
        return None

    @property
    def an_c73(self):
        return None

class SDKConfiguredPort(ConfiguredPort):
    portmap_capable = False

    '''
    SDK port configuration implementation.
    '''
    def __init__(self, mode, physical_port, port_base, mac_index, loopback):
        ConfiguredPort.__init__(self, mode, mac_index, loopback)
        self.port_base = port_base
        self.physical_port = physical_port
        self.ganged_ports = None

    @property
    def label(self):
        return self.physical_port.label

    @property
    def oversub(self):
        return self.physical_port.oversub

    @property
    def speed(self):
        return self.physical_port.speed(self.mode)

    @property
    def num_lanes(self):
        return self.physical_port.num_lanes(self.mode)

    @property
    def num_logical_ports(self):
        return self.physical_port.num_logical_ports(self.mode)

    @property
    def max_logical_ports(self):
        return self.physical_port.max_logical_ports

    @property
    def num_physical_ports(self):
        return self.physical_port.num_physical_ports(self.mode)

    @property
    def num_phys(self):
        return self.physical_port.num_phys

    @property
    def enable_auto_neg(self):
        if self.physical_port.autoneg_pref is not None:
            return self.physical_port.autoneg_pref
        else:
            return self.physical_port.phy_layer in ('10GBaseT', '1GBaseT')

    def hw_intf_num(self, sub):
        '''
        Return the hardware interface number for the given sub-interface (The
        number in hg0).  Never mapped.
        '''
        return int(self.physical_port.hw_intf_num(self.mode, sub))

    def base_hw_intf_num(self, sub):
        '''
        Return the lowest hardware interface number for the given sub-interface
        Never mapped.
        '''
        return int(self.physical_port.base_hw_intf_num(self.mode, sub))

    def sdk_intf_prefix(self, sub):
        '''
        Return the interface prefix for the given sub-interface (xe,
        ge, etc...).  This is the interface name after configuration (hg may
        become xe here).
        '''
        # assume 'xe' ports by default
        prefix = self.physical_port.hw_intf_prefix(self.mode, sub)[:2]
        if self.physical_port.disable_prefix_xlate is True:
            return prefix
        else:
            if prefix == 'hg':
                return 'xe'
            else:
                return prefix

    def sdk_intf_num(self, sub):
        '''
        Return the interface number for the given sub-interface (the
        number in "xe0")
        '''
        return self.hw_intf_num(sub)

    def sdk_unit_intf_num(self, sub):
        return self.sdk_intf_num(sub)

    def sdk_intf(self, sub):
        '''
        Return the full name for the sub-interface (xe0, ge0, etc...)
        '''
        return '%s%u' % (self.sdk_intf_prefix(sub), self.sdk_intf_num(sub))

    def linux_port_name(self, sub):
        '''
        Return the linux port (interface) name
        '''
        portname = self.physical_port.name_override
        if portname is None:
            portname = '%s%s' % (self.linux_prefix, self.gang_label)
            if self.num_logical_ports > 1:
                portname = '%s%ss%u' % (self.linux_prefix, self.gang_label, sub)
        return portname

    def sdk_pbmp(self, sub):
        '''
        Return the port bitmap mask for the sub-interface.
        '''
        return self.physical_port.hw_pbmp(self.mode, sub)

    def phy_addr(self, sub):
        return self.physical_port.phy_addrs[sub]

    def sdk_logical_port_num(self, sub):
        return self.port_base + sub

    @property
    def prim_offset(self):
        return self.physical_port.prim_offset

    @property
    def phy_id1(self):
        return self.physical_port.phy_id1

    @property
    def sys_interface(self):
        return self.physical_port.sys_interface

    def phy_type(self, sub):
        if self.physical_port.phy_types is not None:
            return self.physical_port.phy_types[sub]
        else:
            return None

    def phy_autodetect(self, sub):
        return self.physical_port.phy_autodetect(self.mode, sub)

    def set_leds(self, led_port, sub, led_index_list):
        return self.physical_port.set_leds(self.mode, led_port, sub, led_index_list)

    def hw_port_num(self, sub):
        return self.physical_port.hw_port_num(self.mode, sub)

    def port_mac_index(self, sub):
        return self.physical_port.port_mac_index(self.mode, self.mac_index, sub)

    @property
    def leds(self):
        return self.physical_port.leds

    @property
    def unit(self):
        return self.physical_port.unit

    @property
    def rx_polarity_flip(self):
        return self.physical_port.rx_polarity_flip

    @property
    def tx_polarity_flip(self):
        return self.physical_port.tx_polarity_flip

    def rx_logical_polarity_flip(self, sub):
        if self.gang_leader is True:
            polarity_flip = 0
            bit_pos = 0
            # Sort the ports by serdes number as an integer.  For each
            # port(lane) in the gang group, gather the polarity flip
            # bits into a bitmap.
            for p in sorted(self.ganged_ports, key=lambda x: int(x.physical_port.hw_ports[0][2:])):
                bit = p.physical_port.rx_logical_polarity_flip(self.mode, sub)
                polarity_flip = polarity_flip | (bit << bit_pos)
                bit_pos = bit_pos + 1
            return polarity_flip
        else:
            return self.physical_port.rx_logical_polarity_flip(self.mode, sub)

    def tx_logical_polarity_flip(self, sub):
        if self.gang_leader is True:
            polarity_flip = 0
            bit_pos = 0
            # Sort the ports by serdes number as an integer.  For each
            # port(lane) in the gang group, gather the polarity flip
            # bits into a bitmap.
            for p in sorted(self.ganged_ports, key=lambda x: int(x.physical_port.hw_ports[0][2:])):
                bit = p.physical_port.tx_logical_polarity_flip(self.mode, sub)
                polarity_flip = polarity_flip | (bit << bit_pos)
                bit_pos = bit_pos + 1
            return polarity_flip
        else:
            return self.physical_port.tx_logical_polarity_flip(self.mode, sub)

    def td3_rx_logical_polarity_flip(self, port, portmap, sub):
        hw_intf = port.hw_intf_num(sub) + 1
        rx_flip = port.rx_logical_polarity_flip(sub)
        flip_num = { '10G'   : 1,
                     '25G'   : 1,
                     '4x10G' : 1,
                     '4x25G' : 1,
                     '2x50G' : 2,
                     '50G'   : 2,
                     '40G'   : 4,
                     '100G'  : 4 }
        for i in range(flip_num[port.mode]):
            portmap.append('phy_chain_rx_polarity_flip_physical{%u.0}=%s' %
                           (hw_intf + i, hex((rx_flip >> i) & 0x1)))
        return portmap

    def td3_tx_logical_polarity_flip(self, port, portmap, sub):
        hw_intf = port.hw_intf_num(sub) + 1
        tx_flip = port.tx_logical_polarity_flip(sub)
        flip_num = { '10G'   : 1,
                     '25G'   : 1,
                     '4x10G' : 1,
                     '4x25G' : 1,
                     '2x50G' : 2,
                     '50G'   : 2,
                     '40G'   : 4,
                     '100G'  : 4 }
        for i in range(flip_num[port.mode]):
            portmap.append('phy_chain_tx_polarity_flip_physical{%u.0}=%s' %
                           (hw_intf + i, hex((tx_flip >> i) & 0x1)))
        return portmap

    @property
    def is_fabric(self):
        return self.physical_port.is_fabric

    @property
    def create_fabric(self):
        return self.physical_port.create_fabric

    @property
    def rx_lane_map(self):
        return self.physical_port.rx_lane_map

    @property
    def tx_lane_map(self):
        return self.physical_port.tx_lane_map

    @property
    def mdi_pair_map(self):
        return self.physical_port.mdi_pair_map

    @property
    def pre_emphasis(self):
        return self.physical_port.pre_emphasis

    @property
    def drv_current(self):
        return self.physical_port.drv_current

    @property
    def pre_drv_current(self):
        return self.physical_port.pre_drv_current

    @property
    def phy_init(self):
        return self.physical_port.phy_init

    @property
    def serdes_if_type(self):
        return self.physical_port.serdes_if_type(self.speed)

    @property
    def ganged(self):
        return self.mode in self.physical_port.gang_modes

    @property
    def max_gang_size(self):
        return self.physical_port.max_gang_size

    @property
    def gang_leader(self):
        # The primary port in the set of ganged ports has a non-zero
        # number of logical ports.
        return self.ganged and (self.num_logical_ports != 0)

    @property
    def an_c37(self):
        return self.physical_port.an_c37(self.mode)

    @property
    def an_c73(self):
        return self.physical_port.an_c73(self.mode)

    def sdk_config_intf(self, sub, num_units=None):
        if self.portmap_capable:
            label = str(self.sdk_logical_port_num(sub))
        else:
            label = str(self.sdk_intf(sub))

        if num_units is None or num_units > 1:
            label += '.%u' % self.unit

        return label

    @property
    def set_fiber_pref(self):
        return self.physical_port.set_fiber_pref

    @property
    def sdk_group(self):
        return self.physical_port.sdk_group

class BCMConfiguredPort(SDKConfiguredPort):
    '''
    Broadcom port configuration implementation.
    '''
    pass

class SDKConfiguredMappedPort(SDKConfiguredPort):
    portmap_capable = True

    '''
    A port remapped using portmap_ directives
    '''
    def sdk_pbmp(self, sub):
        '''
        Mapped ports are sequential based on the port_base
        '''
        return 1 << (self.port_base + sub)

    def sdk_intf_num(self, sub):
        '''
        Return the mapped SDK interface number for the sub-interface.
        '''
        return (self.port_base + sub) - 1

class BCMConfiguredMappedPort(SDKConfiguredMappedPort):
    '''
    Broadcom port remapped implementation.
    '''
    def __init__(self, mode, physical_port, port_base, mac_index):
        SDKConfiguredMappedPort.__init__(self, mode, physical_port, port_base, mac_index)

class BCMConfiguredDportPort(BCMConfiguredPort):
    portmap_capable = True

    '''
    A port remapped using portmap and dport_ directives
    '''
    def __init__(self, mode, physical_port, port_base, if_port_base, mac_index, loopback):
        BCMConfiguredPort.__init__(self, mode, physical_port, port_base, mac_index, loopback)
        self.if_port_base = if_port_base

    def sdk_pbmp(self, sub):
        return 1 << (self.sdk_logical_port_num(sub))

    def sdk_intf_num(self, sub):
        '''
        Return the mapped BCM interface number for the sub-interface.
        # sequence number per unit, per if type
        '''
        return (self.if_port_base + sub) - 1

    def sdk_unit_intf_num(self, sub):
        # sequence number per unit
        return (self.port_base + sub) - 1

    def sdk_logical_port_num(self, sub):
        '''
        Return the Dport number for specific physical port
        :param sub:
        :return:
        '''
        return self.physical_port.logical_port_num(self.mode, sub, self.port_base)


class SDKConfig(Config):
    '''
    SDK configuration implementation
    '''

    # Interface types taken from bcm-sdk/include/shared/port.h
    serdes_if_type_map = {'XGMII':6, 'SFI':9, 'XFI':10, 'KR4':12, 'SR4':28, 'KR':11}

    def __init__(self, platform, verbose=False, exploded=False, dummy=False):
        Config.__init__(self, platform, verbose, exploded, dummy=dummy)
        self.read_config('uft.json')
        self.read_config('riot.json')
        self.read_config('hw_desc')
        self.read_config('rc.forwarding')
        self.read_config('datapath.conf')
        self.read_config('sdk.bcm')

    def config(self, config):
        Config.config(self, config)
        max_mac_index = self._calculate_configuration()
        if self.verbose:
            sys.stdout.write('max_mac_index %d macrange %d\n' % (max_mac_index, self.macrange))
        self._calculate_leds()
        self._calculate_units()
        self._calculate_gangs(self.ports)
        if max_mac_index > self.macrange :
            # default to l3 only mode as we do not have enough mac's
            syslog.syslog('Not enough macs in platform, falling to l3-only')
            self.set_macmode('l3-only')

    def _calculate_configuration(self):
        '''
        Calculate port mappings.  Derives internal state, but does not implment
        changes in hardware or configuration files.
        '''
        self.ports = []

        # create config objects for every port
        mac_index = 0
        self.mapped_portnums = {}
        if_nums = {}
        hw_ports = []

        for port in self.platform.switch.ports:
            label = "Fabric-" + port.label if port.is_fabric else port.label

            configured_mode = self.config_map.get(label, port.default_mode)
            if configured_mode == 'loopback':
                loopback = True
                mode = port.default_mode
            else:
                loopback = False
                mode = configured_mode

            # user might have marked a port 'disable' or commented it out altogether.
            # user might have marked a port 'disable' or commented it out altogether.
            if port.configurable(self.config_map):
                for hw_port, mode in port.port_mangler(mode):
                    hw_ports.append((loopback, mode, hw_port))

        for (loopback, mode, hw_port) in hw_ports:
            mapped_portnum = self.mapped_portnums.get(hw_port.unit, 1)
            u_map = if_nums.get(hw_port.unit)
            if not u_map:
                u_map = {}
                if_nums[hw_port.unit] = u_map
            if_type = hw_port.hw_intf_prefix(mode, 0)
            if_portnum = u_map.get(if_type, 1)
            if self.platform.switch.chip.dport_capable is True:
                cp = BCMConfiguredDportPort(mode, hw_port, mapped_portnum, if_portnum, mac_index, loopback)
                if_portnum += cp.num_logical_ports
                u_map[if_type] = if_portnum
            elif self.platform.switch.chip.portmap_capable is True:
                cp = SDKConfiguredMappedPort(mode, hw_port, mapped_portnum, mac_index, loopback)
            else:
                cp = SDKConfiguredPort(mode, hw_port, mapped_portnum, mac_index, loopback)
            mapped_portnum += cp.num_logical_ports
            self.ports.append(cp)
            self.mapped_portnums[hw_port.unit] = mapped_portnum
            # when summing up port address contributions, all ports are queried
            # (not just configurable ports) to allow platform specific control
            # due to port blocking/disablement.
            mac_index += hw_port.mac_addrs_contrib_total(self.config_map)
        return (mac_index)

    def _calculate_leds(self):
        '''
        Calculate led programs for switch chip controlled LEDs
        '''
        self._led_programs = {}

        if self.platform.switch.led_controllers:
            controllers = self.platform.switch.led_controllers
        else:
            controllers = set([ x.leds[0] for x in self.ports if x.leds is not None ])

        # Create an LED program for each controller in the platform
        for controller in sorted(controllers):
            try:
                ledup = self.platform.switch.ledup[controller]()
            except TypeError:
                ledup = self.platform.switch.ledup()

            ports = [ x for x in self.ports if (x.leds is not None and x.leds[0] == controller) ]
            ledup.configure(controller, ports)

            self._led_programs[controller] = ledup

    def _calculate_gangs(self, ports):
        '''
        Calculate port gang configurations
        '''
        gang_leader = None
        ganged_ports = set()

        for port in ports:
            if port.ganged is True:
                if port.gang_leader is True:
                    gang_leader = port
                # gather ganged ports in the gang group
                if port in ganged_ports:
                    raise RuntimeError('platform: %s: Duplicate port %s already in ganged set'
                                       % (self.platform.name, port.label))
                else:
                    ganged_ports.add(port)
                if len(ganged_ports) == port.max_gang_size:
                    # Found all the ports in the gang. Save the ganged
                    # ports with the gang leader port for later
                    # processing.
                    gang_leader.ganged_ports = frozenset(ganged_ports)
                    # Reset for next gang group
                    gang_leader = None
                    ganged_ports = set()

    def _calculate_units(self):
        '''
        Calculate number of units which exists per platform
        '''
        self.units = set([ x.unit for x in self.ports if x.unit is not None ])

    @property
    def led_chains(self):
        return self._led_programs.keys()

    @property
    def led_asm(self):
        return dict([ (x, self._led_programs[x].asm) for x in self._led_programs.keys() ])

    @property
    def led_hex(self):
        return dict([ (x, self._led_programs[x].hex) for x in self._led_programs.keys() ])

    @property
    def port_units(self):
        return self.mapped_portnums.keys()

    @property
    def sdk_pbmp(self):
        pbmp = []

        pbmps = {}
        valid = {}
        oversub = {}
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                pref = port.sdk_intf_prefix(sub)
                # ce ports actually use xe as their prefix
                if pref == 'ce':
                    pref = 'xe'
                prefix = '%s.%u' % (pref, port.unit)
                bits = pbmps.get(prefix, 0) | port.sdk_pbmp(sub)
                pbmps[prefix] = bits
                valid[port.unit] = valid.get(port.unit,
                        self.platform.switch.chip.base_pbmp) | bits
                if port.oversub:
                    oversub[port.unit] = oversub.get(port.unit, 0) | bits

        for intf in pbmps.keys():
            if not intf.startswith('hg'):
                pbmp.append('pbmp_xport_%s=0x%x' % (intf, pbmps[intf]))

        for unit in valid.keys():
            pbmp.append('pbmp_valid.%u=0x%x' % (unit, valid[unit]))

        if len(oversub):
            if self.platform.switch.chip.mixed_oversub:
                for unit in valid.keys():
                    if unit in oversub and self.platform.switch.chip.mixed_oversub:
                        pbmp.append('pbmp_oversubscribe.%u=0x%x' % (unit, oversub[unit]))
            else:
                # The ASIC can't handle per-port oversubscription.
                # Oversubscribe them all.
                pbmp.append('oversubscribe_mode=1')

        return '\n'.join(pbmp)

    @property
    def portmap(self):
        if self.platform.switch.chip.portmap_capable is not True:
            raise RuntimeError('platform: %s is not portmap capable' % self.platform.name)

        portmap = []
        chip = self.platform.switch.chip

        for port in self.ports:
            for sub in range(port.num_logical_ports):
                config_intf = port.sdk_config_intf(sub)
                portmap.append('portmap_%s=%u:%u' % (config_intf, port.hw_intf_num(sub) + 1,
                                                     port.speed))
                if self.platform.switch.chip.needs_lane_config:
                    portmap[-1] += ':%u' % port.num_lanes
                if port.rx_polarity_flip is not None:
                    if (isinstance(chip, cumulus.platform.TridentThreeX7_56870_Chip) or \
                        isinstance(chip, cumulus.platform.TridentThreeX7_56873_Chip)):
                        portmap = port.td3_rx_logical_polarity_flip(port, portmap, sub)
                    else:
                        portmap.append('phy_xaui_rx_polarity_flip_%s=0x%04x' %
                                       (config_intf, port.rx_logical_polarity_flip(sub)))
                if port.tx_polarity_flip is not None:
                    if (isinstance(chip, cumulus.platform.TridentThreeX7_56870_Chip) or \
                        isinstance(chip, cumulus.platform.TridentThreeX7_56873_Chip)):
                        portmap = port.td3_tx_logical_polarity_flip(port, portmap, sub)
                    else:
                        portmap.append('phy_xaui_tx_polarity_flip_%s=0x%04x' %
                                       (config_intf, port.tx_logical_polarity_flip(sub)))
                if port.rx_lane_map is not None:
                    if (isinstance(chip, cumulus.platform.TridentThreeX7_56870_Chip) or \
                        isinstance(chip, cumulus.platform.TridentThreeX7_56873_Chip)):
                        portmap.append('phy_chain_rx_lane_map_physical{%u.0}=0x%04x' %
                                       (port.hw_intf_num(sub) + 1, port.rx_lane_map))
                    else:
                        portmap.append('xgxs_rx_lane_map_%s=0x%04x' %
                                       (config_intf, port.rx_lane_map))
                if port.tx_lane_map is not None:
                    if (isinstance(chip, cumulus.platform.TridentThreeX7_56870_Chip) or \
                        isinstance(chip, cumulus.platform.TridentThreeX7_56873_Chip)):
                        portmap.append('phy_chain_tx_lane_map_physical{%u.0}=0x%04x' %
                                       (port.hw_intf_num(sub) + 1, port.tx_lane_map))
                    else:
                        portmap.append('xgxs_tx_lane_map_%s=0x%04x' %
                                       (config_intf, port.tx_lane_map))
                if port.mdi_pair_map is not None:
                    portmap.append('phy_mdi_pair_map_%s=0x%04x' %
                                   (config_intf, port.mdi_pair_map))

        return '\n'.join(portmap)

    @property
    def dportmap(self):
        if self.platform.switch.chip.dport_capable is not True:
            raise RuntimeError('platform %s is not dportmap capable' % self.platform.name)

        portmap = [
            'dport_map_enable=1\n'
            ]

        for port in self.ports:
            for sub in range(port.num_logical_ports):
                portmap.append('dport_map_port_%u.%u=%u' % (port.sdk_logical_port_num(sub), port.unit, port.sdk_unit_intf_num(sub)+1))
        return '\n'.join(portmap)

    @property
    def portmap_uncapable_config(self):
        if self.platform.switch.chip.portmap_capable is True:
            raise RuntimeError('platform: %s is portmap capable' % self.platform.name)

        portmap = []
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                if port.rx_polarity_flip is not None:
                    portmap.append('phy_xaui_rx_polarity_flip_%d=0x%04x' %
                                   (port.sdk_logical_port_num(sub), port.rx_logical_polarity_flip(sub)))
                if port.tx_polarity_flip is not None:
                    portmap.append('phy_xaui_tx_polarity_flip_%d=0x%04x' %
                                   (port.sdk_logical_port_num(sub), port.tx_logical_polarity_flip(sub)))
                if port.rx_lane_map is not None:
                    portmap.append('xgxs_rx_lane_map_%d=0x%04x' %
                                   (port.sdk_logical_port_num(sub), port.rx_lane_map))
                if port.tx_lane_map is not None:
                    portmap.append('xgxs_tx_lane_map_%d=0x%04x' %
                                   (port.sdk_logical_port_num(sub), port.tx_lane_map))
                if port.mdi_pair_map is not None:
                    portmap.append('phy_mdi_pair_map_%d=0x%04x' %
                                   (port.sdk_logical_port_num(sub), port.mdi_pair_map))

        return '\n'.join(portmap)

    @property
    def port_emphasis(self):
        port_emp = []
        map_nums = {}
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                map_num = map_nums.get(port.unit, 1)
                #for 40G we need to set emphasis for all 4 lanes, for 10G we
                #set just one value for the entire port which is mapped by
                #the sdk to the associated lane.
                if port.mode == '40G':
                    lane_num = 4
                else:
                    lane_num = 1
                if port.pre_emphasis is not None:
                    if lane_num > 1:
                        for lane in range(lane_num):
                            port_emp.append(
                                'serdes_preemphasis_lane%1d_%u.%u=0x%04x' %
                                (lane, map_num, port.unit,
                                port.pre_emphasis[lane]))
                    else:
                        port_emp.append(
                            'serdes_preemphasis_%u.%u=0x%04x' %
                            (map_num, port.unit, port.pre_emphasis[sub]))

                if port.drv_current is not None:
                    if lane_num > 1:
                        for lane in range(lane_num):
                            port_emp.append(
                                'serdes_driver_current_lane%1d_%u.%u=0x%x' %
                                (lane, map_num, port.unit,
                                port.drv_current[lane]))
                    else:
                        port_emp.append(
                            'serdes_driver_current_%u.%u=0x%04x' %
                            (map_num, port.unit, port.drv_current[sub]))

                if port.pre_drv_current is not None:
                    if lane_num > 1:
                        for lane in range(lane_num):
                            port_emp.append(
                                'serdes_pre_driver_current_lane%1d_%u.%u=0x%x' %
                                (lane, map_num, port.unit,
                                port.pre_drv_current[lane]))
                    else:
                        port_emp.append(
                            'serdes_pre_driver_current_%u.%u=0x%04x' %
                            (map_num, port.unit, port.pre_drv_current[sub]))

                map_nums[port.unit] = map_num + 1
        return '\n'.join(port_emp)

    @property
    def port_init(self):
        port_init = []
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                config_intf = port.sdk_config_intf(sub, len(self.units))

                port_init.append('port_init_autoneg_%s=%u' % (config_intf, port.enable_auto_neg))
                if not port.enable_auto_neg:
                    port_init.append('port_init_speed_%s=%s000' % (config_intf, port.speed))
                    port_init.append('port_init_duplex_%s=1' % config_intf)

                if port.serdes_if_type is not None:
                    ifnum = SDKConfig.serdes_if_type_map[port.serdes_if_type]
                    ifname = port.serdes_if_type
                    port_init.append('serdes_if_type_%s=%u # %s' % (config_intf, ifnum, ifname))
        return '\n'.join(port_init)

    @property
    def fiber_pref(self):
        fiber_pref = []
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                if port.set_fiber_pref:
                    config_intf = port.sdk_config_intf(sub, len(self.units))
                    fiber_pref.append('serdes_fiber_pref_%s=1' % config_intf)
        return '\n'.join(fiber_pref)

    @property
    def phys(self):
        phys = []

        for port in self.ports:
            for sub in range(port.num_logical_ports):
                if port.phy_type(sub) is not None and not port.phy_autodetect(sub):
                    config_intf = port.sdk_config_intf(sub, len(self.units))
                    phys.append('phy_%s_%s=1' % (port.phy_type(sub), config_intf))

        return ('\n').join(phys)

    @property
    def phy_addrs(self):
        addrs = []

        for port in self.ports:
            for sub in range(port.num_logical_ports):
                if port.phy_type(sub) is not None:
                    config_intf = port.sdk_config_intf(sub, len(self.units))
                    addrs.append('port_phy_addr_%s=0x%x' %
                                 (config_intf, port.phy_addr(sub)))
        return ('\n').join(addrs)

    @property
    def phy_an(self):
        config_map = { 'off' : 0,
                       'bam' : 1,
                       'on'  : 2,
                       'auto': 3, }
        an = []

        for port in self.ports:
            c37 = config_map.get(port.an_c37, None)
            c73 = config_map.get(port.an_c73, None)

            for sub in range(port.num_logical_ports):
                config_intf = port.sdk_config_intf(sub, len(self.units))
                if c37 is not None:
                    an.append('phy_an_c37_%s=%d' % (config_intf, c37))

                if c73 is not None:
                    an.append('phy_an_c73_%s=%d' % (config_intf, c73))

        return ('\n').join(an)

    @property
    def prim_offsets(self):
        prim_offsets = []

        for port in self.ports:
            if port.prim_offset is not None:
                config_intf = port.sdk_config_intf(0, len(self.units))
                prim_offsets.append('phy_port_primary_and_offset_%s=0x%x' %
                                    (config_intf, port.prim_offset))

        return ('\n').join(prim_offsets)

    @property
    def phy_id1s(self):
        phy_id1s = []

        for port in self.ports:
            if port.phy_id1 is not None:
                phy_id1s.append('port_phy_id1_%d=0x%x' %
                                    (port.sdk_logical_port_num(0), port.phy_id1))


        return ('\n').join(phy_id1s)

    @property
    def sys_interfaces(self):
        sys_interfaces = []

        for port in self.ports:
            if port.sys_interface is not None:
                config_intf = port.sdk_config_intf(0)
                sys_interfaces.append('phy_sys_interface_%s=0x%x' %
                                      (config_intf, port.sys_interface))

        return ('\n').join(sys_interfaces)

    @property
    def phytab(self):
        phytab = []

        (phy_addr, linux_intf, sdk_intf) = (None, None, None)
        for port in self.ports:
            for phy in range(port.num_phys):
                phy_addr = port.phy_addr(phy)
                if port.num_logical_ports > phy:
                    sdk_intf = port.sdk_intf(phy)
                    linux_intf = port.linux_port_name(phy)
                phytab.append('0x%x\t%s\t%s\t%s' %
                              (phy_addr, linux_intf, sdk_intf, port.mode))

        return ('\n').join(phytab)

    def _compress_portlist(self, portlist):
        # The SDK has a limit on the size of setreg commands and/or port
        # range specifiers.  This code compresses ranges of sequential ports so
        # they take up less space.  For example, "xe0,xe1,xe2" becomes "xe0-xe2"
        key = lambda x: '%s%03u' % (x[:2], int(x[2:]))
        grouper = lambda (i, intf): '%s%u' % (intf[:2], i - int(intf[2:]))
        spans = [ list(y) for x,y in groupby(enumerate(sorted(portlist, key=key)), grouper) ]

        compressed = []
        for span in spans:
            if len(span) > 2:
                compressed.append('%s-%s' % (span[0][1], span[-1][1]))
            else:
                compressed += [ x[1] for x in span ]

        return ','.join(compressed)

    def rcports(self, unit):
        rcports = []

        allports = []
        speeds = {}
        lbports = []
        sdk_groups = {}
        for port in self.ports:
            if port.unit == unit:
                for sub in range(port.num_logical_ports):
                    allports.append(port.sdk_intf(sub))

                    if port.speed not in speeds:
                        speeds[port.speed] = list()
                    speeds[port.speed].append(port.sdk_intf(sub))

                    if port.loopback:
                        lbports.append(port.sdk_intf(sub))

                    if port.sdk_group is not None:
                        sdk_groups.setdefault(port.sdk_group, []).append(port.sdk_intf(sub))

        rcports.append('setenv allports %s' % self._compress_portlist(allports))
        for speed in speeds:
            if speed == 1:
                rcports.append('setenv 1Gports %s' % self._compress_portlist(speeds[1]))
            else:
                rcports.append('setenv %sGports %s' %
                               (speed, self._compress_portlist(speeds[speed])))

        if len(lbports):
            rcports.append('setenv lbports %s' % self._compress_portlist(lbports))
            rcports.append('port $lbports lb=mac')

        for sdk_group in sdk_groups:
            rcports.append('setenv sdk_group_%s %s' %
                           (sdk_group, self._compress_portlist(sdk_groups[sdk_group])))

        return '\n'.join(rcports)

    @property
    def porttab(self):
        porttab = []
        interfaces = set([])
        netdevs = []
        use_tuntap = False
        platform = self.platform.switch.chip.sw_base

        if os.environ.get('SWPORT_USE_TUNTAP') == '1':
            use_tuntap = True

        if use_tuntap:
           # first build list of all existing port.linux_prefix's
           cmd = 'tuntap show'
           out = self.iproute2.run(cmd.split())
           for interface in out.splitlines():
               taps = interface.split(':')
               if taps[0]:
                   interfaces.add(taps[0])
        elif platform != 'vx':
           if platform not in ['bcm', 'mlx']:
               raise UnsupportedPlatformError("Kernel netdevs are not supported \
                     on %s, only on Broadcom and Mellanox" % platform)
           link_type = 'bcm_knet' if platform == 'bcm' else 'mlx_sx_netdev'
           cmd = 'show type ' + link_type
           out = self.swport.run(cmd.split())
           for interface in out.splitlines():
               ifs = interface.split(' ')
               if ifs[0] and ifs[0] not in ['switch', 'swid']:
                   interfaces.add(ifs[0])

        # create a list of the needed netdevs based on config files and
        # create tap devices if not bcm
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                portname = port.linux_port_name(sub)
                lp_num = -1
                if self.platform.switch.chip.dport_capable:
                    lp_num = port.sdk_logical_port_num(sub)
                porttab.append('%s\t%s\t%u\t%i\t%u\t%i' %
                               (portname, port.sdk_intf(sub),
                                port.unit, port.is_fabric, lp_num, port.loopback))
                if port.is_fabric is False or port.create_fabric:
                    port_mac_idx = port.port_mac_index(sub)
                    if use_tuntap:
                       self.maketap(portname, port_mac_idx, interfaces)
                    else:
                       self.makekernnet(portname, port_mac_idx, port.unit, interfaces)

        # if bcm make sure to set switchN 'up'
        if self.platform.switch.chip.sw_base == 'bcm' and not use_tuntap:
           cmd = '-o link show'
           out = self.iproute2.run(cmd.split())
           for interface in out.splitlines():
               switches = interface.split(':')
               if len(switches) > 1 and 'switch' in switches[1]:
                   cmd = 'link set %s up' % switches[1]
                   self.iproute2.run(cmd.split())

        self.destroyints(interfaces)
        del interfaces

        return ('\n').join(porttab)

    @property
    def rcled(self):
        rcled = []
        for led in self._led_programs.keys():
          rcled.append('led %d stop\n'
                       'led %d auto on\n'
                       'led %d load /etc/bcm.d/led%d.hex\n'
                       'led %d start\n' % (led, led, led, led, led))
          rcled.extend(self._led_programs[led].led_memory_init_commands(led))
        return '\n'.join(rcled)

    @property
    def rcphy(self):
        rcphy = []

        for port in self.ports:
            for phy in range(port.num_phys):
                phy_addr = port.phy_addr(phy)
                if port.num_logical_ports > phy:
                    for reg in port.phy_init.keys():
                        rcphy.append('phy raw c45 %s %s %s' %
                                     (phy_addr, reg, port.phy_init[reg]))

        return ('\n').join(rcphy)

    @property
    def ethertab(self):
        ethertab = []

        # now create the porttab entries and swp devices
        for port in self.ports:
            if port.num_logical_ports > 1:
                for sub in range(port.num_logical_ports):
                    portname = port.linux_port_name(sub)
                    ethertab.append('%s\t%s' % (portname, port.mac_index + sub))
            elif port.num_logical_ports > 0:
                portname = port.linux_port_name(0)
                ethertab.append('%s\t%s' % (portname, port.mac_index))

        return ('\n').join(ethertab)

    @property
    def linkdefaults(self):
        # Create a JSON file representing the initial an/speed/duplex settings
        # for each port.  ifupdown2 will take this and use it as the default
        # ethtool configuration for each port.
        defaults = { 'README' : 'This file is automatically generated. Do not edit this file.',
                     'ethtool' : { 'defaults' : {}, 'iface_defaults' : {} } }

        if self.platform.switch.chip.sw_base == 'vx':
            defaults['ethtool'].update(self._linkdefaults_vx())
        else:
            defaults['ethtool'].update(self._linkdefaults())

        return json.dumps(defaults, indent=4, sort_keys=True)

    def _linkdefaults_vx(self):
        return { 'defaults' : { "link-autoneg" : "off",
                                "link-duplex" : "full",
                                "link-speed" : "1000" } }

    def _linkdefaults(self):
        iface_defaults = {}
        for port in self.ports:
            for sub in range(port.num_logical_ports):
                # skip fabric ports, ifupdown2 doesn't deal with them.
                if port.physical_port.is_fabric:
                    continue

                portname = port.linux_port_name(sub)
                if port.enable_auto_neg:
                    an = 'on'
                else:
                    an = 'off'

                iface_defaults[portname] = { 'link-autoneg' : an,
                                             'link-duplex' : 'full',
                                             'link-speed' : '%s000' % port.speed }

        return { 'defaults' : {}, 'iface_defaults' : iface_defaults }

    @property
    def sfptab(self):
        sfptab = []
        linux_intf = None
        for port in self.ports:
            psfp = port.physical_port.sfp
            if not psfp:
                continue
            if psfp.accessmethod.find("sdk") > 0:
                sfpaddress = '0x%x' %psfp.address
            else:
                sfpaddress = '%s' %psfp.address
            if port.num_logical_ports > 1:
                for sub in range(port.num_logical_ports):

                    linux_intf = port.linux_port_name(sub)
                    sfptab.append('%s\t%s\t%s\t%s' %
                                (linux_intf, psfp.type, psfp.accessmethod,
                                sfpaddress))
            else:
                linux_intf = port.linux_port_name(0)
                sfptab.append('%s\t%s\t%s\t%s' %
                            (linux_intf, psfp.type, psfp.accessmethod,
                            sfpaddress))
        return ('\n').join(sfptab)

    def output(self, configfile):
        if configfile == 'portsbcm':
            out = [
                '\n# pbmp',
                self.sdk_pbmp,
            ]

            if self.platform.switch.chip.dport_capable is True:
                out += [
                    '\n# dport map',
                    self.dportmap,
                ]
            if self.platform.switch.chip.portmap_capable is True:
                out += [
                    '\n# portmap',
                    self.portmap,
                ]
            else:
                out += [
                    '\n# non portmap capable config',
                    self.portmap_uncapable_config,
                ]

            out += [
                '\n# emphasis',
                self.port_emphasis,
                '\n# port initializations',
                self.port_init,
                '\n# phys',
                self.phys,
                '\n# addrs',
                self.phy_addrs,
                '\n# autoneg',
                self.phy_an,
                '\n# prime_offsets',
                self.prim_offsets,
                '\n# phy_id1s',
                self.phy_id1s,
                '\n# sys_interfaces',
                self.sys_interfaces,

            ]

            fiber_pref = self.fiber_pref
            if fiber_pref:
                out += [
                    '\n# fiber pref',
                    fiber_pref,
                ]

        elif configfile == 'porttab':
            out = [
                '# linux_intf\tsdk_intf\tunit\tis_fabric\tlogical_port\tis_loopback',
                self.porttab,
            ]
        elif configfile == 'linkdefaults':
            out = [
                self.linkdefaults,
            ]
        elif configfile == 'phytab':
            out = [
                '# phy_addr\tlinux_intf\tsdk_intf',
                self.phytab,
            ]
        elif configfile.startswith('rcports_'):
            out = [
                self.rcports(int(configfile.rpartition('_')[2])),
            ]
        elif configfile == 'rcled':
            out = [
                self.rcled,
            ]
        elif configfile == 'rcphy':
            out = [
                self.rcphy,
            ]
        elif configfile == 'ethertab':
            out = [
                self.ethertab,
            ]
        elif configfile == 'ucodebcm':
            if self.exploded:
                if self.platform.switch.has_phy_ucode_mdio:
                    mode = "MDIO"
                    rom_boot = 0
                else:
                    mode = "SPI-ROM"
                    rom_boot = 3000
            else:
                if self.platform.switch.has_phy_ucode_spi_rom:
                    mode = "SPI-ROM"
                    rom_boot = 3000
                else:
                    mode = "MDIO"
                    rom_boot = 0

            out = [
                '# use %s method for ucode download' % (mode),
                'phy_ext_rom_boot=%d' % (rom_boot)
            ]
        elif configfile == 'sfptab':
            out = [
                '# linux_intf\tmode\tsfp_addrtype\tsfp_addr',
                self.sfptab,
            ]

        elif configfile == 'hwdesc' :
            if self.hw_desc == None :
                return None
            out = [
                ' # chip description file',
                self.hw_desc,
                ]

        elif configfile == 'uft' :
            if self.uft == None :
                return None
            out = [
                self.uft,
                ]

        elif configfile == 'riot' :
            if self.riot == None :
                return None
            out = [
                self.riot,
                ]

        elif configfile == 'datapath' :
            if self.datapath_conf == None :
                return None
            out = [
                ' # chip datapath configuration file',
                self.datapath_conf,
                ]

        elif configfile == 'sdk' :
            if self.sdk_bcm == None :
                return None
            out = [
                self.sdk_bcm,
                ]

        elif configfile == 'rcforwarding' :
            if self.rc_forwarding == None :
                return None
            out = [
                ' # chip forwarding configuration file',
                self.rc_forwarding,
                ]

        return '\n'.join(out)

class BCMConfig(SDKConfig):
    '''
    Broadcom configuration implementation
    '''
    def __init__(self, platform, verbose=False, exploded=False, dummy=False):
        SDKConfig.__init__(self, platform, verbose, exploded, dummy=dummy)
